# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#                                   X-BONE
#
#                USC Information Sciences Institute (USC/ISI)
#                   Marina del Rey, California 90292, USA
#                          Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
#
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
#
# ----------------------------------------------------------------------------
#
#  $RCSfile: xb-DNS-config.pl,v $
#
# $Revision: 1.4 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:28 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Start PERL on any system where perl is in the path
# This is a modified version of the "start perl" script
# provided in the PERL man pages, which starts perl 
# on the remainder of the file regardless of whether it
# is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the
# perl path, rather than requiring it be hard-coded.
#
# setenv PERLDB_OPTS "N f=26"
#
# Must be set before perl starts!
#
# N 	: Nonstop (noninteractive)
# f=26	: frame=26 = 16 + 8 + 2,
#		2 = entry and exit,
#		!4 = don't print args to functions
#		8= enabled overloaded stringify and tied FETCH
#		16= print return values from subroutines
#
############################################################
# the following line finds perl and runs it, in sh, csh, or perl:
#
# note - '-T' (TAINT) switch is not included, because 'which' often
# returns a version of perl that isn't secure. don't worry about it.
#
eval '(exit $?0)' && eval 'PERLDB_OPTS="N f=26";export PERLDB_OPTS;PERL=`which perl5`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERLDB_OPTS "N f=26"; setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

############################################################
# PERL CODE STARTS HERE
############################################################

use strict;
use File::Copy 'cp';

#- Global variables -----------------------------------------------------------
  my $ndconf_path;		# path for named config files
  my $rsconf_path;		# path for resolv.conf
  my $zone_path;		# path for DNS zone files
  my $sample_dir = "named_conf_sample";
  my $cksum;
  my $os;			# type of OS
  my $bkup="ORIG";
  my $TESTING=0;		# 1=testing, 0=normal

#- set_path -------------------------------------------------------------------
#  - Set DNS related path (global variables) based on platform (`uname -sr`).
#------------------------------------------------------------------------------
sub set_path()
{
  chop ($os = `uname -s`);
  if($os =~ /Linux/) {
    $ndconf_path = "/etc";
    $zone_path = "/var/named/xbone";
    $rsconf_path = "/etc";
    $cksum = "sum";
  } elsif ($os =~ /FreeBSD/) {
    $ndconf_path = "/etc/namedb";
    $zone_path = "/etc/namedb/xbone";
    $rsconf_path = "/etc";
    $cksum = "md5";
  } else {
    print "!!! $os is currently unsupported by X-Bone. Please update your\n";
    print "    DNS configuration manually. Refer to INSTALL for details.\n";
    exit 0;
  }
}

#- resolv_conf ----------------------------------------------------------------
#  - Add X-Bone nameserver entry to resolv.conf.
#------------------------------------------------------------------------------
sub resolv_conf()
{
  my $rsfile = "resolv.conf";
  my ($line, $sample_line);
  my $server_IP;
  my $confirm;
  my $ns_exist = 0;

  set_path();
  if($TESTING==1) { $rsconf_path="./tmp"; }
  print "==> DNS Client setup [$rsconf_path/$rsfile]:\n";

  open RS_SAMPLE, "$sample_dir/$rsfile" or 
    die "open $sample_dir/$rsfile failed: $!";
  chop( $sample_line = <RS_SAMPLE> );
  if ($sample_line =~ /^nameserver[\s]*(\d+\.\d+\.\d+\.\d+)/) {
    $server_IP = $1;
  } else {
    print "!!! Unrecognized format, please update $rsfile manually.\n";
    exit 0;
    # Our sample resolv.conf has only one line.
  }
  close RS_SAMPLE or die "close $sample_dir/$rsfile failed: $!";

  if(! -f "$rsconf_path/$rsfile"){
    my $cmd = "touch $rsconf_path/$rsfile";
    system ($cmd) == 0 or die "$cmd failed: $!";
  } elsif (! -w "$rsconf_path/$rsfile") {
    print "!!! Cannot write $rsconf_path/$rsfile: Permission denied.\n    ".
        "Modify it manually before running X-Bone. See $sample_dir/$rsfile.\n";
    exit 0;
  }

  #_ backup resolv.conf & check if nameserver entry is already in _____________
  open RS, "$rsconf_path/$rsfile" or 
    die "open $rsconf_path/$rsfile failed: $!";
  $bkup = "ORIG"; 
  if( -f "$rsconf_path/$rsfile.$bkup"){ # if resolv.conf.ORIG exists, leave it
    $bkup = "TMP";
  }else{
    $bkup = "ORIG";
  }
  my $rs_temp = "$rsconf_path/$rsfile.$bkup";
  open RS_BK, "+>$rs_temp" or die "open $rs_temp failed: $!";
  while ($line = <RS>){
    if ($line =~ /^\s*nameserver\s+(\d+\.\d+\.\d+\.\d+)/i){
      if ($1 eq $server_IP) { $ns_exist = 1; }
    }
    print RS_BK $line;
  }
  print "    > old resolv.conf => $rsconf_path/$rsfile.$bkup\n";
  close RS or die "close $rsconf_path/$rsfile failed: $!";

  #_ add the nameserver if it's not in ________________________________________
  if($ns_exist) {
    print "    > The nameserver $1 is already in $rsfile. No action taken.\n";
  }else{
    open RS, ">$rsconf_path/$rsfile" or
      die "open $rsconf_path/$rsfile failed: $!";
    seek RS_BK, 0, 0;
    while($line=<RS_BK>){
      if ($line =~ /^#/) { print RS $line; } # skip beginning comments
      else {
        print "    > add entry: [${sample_line}]\n";
        print RS "### Added by X-Bone: (1 line) ###\n";
        print RS $sample_line."\n";
        print RS $line;
	last;
      }
    }
    while ($line = <RS_BK>){
      print RS $line;
    }
    close RS or die "close $rsconf_path/$rsfile failed: $!";
  }
  close RS_BK or die "close $rs_temp failed: $!";
  print "    < DNS Client setup complete.\n";
}

#- named_conf -----------------------------------------------------------------
#  - Create named conf files and backup old ones.
#------------------------------------------------------------------------------
sub named_conf()
{
  my $ndfile = "named.conf";
  my $cmd;
  my @zone_files = ("xbone.private", "xbone-reverse.private"); 

  set_path();

  if ($TESTING) {
    $ndconf_path = "tmp/namedb";
    $zone_path = "tmp/namedb/xbone";
  }

  print "==> DNS Server setup [named.conf & zone files]:\n";

  if ($os =~ /Linux/) {
    $cmd="perl5 -pi.bak -e 's/etc\\\/namedb/var\\\/named/' $sample_dir/$ndfile";
    system ($cmd) == 0 or die "$cmd failed: $!";
  }

  #_ copy/backup /etc/named.conf ______________________________________________
  if(! -d $ndconf_path) {
    print "    ! Warning: $ndconf_path not found. named might not exist.\n";
    $cmd = "mkdir -p $ndconf_path";
    system ($cmd) == 0 or die "$cmd failed: $!";
  }
  if (! -f "$ndconf_path/$ndfile"){
    $cmd = "touch $ndconf_path/$ndfile";
    system ($cmd) == 0 or die "$cmd failed: $!";
  }
  $bkup = "ORIG";
  if( -f "$ndconf_path/$ndfile.$bkup"){
    $bkup = "TMP";
  }else{
    $bkup = "ORIG";
  }
  rename "$ndconf_path/$ndfile", "$ndconf_path/$ndfile.$bkup" or die "$!";
  print "    > old named.conf => $ndconf_path/$ndfile.$bkup\n";
  cp("$sample_dir/$ndfile", "$ndconf_path/$ndfile");
  print "    > add $ndconf_path/named.conf\n";

  #_ copy/backup X-Bone zone files ____________________________________________
  if(! -d $zone_path){
    $cmd = "mkdir -p $zone_path";
    system ($cmd) == 0 or die "$cmd failed: $!";
  }
  for my $z (@zone_files){
    $bkup = "ORIG";
    if(-f "$zone_path/$z") {
      if(-f "$zone_path/$z.$bkup"){
	$bkup = "TMP";
      }else{
	$bkup = "ORIG";
      }
      rename "$zone_path/$z", "$zone_path/$z.$bkup" or die "$!";
      print "    > old $z => $zone_path/$z.$bkup\n";
    } 
    cp ("$sample_dir/xbone/$z", "$zone_path/$z") or die "$!";
    print "    > add $zone_path/$z\n";
  }
  print "    < DNS Server setup complete.\n";
  
}

#- get_type -------------------------------------------------------------------
sub get_type()
{
  do {
    print "==> XBone DNS setup. Enter setup type [c=clinet | s=server]:";
    chop (my $type = <STDIN>);
    if($type =~ /^\s*c/i){
      return "client";
      last;
    }elsif($type =~ /^\s*s/i){
      return "server";
    }
  } while(1);
}

#=============================================================================
# main
#=============================================================================

  #- check/get cmd-line parameter
  my $type;

  if ($#ARGV != 0){
    $type = get_type();
  }elsif ($ARGV[0] eq "client" or $ARGV[0] eq "server"){
    $type = $ARGV[0];
  }else{
    $type = get_type();
  }

  if ($type eq "client"){
    resolv_conf();
  }else{
    named_conf();
  }




