# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#                                   X-BONE
#
#                USC Information Sciences Institute (USC/ISI)
#                   Marina del Rey, California 90292, USA
#                          Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
#
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
#
# ----------------------------------------------------------------------------
#
#  $RCSfile: xb-Host-check.pl,v $
#
# $Revision: 1.6 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:28 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Start PERL on any system where perl is in the path
# This is a modified version of the "start perl" script
# provided in the PERL man pages, which starts perl 
# on the remainder of the file regardless of whether it
# is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the
# perl path, rather than requiring it be hard-coded.
#
# setenv PERLDB_OPTS "N f=26"
#
# Must be set before perl starts!
#
# N 	: Nonstop (noninteractive)
# f=26	: frame=26 = 16 + 8 + 2,
#		2 = entry and exit,
#		!4 = don't print args to functions
#		8= enabled overloaded stringify and tied FETCH
#		16= print return values from subroutines
#
############################################################
# the following line finds perl and runs it, in sh, csh, or perl:
#
# note - '-T' (TAINT) switch is not included, because 'which' often
# returns a version of perl that isn't secure. don't worry about it.
#
eval '(exit $?0)' && eval 'PERLDB_OPTS="N f=26";export PERLDB_OPTS;PERL=`which perl5`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERLDB_OPTS "N f=26"; setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

############################################################
# PERL CODE STARTS HERE
############################################################

use strict;
use sigtrap;

use Config;

my $new_kame_style;

#---------------------------------------------------------------------
# path_check(@)
#	Determine if the given commands are in the search PATH.
sub path_check (@)
{
  my (@commands) = @_;
  
  print "    - Verify PATH for the following commands:\n";
  print "      PATH = $ENV{PATH}\n";
  my $missing = 0;
  foreach my $c (@commands){
    my $cmd = "which -s $c";
    print "        ";
    print (system($cmd)? "[ ] " : "[v] ");
    print $c,"\n";
    if(system($cmd)) { $missing++;};
  }
  if($missing){
    print "    * Some commands are not in the PATH shown above. You need\n",
	  "      to edit xbone/lib/XB_Defs.pm::ENV{PATH} after xbone is \n",
	  "      installed to include the paths for the missing commands.\n",
	  "      Press ENTER to continue ...";
    my $dummy = <STDIN>;
  }
}

#---------------------------------------------------------------------
# os_check:
#	Determine if the OS is supported by X-Bone.
sub os_check(){
  my @freebsd_cmd = qw (ifconfig route ps sysctl);
  my @kame_cmd =    qw (gifconfig setkey);
  my @cairn_cmd =   qw (mkey spd iticonfig);
  my @linux_cmd =   qw (ifconfig route ps ip insmod lsmod);
  my $default_path = "/usr/local/v6/sbin:/sbin:/bin:/usr/bin:/usr/sbin";
  # Check if root.
  if ($<) {
    die "\n! You must be root to proceed with X-Bone installations.\n\n";
  }else{
    $ENV{PATH}= $default_path;
  }
  my $cmd;
  chop(my $os_name = `uname -sr`);
  print "==> [Operating System: $os_name]\n";
  if ($os_name =~ /^Linux (\d+)\.(\d+)\.(\d+)/){
    if($1>=2 && $2>=2 && $3>=5){
      print "    Version check ok (2.2.5+).\n";
      print "    - Verify PATH for the following commands:\n";
      print "        PATH = $ENV{PATH}\n";
      my $missing = 0;
      foreach my $c (@linux_cmd){
	chop (my $path = `which $c`);
	print "\t  $c: ";
	if($path =~ /^\//) {
	  print "$path\n";
        }else{
	  print "not found.\n";
	  $missing++;
        }
      }
      if ($missing){
	print "    ! Some of the commands are not in the PATH shown above. ",
		    "You need to edit\n",
	      "      xbone/lib/XB_Defs.pm::ENV{PATH} after xbone is installed ",
	            "to include the \n",
              "      paths for the missing commands.\n",
	      "      Press ENTER to continue...";
        my $dummy = <STDIN>;
      }
    }else{
      die "!!! Your version is too old. Upgrade to 2.2.5 or later.\n";
    }
  }elsif ($os_name =~ /^FreeBSD (\d)\.(\d)\-(\w+)/){
    if($1>=3 && $2>=2){
      $cmd = "sysctl -n net.inet.ipsec.def_policy 2> /dev/null";
      if(system $cmd){
        print "    * KAME is not installed. NO IPSec support!\n"; 
      }else{
        print "    - KAME detected. (IPSec & gif tun support)\n";
	print "      - Recursive tunneling support: [ok]\n";
	#gif_check();
	print "    - IPsec support:\n";
	kame_setkey_check();
	path_check(@freebsd_cmd, @kame_cmd);
      }
    }elsif($1==4){
      print "    - IP-IP tunneling support: [ok] => gif.\n";
      print "      - Recursive tunneling support: [ok]\n";
      #gif_check();
      if($2==0) {
	print "    * NO IPsec support: setkey(8) spddelete doesn't work.\n"; 
      } else {
	print "    - IPsec support:\n";
        kame_setkey_check();
      }
      path_check(@freebsd_cmd, @kame_cmd);
    }else{
      die "    * $os_name version not supported!\n";
    }
  }elsif ($os_name =~ /^FreeBSD CAIRN-(\d)\.(\d)$/){
    if($1>=2 && $2>=5){
      print "    FreeBSD CAIRN version ok (2.5+)\n";
      path_check(@freebsd_cmd, @cairn_cmd);
      $cmd="sysctl -n net.inet.ip.iti_recursion 2> /dev/null";
      if(system $cmd){
        print "    - Your system does not support recursive tunnels.\n";
      }else{
        print "    - Your system supports recursive tunnels.\n";
      }
    }else{
      die "!!! Your version is too old. Please upgrade to 2.5 or later.\n";
    }
  }else{
    die "!!! $os_name is currently unsupported by X-Bone.\n";
  }
}

#--------------------------------------------------------------------- 
# kame_setkey_check:
#	Determine if setkey(8) syntax is ok.
# Returns:
#       1 on success
sub kame_setkey_check () {
  chop (my $setkey = `which setkey`);
  # safety check.
  if ( ! -x $setkey ) {
    die "    - Error! Cannot execute \"setkey(8)\" to perform the test.\n";
  }
  # figure out if we're on an old-style (< 9/99) or new (> 9/99) KAME host
  my $pipe = "| setkey -c 2> /dev/null | grep '127.0.0.1' > /dev/null";
# local $SIG{PIPE} = sub { XB_Log::log "err", "Pipe $pipe broke"; return };
  local $SIG{PIPE} = sub { warn "Pipe $pipe broke."; return; };
  open PIPE, $pipe or die "Can not open pipe.";
  my $cmd = "spdadd 127.0.0.1 127.0.0.1 any -P ipsec ah/use ;\n" .
    "spddump ;\nspddelete 127.0.0.1 127.0.0.1 any ;\n";
  print PIPE $cmd;
# XB_Log::log "debug", $cmd;
  close PIPE or not $!
    or die "Cannot close pipe $pipe: $!";
  $new_kame_style = ($? ? 1 : 0); 
  print "      - setkey(8) version: " .
    ($new_kame_style ? "[19990906 & later]\n" : "[before 19990906]\n");

  # if we're on a new-style KAME host, figure out if spddelete works
  if($new_kame_style) {
    my $pipe = "| setkey -c 2> /dev/null | grep 'parse failed' > /dev/null";
#   local $SIG{PIPE} = sub { XB_Log::log "err", "Pipe $pipe broke"; return };
    local $SIG{PIPE} = sub { warn "Pipe $pipe broke."; return; };
    open PIPE, $pipe
      or die "Cannot open pipe $pipe: $!";
    my $cmd = "spddelete 127.0.0.1 127.0.0.1 any -P in ipsec;\n";
    print PIPE $cmd;
#   XB_Log::log "debug", $cmd;
    close PIPE or not $!
      or die "Cannot close pipe $pipe: $!";
    if(not $?) {
      print "      * setkey \"spddelete\" command doesn't work.\n".
	    "      * A new version of setkey(8) will be installed.\n";
#       and die "spddelete syntax check fails.";
      my @cmd = "touch replace_setkey";
      system (@cmd) == 0 or die "Can't do \"@cmd\"";
    }else{
      print "      - setkey(8) syntax:  [ok]\n";
    }
  }

  return 1 unless $@;
  # exception handling
  unless($@ =~ /^(open|close|system|spddelete)/) {
    # unknown exception caught, log and pass up a defined one
    warn "kame_setkey_check: caught unexpected exception $@";
  }
  # pass defined exceptions up to caller
  die "xb-Host-check::kame_setkey_check";
}

sub call($){
  my $pipe=shift;
  my $result;
  open PIPE, $pipe or die "cannot open pipe $pipe: $!";
  $result = <PIPE>;
  close PIPE or die "close pipe";
  return $result;
}

# Description:
#	Checks if multiple tunnels can be established between two end points.
# Arguments:
#	none
# Returns:
#	1 on success
# Exceptions:
#	
sub gif_check() {
  my @fakeIP = qw (10.0.1.2 10.1.2.3 10.2.3.4 10.3.4.5 10.4.5.6 10.5.6.7
		   255.255.255.255);
  my $if;
  my $gif=0;
  my $gif1;
  my $gif2;
  my $str = "A minimum of 16 recommended.\n";
  my $msg = 
  "      * Your system doesn't support multiple gif tunnels between two " .
	  "points.\n" .
  "      * You need to patch KAME gif code and recompile the kernel.\n" .
  "      * See INSTALL & FAQ for details.\n";
  # check the number of gif interfaces
  $if = call("ifconfig -l |");
  while($if =~ /gif\d+/g){ $gif++; }
  if(! $gif){
    print "      * Warning! You don't have any gif interfaces. $str";
  }else{
    # have gifs, check parallel tunneling
    if( $gif < 16){
      print "      * Warning! You only have $gif gif interfaces. $str";
    }else{
      print "      - $gif gif interfaces: [ok]\n";
    }
    # find unused gifs
    $if = call("ifconfig -ld |");
    if($if =~ /(gif\d+)\s+(gif\d+)/){	# grab 2 gifs
      $gif1=$1;
      $gif2=$2;
      my $m;
      my $r;
      my $i;
      my @cmd = (	"ifconfig  $gif1 $fakeIP[0] $fakeIP[1] |",
      			"gifconfig $gif1 $fakeIP[4] $fakeIP[5] |",
      			"ifconfig  $gif2 $fakeIP[2] $fakeIP[3] |",
      			"gifconfig $gif2 $fakeIP[4] $fakeIP[5] |" );
      for ($i=0; $i<=3; $i++){
        eval { $r=call $cmd[$i]; };
	if($@) { 
	  if ($i<3){
	    print "      ! Error: $@";
	    print "        The fake addresses could have been used already.\n";
	    last;
	  }else{
	    print "      ! Error: $@" .
	    print $msg;
            last;
          }
        }elsif($i==3){
	  print "      - Support multi-gif tunnels between two points.\n";
        }
      }
      # clean up the gifs we used
      @cmd = (	"ifconfig  $gif1 delete |", 
	       	"gifconfig $gif1 $fakeIP[6] $fakeIP[6] |",
	       	"ifconfig  $gif2 delete |",
	       	"gifconfig $gif2 $fakeIP[6] $fakeIP[6] |" );
      for (my $j=0; $j<$i; $j++){
	eval { $r=call $cmd[$j]; };
	if($@) {
	  print "      ! Error: $@";
	  print "        Error during clean up. Please delete $gif1 & ".
		        "$gif2 manually.\n";
        }
      }
      for $m ("ifconfig $gif1 down |", "ifconfig $gif2 down |"){
	eval { $r = call $m; };
	if($@){
	  print "      ! Error: $@";
	  print "        Please bring down $gif1 & $gif2 manually.\n";
        }
      }
    }else{
      print "      ! Couldn't find two unused gifs. Test aborted.\n";
    }
  }
}
  


#====================
# Main Program
#====================

os_check();

