# -*- perl -*-
# 
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
# 
# ----------------------------------------------------------------------------
#                                   X-BONE
#  
#                USC Information Sciences Institute (USC/ISI)
#                   Marina del Rey, California 90292, USA
#                          Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#    
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
# 
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
# 
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
# 
# Other copyrights might apply to parts of this software and are so noted when
# applicable. 
# 
# -------------------------------------------------------------------------
# 
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
#
#  $RCSfile: xb-Port-check.pl,v $
# 
# $Revision: 1.3 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:28 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Script to patch ports required by XBone
# 
# Note: Ok, we cheat! Right now the original ports in /usr/ports will be
#       move to portname-old and the customized port will replace the
#       original one.
#
#
# Start PERL on any system where perl is in the path
# This is a modified version of the "start perl" script
# provided in the PERL man pages, which starts perl
# on the remainder of the file regardless of whether it
# is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the
# perl path, rather than requiring it be hard-coded.
#
# setenv PERLDB_OPTS "N f=26"
#
# Must be set before perl starts!
#
# N     : Nonstop (noninteractive)
# f=26  : frame=26 = 16 + 8 + 2,
#               2 = entry and exit,
#               !4 = don't print args to functions
#               8= enabled overloaded stringify and tied FETCH
#               16= print return values from subroutines
#
############################################################
# the following line finds perl and runs it, in sh, csh, or perl:
#
# note - '-T' (TAINT) switch is not included, because 'which' often
# returns a version of perl that isn't secure. don't worry about it.
#
eval '(exit $?0)' && eval 'PERLDB_OPTS="N f=26";export PERLDB_OPTS;PERL=`which perl5`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERLDB_OPTS "N f=26"; setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

############################################################
# PERL CODE STARTS HERE
############################################################

use strict;

my @newports=qw{p5-Mail-Sendmail	p5-Net-Netmask 	ip-tun 
               	p5-File-CounterFile	p5-Net-SSLeay	openssl
		apache13-ssl};

my %subdir= (	"openssl",		"/security/",
		"apache13-ssl",		"/www/",
		"p5-File-CounterFile",	"/misc/",
		"p5-Mail-Sendmail", 	"/mail/",
		"p5-Net-Netmask",	"/net/",
		"p5-Net-SSLeay",	"/security/",
		"ip-tun",		"/net/");

my @patchport=qw{ openssl apache13-ssl };
my %ver=("openssl",	"1.38",
	 "apache13-ssl","2.62" );

my $portsdir=$ARGV[0];
my $curdir=$ARGV[1];

#- port_check -----------------------------------------------------------------
#	Look for required new ports.
# Arguments:
#	none
# Returns:
#	1=port missing; 0=found all ports. 
#------------------------------------------------------------------------------
sub port_check()
{
  my $port;
  my $p;
  my $missing=0;
  print "==> Check for the following new ports:\n";

  foreach $p (@newports){
    $port=$portsdir.$subdir{$p}.$p; 
    if( -d $port ) {
      print "      [X] $p\n";
    }else{
      print "      [ ] $p\n";
      $missing = 1;
    }
  }
  return $missing;
}

#= Main Program ===============================================================

  if(port_check()){
    print "<== * Some required ports are missing. Please update your ports.\n";
  } else {
    print "<== Port check ok. Found all the required ports in $portsdir.\n";
  }



