# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#                                   X-BONE
#
#                USC Information Sciences Institute (USC/ISI)
#                   Marina del Rey, California 90292, USA
#                          Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# ----------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
#
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
#
# -------------------------------------------------------------------------
#
#  $RCSfile: xb-Source-patch.pl,v $
#
# $Revision: 1.12 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:29 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Start PERL on any system where perl is in the path
# This is a modified version of the "start perl" script
# provided in the PERL man pages, which starts perl 
# on the remainder of the file regardless of whether it
# is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the
# perl path, rather than requiring it be hard-coded.
#
# setenv PERLDB_OPTS "N f=26"
#
# Must be set before perl starts!
#
# N 	: Nonstop (noninteractive)
# f=26	: frame=26 = 16 + 8 + 2,
#		2 = entry and exit,
#		!4 = don't print args to functions
#		8= enabled overloaded stringify and tied FETCH
#		16= print return values from subroutines
#
############################################################
# the following line finds perl and runs it, in sh, csh, or perl:
#
# note - '-T' (TAINT) switch is not included, because 'which' often
# returns a version of perl that isn't secure. don't worry about it.
#
eval '(exit $?0)' && eval 'PERLDB_OPTS="N f=26";export PERLDB_OPTS;PERL=`which perl5`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERLDB_OPTS "N f=26"; setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

############################################################
# PERL CODE STARTS HERE
############################################################

#use strict;


#- Global variables -----------------------------------------------------------

  my $TESTING=0;	# 1: a lot of patching outputs

  #- Information require user input -------------------------------------------
  my $OvlSuffix;
  my $OMHost;
  my $pDNS_name;
  my $pDNS_IP;
  my $sDNS_name;
  my $AdminEmail;
  my $mrtd_port = 5674;
  my $WebServerAdmin;
  my $WebServerName;
  my $WebServerDomain;
  my $ServerCert;
  my $ServerKey;

  #- Some paths ---------------------------------------------------------------
  my $WWW_Path = "/usr/local/www";
  my $CERT_Path = "/etc/xbone/cert";

  #- files & paths ------------------------------------------------------------
  my $xb_defs = "xbone/lib/XB_Defs.pm";
  my $xb_rd = "xbone/rd/xb-daemon.pl";
  my $xb_conf = "xbone/rd/apps/xb-config.pl";
  my $dns_path = "named_conf_sample";
  my $named_conf = "named.conf";
  my $resolv_conf = "resolv.conf";
  my $zone_path = "xbone";
  my $zone_file = "xbone.private";
  my $rev_zone_file = "xbone-reverse.private";
  my $httpsd_conf = "apache_conf_sample/httpsd.conf";
  my $www = "xbone/www";
  my $mrtd_conf = "/etc/mrtd.conf";


  #- Variables ----------------------------------------------------------------
  my $gui=0;


  #- Old variables ------------------------------------------------------------
  my $fake_IP = "333.333.333.333";

#- cmd_line_args() ------------------------------------------------------------
#  - Parse & check the command-line arguments.
#------------------------------------------------------------------------------
sub cmd_line_args()
{
  if($#ARGV != 0) 		{ usage();}
  elsif($ARGV[0] eq "gui")	{ $gui=1; }
  elsif($ARGV[0] eq "nogui")	{ $gui=0;}
  else				{ usage();}
}

#- usage ----------------------------------------------------------------------
sub usage()
{
  print "\n";
  print "Usage: xb-patch.pl _option_\n";
  print "       option = gui or nogui\n";
  print "\n";
  exit 1;
}

#- get_mrtd_port --------------------------------------------------------------
#  - Get MRTd telnet port from /etc/mrtd.conf if exists.
#  - input:  none
#  - output: none (set global variable $mrtd_port if found)
#------------------------------------------------------------------------------
sub get_mrtd_port()
{

  if (-f "$mrtd_conf") {
    open MRTd, "$mrtd_conf" or die "open $mrtd_conf failed: $!";
    my $line;
    while ($line = <MRTd>){
      if($line =~ /^\s*port\s*(\d+)\s*$/m){
	# found "port" entry in mrtd.conf, use it as default
	if($TESTING) { print "From $mrtd_conf: port = $1\n"; }
	$mrtd_port = $1;
	last;
      }
    }
    close MRTd or die "close $mrtd_conf failed: $!";
  }
  return $mrtd_port;
}

#- check_input ----------------------------------------------------------------
#  - Check STDIN against specified format.
#  - input:  prompt_msg, 0/1 (1=empty ok), "name"/"ip"/"path"/"num", 
#	     [default value]
#  - output: valid input or null
#------------------------------------------------------------------------------
sub check_input($$$;$)
{
  my ($prompt, $empty_OK, $type, $default) = @_;
  my $answer;
  my $ok=0;
  my $trailer;

  if(!defined $default) { 
    $trailer="NONE";
  }else{
    $trailer=$default;
  }

  do{
    print "$prompt"."[Default: $trailer] ";
    chop($answer = <STDIN>);
    SWITCH: {
      if($type eq "name") {
        if($answer =~ /^\s*[a-zA-Z_0-9~\/-]+(\.[a-zA-Z_0-9~\/-]+)+$/){
          return $answer;
          $ok=1;
        }
	last SWITCH;
      }
      if($type eq "ip") {
	if($answer =~ /^\s*\d+\.\d+\.\d+\.\d+$/){
	  return $answer;
	  $ok=1;
        }
	last SWITCH;
      }
      if($type eq "email") {
	if($answer =~ 
	/^(\s*[a-zA-Z_0-9~\/.-]+)\@([a-zA-Z_0-9~\/-]+(\.[a-zA-Z_0-9~\/-]+)+)$/){
	  return $1."\\@".$2;
	  $ok=1;
        }
	last SWITCH;
      }
      if($type eq "path") {
	if($answer =~
	/^(\/([a-zA-Z_0-9~.-]+\/)+)([a-zA-Z_0-9~.-]+)$/ ) {
          return $answer;
	  $ok=1;
        }
	last SWITCH;
      }
      if($type eq "num") {
        if($answer =~ /^\d+$/ ) {
          return $answer;
          $ok=1;
        }
        last SWITCH;
      }
    } # SWITCH
    if($answer eq ""){
      if($empty_OK){
        return $default;
        $ok=1;
      }else{
        print "       ! No default values. Must enter a name.\n";
        $ok=0;
      }
    }else{
      print "       ! Not a valid entry. Try again.\n";
      $ok=0;
    }
  } until ($ok==1);
  # Shouldn't be here. But who knows?
  return $answer;
}

#- user_input -----------------------------------------------------------------
#  - Gather information from STDIN.
#------------------------------------------------------------------------------
sub get_input()
{
  my $prompt;
  my $hn = "Fully-qualified hostname";
  my $nl = "\n       ";

  my @greeting = ( 
      "\n==> Enter the following info to finish XBone installation:\n");
  print @greeting;

  $prompt="    >> $hn where Overlay Manager runs:$nl";
  $OMHost = check_input($prompt, 0, "name");

  $prompt="    >> DNS domain name suffix for hosts & routers on the overlays:".
	  $nl;
  $OvlSuffix = check_input($prompt, 1,"name", "xbone.private");

  $prompt="    >> $hn of the primary DNS server for the $nl".
	  "\"$OvlSuffix\" domain:";
  $pDNS_name = check_input($prompt, 0, "name");
	  
  $prompt="    >> IP address of the primary DNS server for the ".
	  "\"$OvlSuffix\" domain: $nl";
  $pDNS_IP = check_input($prompt, 0, "ip");
    
  $prompt="    >> $hn of the secondary DNS server for the $nl\"$OvlSuffix\" ".
	  "domain (Skip if none):";
  $sDNS_name = check_input($prompt, 1, "name");

  $prompt="    >> Email address of the adminstrator of this X-Bone ".
	  "installation:$nl";
  $AdminEmail = check_input($prompt, 0, "email");
  get_mrtd_port;
  $prompt="    >> MRTd port number ([ENTER] if MRTd is not used): ";
  $mrtd_port = check_input($prompt, 1, "num", $mrtd_port);

  # Derive default values for apache conf file from the above. 
  $WebServerName = $OMHost;
  $WebServerDomain = $WebServerName;
  $WebServerDomain =~ s/[a-zA-Z_0-9~\/-]*\.(\S*)/$1/;
  $WebServerAdmin = $AdminEmail;
  $WebServerAdmin =~ s/^(\S+)\\(\S+)/$1$2/;
  $ServerCert = "$CERT_Path/$WebServerName.cert.pem";
  $ServerKey =  "$CERT_Path/$WebServerName.key.pem";

  # Only display the following prompt when doing GUI installation.
  if ($gui == 1) { 
  $prompt="    >> $hn of the web server for the X-Bone user interface.\n".
	  "       This name should NOT be inside the overlay namespace:$nl";
  $WebServerName = check_input($prompt, 1, "name", $WebServerName);

  $prompt="    >> DNS domain name suffix of the web server for the X-Bone ".
	  "user interface;${nl}This name should NOT be in side the ".
	  "overlay domain namespace:$nl";
  $WebServerDomain = check_input($prompt, 1, "name", $WebServerDomain);

  $prompt=
  "    >> Complete filesystem path of the web server public key certificate:\n".
  "       If one is not available, note the location you enter below, and \n".
  "       copy the file later.${nl}";
  $ServerCert = check_input($prompt, 1, "path", $ServerCert);

  $prompt=
  "    >> Complete filesystem path of the web server private key certificate:".
  "\n       If one is not available, note the location you enter here, and \n".
  "       copy the file later.${nl}";
  $ServerKey = check_input($prompt, 1, "path", $ServerKey);
  } # if($gui == 1)
}

#- replace_str ----------------------------------------------------------------
#  - Replace strings with user inputs for given files.
#  - input:  filename
#------------------------------------------------------------------------------
sub replace_str($)
{
  my ($file) = @_;
  my $hostname = `hostname`;
  chop $hostname;
  #- string matching key pairs ------------------------------------------------
  my %pair_xbdef = (
		"\"www.xbone.net\"",            "\"$OMHost\"",
		"\"xbone.net\"",                "\"$OvlSuffix\"",
		"rum.isi.edu",                  $pDNS_name,
		"moo.isi.edu",                  $sDNS_name,
		"www.xbone.net",                "$OMHost",
		"\<..............\>",           "<$AdminEmail>",
		"X-Bone Certification Authority",
		"X-Bone Administrator",
		"\"/etc/X",			"\"/etc/xbone/X",
		".XB_Defs..XBONE_HOME/om",	"/etc/xbone",
		".XB_Defs..XBONE_HOME/lib",	"/etc/xbone",
		"5674",				"$mrtd_port"
		);
  my %pair_xbrd=(".XB_Defs..XBONE_HOME...\"/rd/\"...", "");
  my %pair_xbconf = ("\"cert",			"\"/etc/xbone/cert");
  my %pair_named =( "\"xbone.private\"",	"\"$OvlSuffix\"" );
  my %pair_zone = ( "xbone.private",		$OvlSuffix ); 
  my %pair_resolv=( "333.333.333.333",		$pDNS_IP );
  my %pair_httpsd = (
		"ServerName www.xbone.net",	"ServerName ${WebServerName}",
		"VirtualHost www.xbone.net",	"VirtualHost ${WebServerName}",
		"from .isi.edu",		"from .${WebServerDomain}",
		"xbone\@isi.edu",		"${WebServerAdmin}",
		"/usr/local/www/server-cert/www.xbone.net.cert.pem",
		"$ServerCert",
		"/usr/local/www/server-cert/www.xbone.net.key.pem",
		"$ServerKey",
		"/usr/local/www/lib",		"/etc/xbone"
		);
  my %pair_www = ("....home.xbone.perl..arch.", "");

  SWITCH:{
    if ($file =~ /XB_Defs/){	%str_pair = %pair_xbdef;	last SWITCH; }
    if ($file =~ /xb-conf/){	%str_pair = %pair_xbconf;	last SWITCH; }
    if ($file =~ /named\.conf/){%str_pair = %pair_named;	last SWITCH; }
    if ($file =~ /private/){	%str_pair = %pair_zone;		last SWITCH; }
    if ($file =~ /resolv/){	%str_pair = %pair_resolv;	last SWITCH; }
    if ($file =~ /httpsd/){	%str_pair = %pair_httpsd;	last SWITCH; }
    if ($file =~ /www/){	%str_pair = %pair_www;		last SWITCH; }
    if ($file =~ /daemon/){	%str_pair = %pair_xbrd;		last SWITCH; }
    die "$file: Unknown filename.";
  }

  unless(-f "$file.ORIG") {
    unless((rename "$file", "$file.ORIG")==1){
      die $!, "\n";
    }
  }

  open OLD, "$file.ORIG"
    or die "open $file.ORIG failed: $!";
  open NEW, ">$file"
    or die "open $file failed: $!";

  while (my $line = <OLD>) {
    foreach my $k (keys %str_pair){
      if($line =~ /$k/){
	if($TESTING) {	print "OLD: $line"; }
        $line =~ s/$k/$str_pair{$k}/g;
        if($TESTING) { print "NEW: $line"; }
      }
    }
    print NEW $line;
  }

  close OLD or die "close $file.ORIG failed: $!";
  close NEW or die "close $file failed: $!";

}


#- patch_files ----------------------------------------------------------------
sub patch_files()
{
  #- files & paths ------------------------------------------------------------
  my $xb_defs =		"xbone/lib/XB_Defs.pm";
  my $dns_path =	"named_conf_sample";
  my $named_conf =	"named.conf";
  my $resolv_conf =	"resolv.conf";
  my $zone_path =	"xbone";
  my $zone_file =	"xbone.private";
  my $rev_zone_file =	"xbone-reverse.private";
  my $httpsd_conf =	"apache_conf_sample/httpsd.conf";
  my $prefix = 		".";
  unless (-d "xbone"){
    print "\n==> Warning: ./xbone doesn't exist. Try /usr/local.\n";     
    if(-d "/usr/local/xbone"){
      $prefix = "/usr/local";
    }
  }
  my @files = (		"$prefix/$xb_defs",
			"$prefix/$xb_rd",
			"$prefix/$xb_conf",
			"$dns_path/$named_conf",
			"$dns_path/$resolv_conf",
			"$dns_path/$zone_path/$zone_file",
			"$dns_path/$zone_path/$rev_zone_file",
			"$httpsd_conf",
			"$prefix/$www/cgi-bin/xb-get-ca-cert.pl",
			"$prefix/$www/cgi-bin/xb-req-user-cert.pl",
			"$prefix/$www/cgi-bin/index.pl",
			"$prefix/$www/s-cgi-bin/XB_Interface.pm",
			"$prefix/$www/s-cgi-bin/xb-admin.pl",
			"$prefix/$www/s-cgi-bin/xb-cookies.pl",
			"$prefix/$www/s-cgi-bin/xb-create.pl",
			"$prefix/$www/s-cgi-bin/xb-destroy.pl",
			"$prefix/$www/s-cgi-bin/xb-req-host-cert.pl",
			"$prefix/$www/s-cgi-bin/xb-status.pl",
			"$prefix/$www/s-cgi-bin/index.pl"
		);

  print "\n==> Patch xbone files:\n";
  for my $f (@files){
    print "    >> Patching $f\n";
    replace_str($f);
    if( $f =~ /\.pl/){ chmod 0755, $f; }
  }
} 

#==============================================================================
# main
#==============================================================================

cmd_line_args();	# get command-line argument: "gui" or "nogui"

get_input();		# read user inputs

patch_files();		# patch files with user inputs


