# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
# 
#
# ----------------------------------------------------------------------------
#  $RCSfile: XB_Defs.pm,v $
#
# $Revision: 1.61 $
#   $Author: yushunwa $
#     $Date: 2000/10/27 23:46:18 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
#
#############################################
#
# Common definitions used by XBONE Perl modules.
#
# Principal authors: Gregory Finn, Anindo Bannerjea
#
#############################################

package XB_Defs;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw();
@EXPORT_OK = qw(NODEOS ALLOWED_NODEOS XBONE_NET
		DNS_FILE_FORWARD DNS_FILE_REVERSE NAMED_PID XBONE_HOME
		XBONE_PROTOCOL DNS_SERVER DNS_SERVER_SECONDARY
		DNS_UPDATE_MODE SECURITY_CA_CERT SECURITY_SERVER_CERT
		SECURITY_SERVER_KEY MAX_MCAST_RADIUS MCAST_PORT
		MCAST_ADDR CA_EMAIL CONNECT_QUEUESIZE OVLMGR_PORT
		NODEOS_LABEL IP_MULTICAST_IF IP_MULTICAST_TTL
		IP_MULTICAST_LOOP IP_ADD_MEMBERSHIP IP_DROP_MEMBERSHIP
		IP_MULTICAST_VIF SOL_IP ACTUATOR_WAIT
		ACTUATOR_MSG_LIMIT DAEMON_CMDWAIT GATED_PATH
                LOG_DEST LOG_MASK LOG_FILE DUPLICATE_DETECTION_INTERVAL
                XBONE_RELEASE XBONE_OVLMGR_BACKCOMPAT XBONE_DAEMON_BACKCOMPAT
		MRTD_PORT MRTD_CONF MRTD_CONNECTION_TIMEOUT
		LOOPBACK_IP_ADDRESS IP_ADDRESS_LENGTH);

use strict;
use sigtrap;
use FindBin;
use Config;


# XXX THE XBONE DIRECTORY MUST BE NAMED "xbone" FOR THIS TO WORK!
$XB_Defs::XBONE_HOME = $FindBin::RealBin;
if($XB_Defs::XBONE_HOME =~ /(.*\/xbone)/) { $XB_Defs::XBONE_HOME = $1; } 



############################################################
# The following are used only by xb-overlay-manager
# 	They are provided here because
#		- xb-overlay-manager has no config file
#		- this is the place for changeable constants
############################################################

$XB_Defs::XBONE_OVERLAY_MANAGER =   # Hostname used by the OM to 
  "www.xbone.net";                  # identify itself in heartbeats

$XB_Defs::SECURITY_SERVER_CERT =    # used by the OM as its public key
  "$XB_Defs::XBONE_HOME/om/cert/www.xbone.net.cert.pem";

$XB_Defs::SECURITY_SERVER_KEY =     # used by the OM as its private key
  "$XB_Defs::XBONE_HOME/om/cert/www.xbone.net.key.pem";

$XB_Defs::CA_EMAIL =                # email address of the CA that signs 
  "X-Bone Certification Authority ".# certificates for this xbone installation
  "<xbone\@isi.edu>";             

$XB_Defs::DEMO_MODE = 0;            # Flag for demo mode. Set this to 1 to
                                    # force the OM to pick nodes in the 
                                    # reverse order. 0 is normal operation.

$XB_Defs::XBONE_OVLMGR_BACKCOMPAT = # Overlay Manager is backward compatible
  "";                               # with these XBONE_RELEASE release versions
                                    # (separated by spaces)

$XB_Defs::OVLMGR_PING_INTERVAL = 60*15;
                                    # Overlay Manager sweeps all hosts in
  				    # all overlays, issuing heartbeat
				    # messages to each of them.  This sweep
				    # is performed as once every interval.
				    # Value is seconds.


############################################################
# The following are used only by nearly everybody :-)
############################################################

$XB_Defs::XBONE_VERSION_DATE =	    # Date this version was created.
  "15/10/2000";                      # "<day_num>/<month_num>/<year_num>"

$XB_Defs::XBONE_OBSOLETE_LIMIT =    # Seconds after the version date
  60*60*24*30*6;                    # when the version should be obsolete.

$XB_Defs::XBONE_PROTOCOL = "1.0";   # Release version of the Xbone protocol

$XB_Defs::XBONE_RELEASE = "1.2";    # Release version of Xbone system

$XB_Defs::XBONE_NET =		    # The domain name associated with this
  "xbone.net";                      # particular Xbone overlay network.

$XB_Defs::NODEOS = "";              # Holds OS name read from daemon
  				    # configuration file.

@XB_Defs::ALLOWED_NODEOS =	    # List of allowed OS names for
  qw(freebsd linux solaris cairn kame nist);  # Resource Daemon (RD) hosts.

@XB_Defs::ALLOWED_DNS_NODEOS =	    # List of allowed OS names for the one
  qw(freebsd linux solaris cairn kame nist);  # RD host that manipulates the Xbone
                                    # DNS domain.

%XB_Defs::NODEOS_LABEL =            # labels (textual representation) of 
  ( freebsd => "FreeBSD",           # allowed NODEOS tags - use these when 
    cairn   => "FreeBSD/CAIRN",     # printing (e.g. web pages)
    kame    => "FreeBSD/KAME", 
    linux   => "Linux", 
    nist    => "Linux/NIST",
    solaris => "Solaris" );

$XB_Defs::SYSLOG = (2**0);
$XB_Defs::ERR    = (2**1);
$XB_Defs::OUT    = (2**2);
$XB_Defs::FILE   = (2**3);

$XB_Defs::LOG_DEST =                # Log to both the log file and stderr.
#  $XB_Defs::FILE;                  # Also see lib/XB_Log.pm.
  $XB_Defs::FILE|$XB_Defs::OUT;     # Also see lib/XB_Log.pm.


$XB_Defs::LOG_MASK = "emerg alert crit err warning";
				    # Specify the set of things you
#  "emerg alert crit err warning ". # want to log.  Possible things are:
#  "notice info debug debug1 ".     # emerg, alert, crit, err, warning
#  "debug2 debug3 debug4 debug5";   # notice, info, debug0, debug1, ...
                                    #
                                    # For debugging, info, debug and debug1
                                    # are useful, but becomes very wordy.


$XB_Defs::LOG_FILE =                # File name where logging data is written.
  "/usr/tmp/Xbone.log";             # Use only when XB_Defs::LOG_DEST & "FILE".


$XB_Defs::DAEMON_CONF_FILE = "/etc/Xbone_daemon.conf";
                                    # location of Xbone daemon's configuration

$XB_Defs::DAEMON_STATE_FILE = "/etc/Xbone_daemon.state";
                                    # File where Xbone daemon's overlay state
                                    # is kept for daemon crash recovery.

%XB_Defs::DAEMON_STATE = ();	    # Hash of Xbone daemon's current state

$XB_Defs::ERASE_AT_STARTUP = 0;     # Erase Xbone daemon's tunnel, IPSec
  				    # and route state.  NOTE: This should be
                                    # non-zero only at ISI.

$XB_Defs::DAEMON_STATE_TIME = 1;    # Last time that the Xbone daemon's state
  				    # was dumped out to the state file.

$XB_Defs::OVERLAY_ALIVE_TIME = 60*120;
                                    # Seconds that the resource daemon will
  				    # allow an overlay to remain without
                                    # receiving a keep-alive message from
                                    # an overlay manager.

$XB_Defs::OVERLAY_SWEEP_INTERVAL = 60*10;
                                    # Seconds that resource daemon waits
                                    # between sweeps, looking for overlays
                                    # that have not been kept alive.

#------------------------------------------------------------
# OS dependent section to define NAMED_ZONE_PATH & NAMED_PID.
#------------------------------------------------------------
  
if ($Config{osname} eq "linux") {
  $XB_Defs::NAMED_ZONE_PATH="/var/named/xbone"; # NEED VERIFICATION!
  $XB_Defs::NAMED_PID="/var/run/named.pid";
}
if ($Config{osname} eq "solaris") {
  $XB_Defs::NAMED_ZONE_PATH="/var/adm/domain/xbone";
  $XB_Defs::NAMED_PID="/etc/named.pid";
}
if ($Config{osname} eq "freebsd") {
  $XB_Defs::NAMED_ZONE_PATH="/etc/namedb/xbone";
  $XB_Defs::NAMED_PID="/var/run/named.pid";
}     

$XB_Defs::DNS_FILE_FORWARD =        # Forward DNS database file for overlays.b
  "$XB_Defs::NAMED_ZONE_PATH/xbone.private";

$XB_Defs::DNS_FILE_REVERSE =        # Reverse DNS database file for overlays
  "$XB_Defs::NAMED_ZONE_PATH/xbone-reverse.private";

$XB_Defs::DNS_SERVER =              # Primary Xbone DNS name server
  "rum.isi.edu";

$XB_Defs::DNS_SERVER_SECONDARY =    # Secondary Xbone DNS server
  "moo.isi.edu"; 

###################################################################
# The only allowed values for the DNS update mode are:
#
# $DNS_UPDATE_MODE = "update_both";
# Update both assumes that the DNS is running in two primaries
# mode and that the DNS system will not update the second primary
#
# $DNS_UPDATE_MODE = "update_1_hup_2";
# Update 1 hup 2 assumes that we have one primary and one secondary
# so we update the primary and hup the secondary to get an instantaneous
# update.
#
# The current decision is to run as update_1_hup_2
###################################################################

$XB_Defs::DNS_UPDATE_MODE =         # See above block comment.
  "update_1_hup_2";

$XB_Defs::DNS_TIMEOUT = 15; 	    # Fifteen second DNS cache time.
  				    # A value lower than 15 is undefined.

# used by both RD and OM when calling XB_SSL::init()
$XB_Defs::SECURITY_CA_CERT = 
  "$XB_Defs::XBONE_HOME/lib/cert/CAcert.pem";

$XB_Defs::MCAST_ADDR =		    # Multicast address used by XBONE
  "224.192.0.1";

$XB_Defs::MCAST_PORT = 265;         # Multicast port used by XBONE and also
  				    # the unicast TCP and UDP ports used.

$XB_Defs::OVLMGR_PORT = 2165;       # TCP port used by overlay manager
 		                    # These are the OFFICIAL IANA X-Bone ports

$XB_Defs::XB_IP_EXTERNAL_BLOCK =    # IP block for releases, from RFC 1918
  "192.168.6.0/24";                 # (breaks rev DNS w/o RD using XB DNS) 
$XB_Defs::XB_IP_INTERNAL_BLOCK =    # IP block for releases, from RFC 1918
  "192.168.7.0/24";                 # (breaks rev DNS w/o RD using XB DNS) 

#$XB_Defs::XB_IP_EXTERNAL_BLOCK =    # IP block owned by X-Bone project, for
#  "198.32.6.0/24";                  # local ISI use
#$XB_Defs::XB_IP_INTERNAL_BLOCK =    # IP block owned by X-Bone project, for
#  "198.32.7.0/24";                  # local ISI use

$XB_Defs::XB_OVL_SUBNET_BITS = 3;

$XB_Defs::MAX_MCAST_RADIUS = 16;    # The largest radius that should be used

$XB_Defs::DUPLICATE_DETECTION_INTERVAL  # time interval in seconds we should
  = 120;                                # keep msgs to prevent duplicates 

$XB_Defs::DAEMON_CMDWAIT = 15;	    # Number of seconds the XBONE host daemon
				    # waits for commands from the overlay mgr

$XB_Defs::ACTUATOR_WAIT = 15;       # Number of seconds the XBONE overlay mgr
  				    # waits for UDP responses from host daemons

$XB_Defs::CGI_CONNECTION_TIMEOUT =  # The amount of time the CGI script
  120;				    # waits for responses from the overlay
				    # manager before giving up (in seconds)

$XB_Defs::CONNECTION_TIMEOUT =	    # The amount of time the overlay manager
  60;				    # waits for responses from the daemon
				    # before giving up (in seconds)

$XB_Defs::CONNECT_QUEUESIZE = 4;    # TCP connection queue size limit

$XB_Defs::ACTUATOR_MSG_LIMIT =	    # Maximum size of overlay mgr's reservation
    65536;		            # datagram

$XB_Defs::IP_MULTICAST_IF = 9;
$XB_Defs::IP_MULTICAST_TTL = 10;
$XB_Defs::IP_MULTICAST_LOOP = 11;
$XB_Defs::IP_ADD_MEMBERSHIP = 12;
$XB_Defs::IP_DROP_MEMBERSHIP = 13;
$XB_Defs::IP_MULTICAST_VIF = 14;


########################################################
# getsockopt() and setsockopt layer for IP protocol.
# Needed due to bug in Perl version of getprotobyname().
########################################################

$XB_Defs::SOL_IP = 0;


#######################################
# Global variables used across modules
#######################################

@XB_Defs::DNS_ADD_FWD_PARMS = ();     # Holds DNS forward addition parameters
@XB_Defs::DNS_DEL_FWD_PARMS = ();     #   "    "  forward deletion     "

$XB_Defs::GATED_PATH = "/etc/";       # dir where gated.conf is
$XB_Defs::GATED_PID_PATH = "/etc/";   # dir of gated.pid

$XB_Defs::MRTD_PORT = 5674;           # Port to try to connect to mrtd

$XB_Defs::MRTD_CONF                   # Configuration file of mrtd, used
  = "/etc/mrtd.conf";                 # to grab the port number as a last 
                                      # resort.

$XB_Defs::MRTD_CONNECTION_TIMEOUT     # The Amount of time the RD waits for
    = 20;                             # a response from mrtd. (in seconds)

$XB_Defs::LOOPBACK_IP_ADDRESS         # IP Loopback address 
    = "127.0.0.1";

$XB_Defs::IP_ADDRESS_LENGTH           # Length of IP Address
    = 32;

$ENV{PATH} = "";
foreach my $thispath (qw(/usr/local/v6/sbin /bin /sbin /usr/bin /usr/sbin)) {
  if(-d $thispath) { $ENV{PATH} .= ":$thispath"; }
}
$ENV{PATH} =~ s/^://;

%XB_Defs::KEY_LENGTHS = ( "md5"      =>  128,
			  "sha1"     =>  160,
			  "null"     => 2048, #  0-2048-bit key
			  "des"      =>   64,
			  "3des"     =>  192,
			  "simple"   => 2048, #  0-2048-bit key
			  "blowfish" =>  448, #  40-448-bit key
			  "cast128"  =>  128, #  40-128-bit key
			  "rc5"      => 2048  # 40-2048-bit key
			);

$XB_Defs::CAIRN_SPD_RULE_MIN = 50000; # only use rule numbers >= this
$XB_Defs::CAIRN_SPD_RULE_MAX = 60000; # only use rule numbers <= this


# Interfaces we know how to tunnel with, in the order they should be tried
# by Tunnel.pm.


@XB_Defs::TUNNELING_IFS = qw(iti gif tunl tun); # GIF IS BROKEN under 3.4!


$XB_Defs::XB_TUNNEL_CMD =           # relative! path to ip-tun binary
  "rd/test/xb-ip-tunnel/ip-tun";    # (also see XB_Tunnel::init)

#- Tunnel Aliasing ------------------------------------------------------------
#  - Use aliases of tunnel interfaces (KAME/gif, Linux/tunl) when more than one
#    tunnel is needed between two points.
#  - Currently only works on Kame/gif & Linux tunl interfaces.
#  - Only turn this off when XBone-specific patches to Kame & Linux ipip module
#    are applied.
#------------------------------------------------------------------------------

$XB_Defs::TUN_ALIASING =	# use aliasing on tunnel interface 
#	0;                      # turn off tunnel aliasing
	1;                      # turn on  tunnel aliasing

#- Tunnel Interface Sharing (Tunnel Aliasing) ---------------------------------
#  SHARE_TUN_IF = 0 --> disable sharing of tunnel interfaces
#    - XBone will NOT create tunnel aliases on an interface which already has
#      NON-XBone addresses present.
#    - XBone will DELETE/DOWN an interface once the last X-Bone alias is
#      deleted even if there may still be other NON-XBone aliases.
#  SHARE_TUN_IF = 1 --> enable sharing of tunnel interfaces 
#    - The opposite.
#------------------------------------------------------------------------------

$XB_Defs::SHARE_TUN_IF =	# share tunnel interfaces w/ non-xbone apps
#	0;			#   disable sharing
	1;			#   enable  sharing


1;                            # Insure TRUE return if module is interpreted.


















