# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
# 
#
# ----------------------------------------------------------------------------
#  $RCSfile: XB_Utils.pm,v $
#
# $Revision: 1.13 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:32 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Authors: Anindo Banerjea, Gregory Finn

#############################################
#
# Common utilities used by XBONE Perl modules.
#
#############################################

package XB_Utils;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(XB_mcast_ttl XB_mcast_loopback XB_mcast_no_loopback
	     XB_mcast_send_socket XB_mcast_recv_socket XB_udp_socket
	     XB_ip_base XB_ip_mask XB_obsolete XB_banner_msg);
@EXPORT_OK = qw();

use strict;
use sigtrap;

use Socket;
use Net::Netmask;
use Time::Local;

use XB_Defs;
use XB_Log;


##############################
#
# XB_mcast_ttl ( mcast_socket, ttl )
#
# Set the TTL value for the multicast socket mcast_socket to the value
# passed in ttl, which must be in the range 0-255.
##############################

sub XB_mcast_ttl ($$)
{
    my ($mcast_socket, $ttl) = @_;
    my $subrname = "XB_mcast_ttl";

    XB_Log::log ("info", "->$subrname (@_)");

    unless (setsockopt ($mcast_socket, $XB_Defs::SOL_IP,
			$XB_Defs::IP_MULTICAST_TTL, pack ("C", $ttl)))
    {die "Cannot set multicast socket TTL: $!.\n"; };

    XB_Log::log ("info", "<-$subrname");
}






##############################
#
# XB_mcast_loopback ( mcast_socket )
#
# Set the multicast socket mcast_socket to loopback.
##############################

sub XB_mcast_loopback ($$)
{
    my ($mcast_socket, $ttl) = @_;
    my $subrname = "XB_mcast_loopback";

    XB_Log::log ("info", "->$subrname (@_)");

    unless (setsockopt ($mcast_socket, $XB_Defs::SOL_IP,
			$XB_Defs::IP_MULTICAST_LOOP, pack ("C", 1)))
    {die "Cannot set multicast socket loopback: $!.\n"; };

    XB_Log::log ("info", "<-$subrname");
}





##############################
#
# XB_mcast_no_loopback ( mcast_socket )
#
# Set the multicast socket mcast_socket to loopback.
##############################

sub XB_mcast_no_loopback ($$)
{
    my ($mcast_socket, $ttl) = @_;
    my $subrname = "XB_mcast_no_loopback";

    XB_Log::log ("info", "->$subrname (@_)");

    unless (setsockopt ($mcast_socket, $XB_Defs::SOL_IP,
			$XB_Defs::IP_MULTICAST_LOOP, pack ("C", 0)))
    {die "Cannot clear multicast socket loopback: $!.\n"; };

    XB_Log::log ("info", "<-$subrname");
}







##############################
#
# XB_mcast_send_socket ( \fh, ip_iface, mcast_addr )
#
# Obtains a multicast send socket that is associated with the IP interface
# passed in ip_iface.  The multicast address is passed by mcast_addr.
#
# ip_iface is a string and may be either a recognized host name or a
# dotted decimal address.  If it is NULL, the INADDR_ANY address is
# substituted.
#
# The filehandle used to attach to the socket is passed by reference
# in \fh.
##############################

sub XB_mcast_send_socket ($$$)
{
    my ($fh, $ip_iface, $mcast_addr) = @_;
    my $subrname = "XB_mcast_send_socket";
    my ($mcastaddr, $ipaddr, $mcastport, $conn_protonumber);

    XB_Log::log ("info", "->$subrname (@_)");

    if ($ip_iface)
    {
	unless ($ipaddr = inet_aton ($ip_iface))
	{ die "Cannot resolve IP address.\n"; };
    }
    else  { $ipaddr = INADDR_ANY; };

    unless ($mcastaddr = inet_aton ($mcast_addr))
    { die "Cannot resolve mcast_addr.\n"; };

    unless ($conn_protonumber = getprotobyname ('udp'))
    {die "Cannot resolve UDP protocol.\n"; };

    unless (socket ($fh, PF_INET, SOCK_DGRAM, $conn_protonumber))
    {die "Cannot obtain socket: $!.\n"; };

    unless (setsockopt ($fh, $XB_Defs::SOL_IP,
			$XB_Defs::IP_MULTICAST_IF, $ipaddr))
    {die "Cannot set multicast interface: $!.\n"; };

    XB_Log::log ("info", "<-$subrname");
}





##############################
#
# XB_mcast_recv_socket ( \fh, ip_iface, mcast_addr, mcast_port )
#
# Obtains a multicast receive socket that is associated with the IP
# interface passed in ip_iface.  The multicast address is passed by
# mcast_addr and the port by mcast_port.
#
# ip_iface is a string and may be either a recognized host name or a
# dotted decimal address.  If it is NULL, the INADDR_ANY address is
# substituted.
#
# The filehandle used to attach to the socket is passed by reference
# in \fh.
##############################

sub XB_mcast_recv_socket ($$$$)
{
    my ($fh, $ip_iface, $mcast_addr, $mcast_port) = @_;
    my $subrname = "XB_mcast_recv_socket";
    my ($mcastaddr, $ipaddr, $mcastport, $conn_protonumber);

    XB_Log::log ("info", "->$subrname (@_)");

    if ($ip_iface)
    {
	unless ($ipaddr = inet_aton ($ip_iface))
	{ die "Cannot resolve IP address.\n"; };
    }
    else  { $ipaddr = INADDR_ANY; };

    unless ($mcastaddr = inet_aton ($mcast_addr))
    { die "Cannot resolve mcast_addr.\n"; };

    $mcast_port = pack_sockaddr_in ($mcast_port, $mcastaddr);

    unless ($conn_protonumber = getprotobyname ('udp'))
    {die "Cannot resolve UDP protocol.\n"; };

    unless (socket ($fh, PF_INET, SOCK_DGRAM, $conn_protonumber))
    {die "Cannot obtain socket: $!.\n"; };

    unless (setsockopt ($fh, SOL_SOCKET, SO_REUSEADDR, pack ("l", 1)))
    {die "Cannot reuse socket address: $!.\n"; };

    unless (setsockopt ($fh, $XB_Defs::SOL_IP,
			$XB_Defs::IP_ADD_MEMBERSHIP,
			pack ("a4a4", $mcastaddr, $ipaddr)))
    {die "Cannot set multicast membership: $!.\n"; };

    unless (bind ($fh, $mcast_port))
    {die "Cannot bind to multicast group: $!.\n"; };

    XB_Log::log ("info", "<-$subrname");
}





##############################
#
# XB_udp_socket ( \fh, dest_addr, udp_port )
#
# Obtains a UDP socket.  The destination address is passed by dest_addr
# and the port by udp_port.  If dest_addr is false, INADDR_ANY is assumed.
#
# The filehandle used to attach to the socket is passed by reference
# in \fh.
##############################

sub XB_udp_socket ($$$)
{
    my ($fh, $dest_addr, $udp_port) = @_;
    my $subrname = "XB_udp_socket";
    my ($destaddr, $ipaddr, $mcastport, $conn_protonumber);

    XB_Log::log ("info", "->$subrname (@_)");

    if ($dest_addr)
	{
	    unless ($destaddr = inet_aton ($dest_addr))
	    { die "Cannot resolve dest_addr.\n"; };
	}
    else { $destaddr = INADDR_ANY; };

    $udp_port = pack_sockaddr_in ($udp_port, $destaddr);

    unless ($conn_protonumber = getprotobyname ('udp'))
    {die "Cannot resolve UDP protocol.\n"; };

    unless (socket ($fh, PF_INET, SOCK_DGRAM, $conn_protonumber))
    {die "Cannot obtain socket: $!.\n"; };

    unless (setsockopt ($fh, SOL_SOCKET, SO_REUSEADDR, pack ("l", 1)))
    {die "Cannot reuse socket address: $!.\n"; };

    XB_Log::log ("info", "<-$subrname");
}


# Description: 
#	Return value of sysctl variable $var.
# Arguments:
#	$var	sysctl variable to check
# Returns:
#	value of $var if found
#	undef if $var not found
# Exceptions:
#	"sysctl_read" on failure
sub sysctl_read ($)
{
  my $var = shift; # variable to read
  my $val;
  # print trace line
  XB_Log::log "info", "-> XB_Utils::sysctl_read $var";
  eval {
  # get the value by reading from a pipe to sysctl
    my $pipe = "sysctl -n $var 2> /dev/null |";
    open PIPE, $pipe or XB_Log::log "err", "cannot open pipe $pipe: $!" 
      and die "open";
    # read the output
    $val = <PIPE>;
    # done with pipe
    close PIPE or not $! or XB_Log::log "err", "cannot close pipe $pipe: $!" 
      and die "close";
    # post-process
    if(defined $val) { chomp $val; }
  };
  # print trace line
  XB_Log::log "info", "<- XB_Utils::sysctl_read $var";
  return $val unless $@; # success if no exception
  # exception handling
  if($@ =~ /^(open|close)/) {
    # known exception, nothing to handle, pass up
  } else {
    # unknown exception caught, log and pass up a defined one
    XB_Log::log "warning", "XB_Utils::sysctl_read: caught unexpected " .
      "exception $@";
  }
  # pass defined exceptions up to caller
  die "sysctl_read";
}




# Description: 
#	Write value $val into sysctl variable $var.
# Arguments:
#	$var	sysctl variable to write
#	$val	value to write
# Returns:
#	$val on success
# Exceptions:
#	"sysctl_write" on failure
sub sysctl_write ($$)
{
  my ($var, $val) = @_;
  XB_Log::log "info", "-> XB_Utils::sysctl_write $var, $val";
  eval {
    # safe pipe open to sysctl (using user args)
    my $cpid = open PIPE, "-|";
    if($cpid == 0) {
      # child, redirect stderr to taka-tuka land
      open STDERR, ">/dev/null";
      my @cmd = ("sysctl", "-wn", "$var=$val");
      exec @cmd or XB_Log::log "err", "@cmd failed: $!" and exit;
    }
    # parent
    if(not defined $cpid) { XB_Log::log "err", "cannot fork: $!" and die "open"; }
    # if we don't read the output, we cannot close the pipe
    my @output = <PIPE>;
    # done with pipe
    close PIPE or not $! or XB_Log::log "err", "cannot close pipe: $!" 
      and die "close";
    # reap zombie
    waitpid $cpid, 0 == $cpid or XB_Log::log "err", "waitpid $cpid, 0: $!" 
      and die "waitpid";
    # check return value of child (always -1? thus commented out)
    # $? == 0 or warn $? and die "value";
    
  };
  # print trace line
  XB_Log::log "info", "<- XB_Utils::sysctl_write $var, $val";
  return $val unless $@; # success if no exception
  # exception handling
  if($@ =~ /^(open|close|waitpid|value)/) {
    # known exception, nothing to handle, pass up
  } else {
    # unknown exception caught, log and pass up a defined one
    XB_Log::log "warning", 
      "XB_Utils::sysctl_write: caught unexpected exception $@";
  }
  # pass defined exceptions up to caller
  die "sysctl_write";
}




sub rand_seed ()
{
  # init random number generator in a portable and relatively secure fashion
  my ($rand_seed, $rand_src) = (undef, undef);
  # try to find a "good" random source"
  foreach my $s ("/dev/urandom", "/dev/random") { 
    if(-e $s) { $rand_src = $s; last; } 
  }
  if(defined ($rand_src) and -e $rand_src) {
    # found good random source, now get a seed
    open RAND, $rand_src 
      or XB_Log::log "warning", "cannot open $rand_src: $!, continuing";
    if(read(RAND, $rand_seed, 4) == 4) { 
      # transmogrify into a string
      $rand_seed = unpack "L", $rand_seed;
    } else {
      # warn and discard value
      XB_Log::log "warning", 
        "cannot read 4 random bytes from $rand_src: $!, continuing";
      $rand_seed = undef;
    }
    close RAND #or not $! 
      or XB_Log::log "warning", "cannot close $rand_src: $!, continuing";
  }
  else
  {
    # we have to use a somewhat compromised seed value
    XB_Log::log "warning",
      "cannot find good random source, generated keys will be weak";
    $rand_seed = time ^ ($$ + ($$ << 15));
  }
  return $rand_seed;
}




##############################################################################
# XB_is_xbone_address($)
# Input: (address)
#       address: IP address to test
# Output: (match)
#       match: 1 is it is an Xbone address, 0 if not.
##############################################################################
sub XB_is_xbone_address($)
{
    use Net::Netmask;

    my ($address) = @_;

    my $eblock = new Net::Netmask($XB_Defs::XB_IP_EXTERNAL_BLOCK);
    my $iblock = new Net::Netmask($XB_Defs::XB_IP_INTERNAL_BLOCK);

    if ($eblock->match($address)) {
      return 1;
    } elsif ($iblock->match($address)) {
      return 1;
    } else {
      return 0;
    }
 
}




##############################################################################
# XB_ip_base()
# Input: NONE
# Output: (base)
#       base: base address of the external block allocated for
#             Xbone use
##############################################################################
sub XB_ip_base()
{
    use Net::Netmask;

    my $eblock = new Net::Netmask($XB_Defs::XB_IP_EXTERNAL_BLOCK);
    return $eblock->base();
}



 
##############################################################################
# XB_ip_mask()
# Input: NONE
# Output: (mask)
#       mask: mask of the external block allocated for Xbone use
##############################################################################
sub XB_ip_mask()
{
    use Net::Netmask;

    my $eblock = new Net::Netmask($XB_Defs::XB_IP_EXTERNAL_BLOCK);
    return $eblock->mask();
}




sub XB_obsolete ()
{
    my $seconds;
    my ($day, $month, $year) =
	split /\//, $XB_Defs::XBONE_VERSION_DATE;

    $seconds = timelocal (0, 0, 0, $day, $month-1, $year-1900);

    if ((time() - $seconds) >= (2 * $XB_Defs::XBONE_OBSOLETE_LIMIT))
    {
	return -1;
    }
    elsif ((time() - $seconds) >= $XB_Defs::XBONE_OBSOLETE_LIMIT)
    {
	return 0;
    }

    return 1;
}




sub XB_banner_msg ()
{
    my ($datestr, $msg);
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime();

  $msg = "\n\n#######################################\n";
  $msg .= "#######################################\n";
  $msg .= "STARTUP: $0 @ARGV\n";
  $msg .= "Release version: $XB_Defs::XBONE_RELEASE\n";
  $msg .= "Protocol version: $XB_Defs::XBONE_PROTOCOL\n";

    $datestr = "$hour\:$min\:$sec ";
    $datestr .= ("Sun", "Mon", "Tues", "Wednes", "Thurs", "Fri", "Satur")[$wday];
    $datestr .= "day " . ("Jan", "Feb", "Mar", "Apr", "May",
			 "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec")[$mon];
    $year += 1900;
    $datestr .= " $mday, $year";
    $msg .= "$datestr\n";

  $msg .= "#######################################\n";
  $msg .= "#######################################\n\n\n";
  XB_Log::log ("warning", "$msg");
}




1;				# Make sure this file evaluates to TRUE.
