# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
# 
#
# ----------------------------------------------------------------------------
#  $RCSfile: XB_MIB.pm,v $
#
# $Revision: 1.7 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:34 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Author: Anindo Banerjea


##################################################################
##
## Xbone MIB and API Routines
## ==========================
##
##
## API TEXT TO INTERAL HASH FUNCTIONS
## ----------------------------------
## o @textlines = @xb_generic_input($parameter)
##   RETURNS consistent API Text format.
##   $parameter can be:
##    - filename
##    - single string with internal newlines (i.e. from web)
##    - list of lines
##
##   Interprets:
##    - trailing "\" as continuation (concated in return value)
##    - whitespace inside of keys/values (e.g. Name="Joe Touch")
##
## o xb_line2hash($string)
##   RETURNS \%hash from single API line/string.
##
## o xb_text2hash_index(\%keyhash, @strings)
##   RETURNS \%hash{}{} from list of API text strings
##   - Uses \%keyhash to deal with multiple occurances,
##     e.g. if multiple occurances of XboneAclList is expected,
##
##          %keyhash = (XboneAclList, UserAuth);
##
##     Will cause all API lines "XboneAclList" to be put into
##     the returned hash arrays index by the value of the
##     "UserAuth" field:
##
##       %hash{XboneAclList}{1} = <user_auth_field>
##       %hash{user_auth_field} = <hash ref of attribute value pairs>
##
## o       xb_text2hash_new()    ## old version (to be deleted)
## o       xb_text2hash()        ## old version (to be deleted)
##
##

package XB_MIB;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(xb_generic_input xb_text2hash_index xb_line2hash hash2text);
@EXPORT_OK = qw();

use strict;
use sigtrap;

use XB_Log;

##################################################################
## xb_generic_input preprocesses text command strings, from
## one of the following three valid formats, to a common format
## as an array of strings, each one line long.
## 3 Formats for Text Command Strings
##   -- input file name
##      (read in as @lines list)
##   -- single string with internal "\n"
##      (split on "\n")
##   -- @lines list of strings
##      (process as is)
##
## Side effect: strip out continue marker ("\") and concat lines
## RETURNS(@list)
##############################################################
sub xb_generic_input (@)
{
   my(@instuff)  = @_;
   my(@outstuff) = ();
   my($a, $bb, $newline, $num);

   chomp($a = shift(@instuff));

   ########################################################
   # If the input string does not contain a newline and if
   # there is a file with that name in this directory and
   # if there are no more inputs in the array
   ########################################################
   if ((($a !~ /\n/)) && (-e $a) && (@instuff == 0)) {
      XB_Log::log "debug4", "xb_generic_input() FILE\n";
      open(INFILED, $a);
      push(@outstuff, <INFILED>);
      close(INFILED) or XB_Log::log "warning", "$!: Close failed\n";
      goto returnit;
   }

   if (($a =~ /\n/) && (@instuff == 0)) {
      XB_Log::log "debug4", "xb_generic_input() LONG-STRING\n";
      push(@outstuff, map $_ .= "\n", split('\n', $a));
      goto returnit;
   }

   XB_Log::log "debug4", "xb_generic_input() LIST $#instuff\n";
   push(@outstuff, "$a\n", @instuff);

returnit:

   @instuff = ();
   $newline="";

   while (@outstuff > 0) {
      $bb = shift(@outstuff);
      $bb =~ s/^(\s+)//;
      if ($bb =~ s/^(\s*)(.*\w)(\s*\\\s*)$/$2 /) {
         chomp $bb;
         $newline .= $bb;
      } else {
         chomp($bb);
         $newline .= $bb;
         push(@instuff, "$newline\n");
         $newline="";
      }
   }

   $num = @instuff;
   XB_Log::log "debug4", "xb_generic_input returning $num lines\n";
   return(@instuff);
}# xb_generic_input();

##################################################################
# xb_text2hash_index takes a keyref (list of parameters to be
# given special treatment) and an array of strings (each one line long)
# and puts then into hash structure. Each line is of the form
# kword key=value key=value key=value\n
# The resulting hash structure can be indexed with
# $hash{kword}{key} to get value.
# E.g., if the original line in the array was
# XboneCreator Name="Joe Touch" Email=touch@isi.edu
# then $hash{XboneCreator}{Name} contains "Joe Touch"
# in case kword is repeated in the original array of strings: e.g.,
# XboneNode ID=mul Class=host
# XboneNode ID=sub Class=router NodeCount=1
# Then $hash{XboneCreator} is an hash with the keys 0, 1, ...
# $hash{XboneCreator}{0} gives "mul"
# and $hash{mul}{key} contains the value
# i.e., $hash{mul}{Class} would be "host"
###############################################################
 
sub xb_text2hash_index ($@)
{
  my($keyref, @lines) = @_;
  my ($a, $kword, @parms, @attribs, $attrib, $key, $value, $href);
  my(%ahash, $tmpcnt);
  my ($cmdcnt, $authcnt, $nodecnt) = (1, 1, 1);
 
  while (@lines > 0) {
    chomp($a = shift(@lines));
    ($kword, $href) = &xb_line2hash($a);
    next unless ($kword);
 
    if ($keyref && ($value = $$keyref{$kword})) {
       $tmpcnt = 1;
       while (defined($$href{$value}{$tmpcnt})) {
          $tmpcnt++;
       }
       $ahash{$kword}{$tmpcnt} = $value;
       $ahash{$value} = $href;
    }
 
 
    if ($kword eq "XboneNodeCommand") {
      $ahash{$kword}{$cmdcnt++} = "$a";
      next;
    }
 
    if ($kword eq "XboneAclList") {
      $value = $$href{SrcIpAuth} unless ($value = $$href{UserAuth});
      $ahash{$value} = $href;
      $ahash{$kword}{$authcnt++} = $value;
      next;
    }
 
    if ($kword eq "XboneNode") {
      $value = $kword unless ($value = $$href{ID});
      $ahash{$value} = $href;
      $ahash{$kword}{$nodecnt++} = $value unless ($kword eq $value);
      next;
    }
 
    $ahash{$kword} = $href;
  }
 
  return(\%ahash);
}

##################################################################
# xb_line2hash take a string of the form
# kword key=value key=value key=value
# and returns it as a pair (kword, hash)
##################################################################
sub xb_line2hash ($)
{
  my($a) = @_;
 
  my ($kword, @parms, @attribs, $attrib, $key, $value, $tmp);
  my(%ahash) = ();
 
  chomp($a);
  return(0, \%ahash) unless ($a =~ /\S/);
  return(0, \%ahash) if ($a =~ /^\s*#/);
 
  ($kword, @attribs) = split(' ', $a);
  while (@attribs > 0) {
    $attrib = shift(@attribs);
    ($key, $value) = split('=', $attrib);
    if (not defined $value) {
        $value = "DEFINED";
    } elsif ($value =~ /^\"/) {
        do {
            $value .= " " . shift(@attribs);
        } until ($value =~ /^\".*\"/)
    }
    $ahash{$key} = $value;
    XB_Log::log "debug5", "$kword : $key : $value :\n";
  }
  XB_Log::log "debug5", "\n";
 
  return($kword, \%ahash);
}

sub hash2text ($)
{
    my ($hashref) = @_;
    my $return_string = "Xbone Version=" . $hashref->{Xbone}->{Version} . "\n";
    $return_string .= "XboneControl Command=" .
			$hashref->{XboneControl}->{Command} .
			" Transaction=". $hashref->{XboneControl}->{Transaction}
			.  "\n";
    $return_string .= "XboneOverlay" .
			" Name=" .  $hashref->{XboneOverlay}->{Name} .
		" SearchRadius=" .  $hashref->{XboneOverlay}->{SearchRadius} .
			"\n";
    $return_string .= "XboneNetwork" .
			" Topology=" .  $hashref->{XboneNetwork}->{Topology} .
			"\n";
    $return_string .= "XboneCreator" .
			" Name=" .  $hashref->{XboneCreator}->{Name} .
			" Email=" .  $hashref->{XboneCreator}->{Email} .
			"\n";
    $return_string .= "XboneSecurity" .
			" UserId=" .  $hashref->{XboneSecurity}->{UserId} .
			" AuthType=" .  $hashref->{XboneSecurity}->{AuthType} .
			"\n";
    foreach my $key (keys %{$hashref->{XboneNode}}) {
	my $name = $hashref->{XboneNode}{$key};
	$return_string .= "XboneNode";
	foreach my $key2  (keys %{$hashref->{$name}}) {
	     unless ($hashref->{$name}->{$key2} =~ /DEFINED/i) {
		 $return_string .= " $key2=" . 
				 $hashref->{$name}->{$key2};
	     }
	}
	foreach my $key2  (keys %{$hashref->{$name}}) {
	     if ($hashref->{$name}->{$key2} =~ /DEFINED/i) {
		 $return_string .= " $key2";
	     }
	}
	$return_string .= "\n";
    }
    $return_string .= "XboneEOC\n";
    return $return_string;
}

1;
