# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
# 
#
# ----------------------------------------------------------------------------
#  $RCSfile: XB_OM_App_Start.pm,v $
#
# $Revision: 1.7 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:34 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
#
# Primary Author: Oscar Ardaiz

package	XB_OM_App_Start;

require 	Exporter;
@ISA 		= qw(Exporter);
@EXPORT 	= qw();
@EXPORT_OK	= qw(appOvlNodeCommands appDeleteOvlNodeCommands);

# XXX oscar's code does not work with these! fix.
# use strict;
# use sigtrap;

########################################################################
#sub appOvlNodeCommands
# input:
# $ovlNode is the ovl node name
# $ovlNodeOS is the OS of the node
# $ovlNodeNotSupportsApps is true if the node can not run applications
# \@installedPackages is the aeslications 
# $app is the type of app
# \%ovlTopology is a hash which contains next topology information fields
#   Vertices is the array of node_domain_name-[virtual_address array] pair 
#   Type is either "star","line","ring","mesh"
#   Structure is hash of routers and host nodes
#   that represents diferent things for each topology type,
#       for a star, the router value is  the star center node,
#       for a ring, routers are the nodes order of the ring, 
#        hosts are link to each router
#       for a line, it is the order of the line, starting by one end
#       for a mesh, all nodes, unordered
# output:
# returns $cmdmsg, which contains: 
# - noe one or various install command (depending on whether we know the app is installed and we know the os of the node,
# "XboneNodeCommand application install -app_name xxxx -app_version zzzz -app_url yyyyyy [-app_package_os ooooo]\n"
# - one execution command,
# "XboneNodeCommand application execute -app_name xxxx -app_id iiiii -app_binding_to iiiii [-app_neighbour_type_1 pppp] [-app_neighbour_type_2 sssss] \n"
#######################################################################

$binarylocationfile="../appovlmgr/BinaryLocations";

sub appOvlNodeCommands (@)
{
    my ($ovlNode,$ovlNodeOS,$ovlNodeSupportsApps,$ovlNodeInstalledPackagesref,$app,$ovlTopologyref) = @_;
    my $cmdmsg="";

    $ovlVerticesref = ${$ovlTopologyref}{Vertices};
    $topologyType=${$ovlTopologyref}{Type};
    $topologyStructureref=${$ovlTopologyref}{Structure};

    #if it does not support apps return,

    if ($ovlNodeSupportsApps=~/^no$/)
    { return $cmdmsg; }

    #first we need to findout if it is installed and install it if necesary,
    #if it is not installed, it is not installed the latest version, 
    #or we dont know what is installed.


    #if we know the OS, we will findout if it is installed or not, and if
    # it is not installed the latest version, or if we dont know what is
    # installed,

    ($app_package_versionref,$app_package_urlref,$app_package_osref)=binaryLocation($ovlNodeOS,$app);

    $notinstalled=1;
    if ($$app_package_osref[0]=~/^$ovlNodeOS$/)
    {
	foreach $_ (@$ovlNodeInstalledPackagesref)
        {
          if ($_=~/^$app-$$app_package_versionref[0]$/) {$notinstalled=0;}
        }

        if ($notinstalled)
        {
          $cmdmsg.="XboneNodeCommand application install $app $$app_package_versionref[0] $$app_package_urlref[0]\n";
	}	
    }

    # if we dont know which os the system has we send the URLs of packages
    # for all os we know, even if it said it has this application
    # installed, since we can not know if this is the last version.

    else
    {
	$i=0;
	foreach (@$app_package_osref)
        {
           $cmdmsg.="XboneNodeCommand application install $app $$app_package_versionref[$i] $$app_package_urlref[$i] $$app_package_osref[$i]\n";
           $i=+1;						
	}	
    }

    # now we will create the commannode name ( or IP)te an instance of this
    # application in that overlay, # we shall identify each
    # application instance by the IP address to which it binds to

    $cmdmsg.="XboneNodeCommand application execute $app $ovlNode";

    # and we tell the application to which IP address it has to bind to.

    $cmdmsg.=" $ovlNode";

    #finding out to which other application instance it is connected,

# the protocol now is application independant, the rd will know that
# if the app=squid, then appNeighbourType1 means a parent, and Type2 a
# sibling. the appNeighbour function is application dependant, it says
# whose neighbours a node has and whose their role based on the
# application.

    ($appNeighbourType1ref, $appNeighbourType2ref) =
            appNeighbours ($ovlNode, $ovlVerticesref, $topologyType, $topologyStructureref, $app);
	
    foreach (@$appNeighbourType2ref)
    {
	$cmdmsg.=" -app_neighbour_type_1 $_";
    }

    foreach (@$appNeighbourType1ref)
    {
	$cmdmsg.=" -app_neighbour_type_2 $_";
    }

    $cmdmsg.="\n";
    return $cmdmsg;
}

	
########################################################################
#sub appNeighbours
#input:
# $ovlNode 
# \@ovlVertices is the array of node_domain_names 
# $topologyType is either "star","line","ring","mesh"
# \%topologyStructure is hash of routers and host nodes
# that represents diferent things for each topology type,
# for a star, the router value is  the star center node,
# for a ring, routers are the nodes order of the ring, 
# hosts are link to each router
# for a line, it is the order of the line, starting by one end
# for a mesh, all nodes, unordered
#output:
#\@appNeighboursType1 if squid are siblings, for other app other things
#\@appNeighboursType2 if squid are parents, for other apps other things.
#######################################################################

sub appNeighbours {
	my($ovlNode,$ovlVerticesref,$topologyType,$topologyStructureref,$app)=@_;
	@appNeighboursType1 =();
	@appNeighboursType2 =();
if ($topologyType=~/^star$/) {
	if ($topologyStructureref->{Routers}[0]=~/^$ovlNode$/) {
	}
	else {
		if ($app=~/^squid$/){
		$appNeighboursType2[0] = $topologyStructureref->{Routers}[0];
			}
		}
}
elsif ($topologyType=~/^ring$/) {
	$i=0;
	while ($topologyStructureref->{Routers}[$i]){
		if ($topologyStructureref->{Routers}[$i]=~/^$ovlNode$/) {
			$nodePosition=$i;
		 }
		$i+=1;
	}
	$last=$i-1;
	if ($nodePosition=~/^0$/)	{
		$appNeighboursType1[0] =$topologyStructureref->{Routers}[1];
		$appNeighboursType1[1] =$topologyStructureref->{Routers}[$last];

		}
	elsif (($nodePosition)=~/^$last$/)	{
		$appNeighboursType1[0]=$topologyStructureref->{Routers}[($last-1)];
		$appNeighboursType1[1] = $topologyStructureref->{Routers}[0];

		}
	else {
		$appNeighboursType1[0]=$topologyStructureref->{Routers}[$nodePosition-1];		$appNeighboursType1[1]=$topologyStructureref->{Routers}[$nodePosition+1];	
	}
	
}
elsif ($topologyType=~/^line$/) {
	$i=0;
	while ($topologyStructureref->{Routers}[$i]){
		if ($topologyStructureref->{Routers}[$i]=~/^$ovlNode$/) {
			$nodePosition=$i;
		 }
		$i+=1;
	}
	$last=$i-1;
	if ($nodePosition=~/^0$/)	{
		$appNeighboursType1[0]=$topologyStructureref->{Routers}[1];
	}
	elsif (($nodePosition)=~/^$last$/)	{
		$appNeighboursType1[0]=$topologyStructureref->{Routers}[($last-1)];
	}
	else {
		$appNeighboursType1[0]=$topologyStructureref->{Routers}[$nodePosition-1];			
		$appNeighboursType1[1]=$topologyStructureref->{Routers}[$nodePosition+1];	
	}
}
elsif ($topologyType=~/^mesh$/) {
		$i=0;
		$j=0;
	while ($topologyStructureref->{Routers}[$i]){
		if ($topologyStructureref->{Routers}[$i]=~/^$ovlNode$/) {
			}
		else   {
			$appNeighboursType1[$j]=$topologyStructureref->{Routers}[$i];		
			$j+=1;
			}
		$i+=1;
		}
}
return(	\@appNeighboursType1,\@appNeighboursType2);
}



########################################################################
#sub binaryLocation
#######################################################################

sub binaryLocation {
	my ($OS,$app)=@_;
	@binarylocation=();
	open(BINARYLOCATIONFILE,"<$binarylocationfile") ||
   die "Can't open binarylocationfile $!";
	$i=0;
	while (<BINARYLOCATIONFILE>) {
		$file[$i]=$_;
		$i+=1;
		}
	close BINARYLOCATIONFILE;

	@binarylocationforapp=grep {/$app/} @file;
	@binarylocationforOS=grep {/$OS/} @binarylocationforapp;
      # if some matched found, or $OS=""
	if ($#binarylocationforOS>-1) {
	@binarylocation= @binarylocationforOS;
	  }
	else {
      #if no match return all
	@binarylocation= @binarylocationforapp;
		}
    	$i=0;
   	@app_package_version=();
	@app_package_url=();
	@app_package_os=();
	foreach (@binarylocation) {
		($app,$app_package_version[$i],$app_package_url[$i],$app_package_os[$i])=split;
		$i+=1;
		}
	return (\@app_package_version,\@app_package_url,\@app_package_os);

}

########################################################################
#sub appDeleteOvlNodeCommands
# input:
# $ovlNode is the ovl node name
# $app is the type of app
# output:
# returns $cmdmsg, which contains: 
# -one kill command,
# "XboneNodeCommand application kill -app_name xxxx -app_id iiiii\n"
# - none or one deinstall command 
# "XboneNodeCommand application deinstall -app_name xxxx\n"
#######################################################################

sub appDeleteOvlNodeCommands(@) {
	my ($ovlNode,$app)= @_;
	my $cmdmsg="";

	$cmdmsg.="XboneNodeCommand application kill $app $ovlNode\n";

	$cmdmsg.="XboneNodeCommand application deinstall $app\n";

	return $cmdmsg;
}



1;
