# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
# 
#
# ----------------------------------------------------------------------------
#  $RCSfile: XB_Ping.pm,v $
#
# $Revision: 1.15 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:34 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Author: Greg Finn

################################################
#
# Timer related routines used by Overlay Manager
#
# Principal author: Gregory Finn
#
################################################



package XB_Ping;


require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(XB_ping_overlays);
@EXPORT_OK = qw();

use strict;
use sigtrap;

use Socket;
use FileHandle;
use Data::Dumper;

use XB_Defs;
use XB_Log;








#######################################################################
# XB_ping_overlays ( ohptr, oaptr, oallptr )
# 
# Input: reference to the %overlay_index hash, reference to the
#        @overlay_array, reference to the @overlay_allocation array
# Output: none
#
# Side effect: Resets the Resource Daemon soft-state timeout for each
# host in all current overlays.
#
# This procedure can be called without regard to whether or not
# another process is manipulating the underlying overlay data
# structures.  This procedure does not alter any of those underlying
# data structures.
#
#######################################################################

sub XB_ping_overlays ($$$)
{
    my ($ohptr, $oaptr, $oallptr) = @_;
    my $procname = "XB_ping_overlays";
    my $ovlname;

    XB_Log::log "info", "-> $procname: @_\n";

#    {
#	my $d = Data::Dumper->new([$ohptr], [qw(overlay_index)]);
#	print $d->Dump;
#    }
#    {
#	my $d = Data::Dumper->new([$oaptr], [qw(overlay_array)]);
#	print $d->Dump;
#    }
#    {
#	my $d = Data::Dumper->new([$oallptr], [qw(overlay_allocation)]);
#	print $d->Dump;
#    }


    foreach $ovlname (keys %$ohptr)
    {
      XB_Log::log "debug0",
	"overlay name: $ovlname, overlay index=$ohptr->{$ovlname}\n";

	XB_ping_overlay ($ovlname, $ohptr->{$ovlname}, $oaptr, $oallptr);
    }

    XB_Log::log "info", "<- $procname:\n";

    return;
}





#######################################################################
# XB_ping_overlay ( ovlname, oindex, oaptr, oallptr )
# 
# Input: overlay name, index for that overlay, reference to the
#        overlay_array, reference to the overlay_allocation array
# Output: none
#
# Side effect: Resets the Resource Daemon soft-state timeout for each
# host in the named overlay.
#######################################################################

sub XB_ping_overlay ($$$$)
{
    my ($ovlname, $ovlindex, $oaptr, $oallptr) = @_;
    my $procname = "XB_ping_overlay";
    my ($hostidx, $nodehptr, $oahptr, $oallhptr, $sechptr, $addrhptr);
    my ($hname);
    my ($release);

    XB_Log::log "info", "-> $procname: @_\n";

    if (!defined ($oaptr->[$ovlindex]))
    {
      XB_Log::log "err",
	"Overlay: $ovlname missing its hash in overlay_array\n";

	return;
    };
    $oahptr = $oaptr->[$ovlindex];

    if (!defined ($oallptr->[$ovlindex]))
    {
      XB_Log::log "err",
	"Overlay: $ovlname missing its hash in overlay_allocation\n";

	return;
    };
    $oallhptr = $oallptr->[$ovlindex];

    if (!defined ($oahptr->{'XboneNode'}))
    {	
      XB_Log::log "err",
	"Overlay: XboneNode missing in overlay_array hash\n";

      return;
    };
    $nodehptr = $oahptr->{'XboneNode'};
    
    if (!defined ($oallhptr->{'address'}))
    {	
      XB_Log::log "err",
	"Overlay: address hash missing in overlay_allocation hash\n";

      return;
    };
    $addrhptr = $oallhptr->{'address'};
    
    if (!defined ($oahptr->{'XboneSecurity'}))
    {	
      XB_Log::log "err",
	"Overlay: XboneSecurity missing in overlay_array hash\n";

      return;
    };
    $sechptr = $oahptr->{'XboneSecurity'};

    $release = $oahptr->{'XboneOverlay'}{'Release'};


    ################################################
    # We need to ping the hosts in the overlay.  To
    # find their original fully qualified host names
    # we match the partial hostnames in the overlay
    # against all the hosts at replied to the
    # multicast enquiry.  Those matching are in the
    # overlay.
    ################################################

    foreach $hname (keys (%$addrhptr))
    {
	foreach $hostidx (keys (%$nodehptr))
	{
	    @_ = split /\./, $nodehptr->{$hostidx};

	    if ($hname =~ /$_[0]/)
	    {
		XB_ping_host ($ovlname, $nodehptr->{$hostidx}, $sechptr, 
			      $release);
	    }
	}
    }
    
    XB_Log::log "info", "<- $procname:\n";

    return;
}





#######################################################################
# XB_ping_host ( ovlname, hostaddr, sechptr, release)
# 
# Input: overlay name, address of host in that overlay, pointer to
#        XboneSecurity hash in overlay_array, release of that node
# Output: none
#
# Side effect: Resets the Resource Daemon soft-state timeout for this
# host and this overlay.
#######################################################################

sub XB_ping_host ($$$$)
{
    my ($ovlname, $hostaddr, $sechptr, $release) = @_;
    my $procname = "XB_ping_host";
    my $header_string = "Xbone Version=$XB_Defs::XBONE_PROTOCOL ".
	"Release=$release\nXboneControl Command=OverlayStatus ".
	    "Transaction=Request\n";
    my ($hbmsg, $sock, $inaddr, $paddr);


    XB_Log::log "info", "-> $procname: @_\n";

    #############################
    # Build the heartbeat message
    #############################

    $hbmsg = "$header_string" . "XboneSecurity ";
    $hbmsg .= "AuthType=$sechptr->{'AuthType'} ";
    $hbmsg .= "UserId=$sechptr->{'UserId'}\n";

    $hbmsg .= "XboneOverlay Name=$ovlname ";
    $hbmsg .= "OverlayManager=$XB_Defs::XBONE_OVERLAY_MANAGER\n";

    $hbmsg .= "XboneNodeCommand heartbeat\n";

    $hbmsg .= "XboneEOC\n";

    #############################
    # Send it as a UDP message
    #############################

    unless ($sock =
	  IO::Socket::INET->new ( Proto => "udp",
				  LocalAddr => $XB_Defs::XBONE_OVERLAY_MANAGER,
				  PeerPort => $XB_Defs::MCAST_PORT,
				  PeerAddr => $hostaddr,
				  Type => SOCK_DGRAM
				 )
	    )
    {
        XB_Log::log "err", "$procname: IO::Socket::INET->new failed\n";
        goto EXIT;
    }

    $sock->autoflush(1);

    $inaddr = inet_aton ($hostaddr);
    $paddr = sockaddr_in ($XB_Defs::MCAST_PORT, $inaddr);

    XB_Log::log "debug0", "Pinging host: $hostaddr\n";
    XB_Log::log "debug0", "Sending heartbeat msg:\n$hbmsg\n";

    unless ($sock->send ($hbmsg, 0, $paddr))
    {
	XB_Log::log "err", "$procname: Cannot send heartbeat message: $!.\n";
	goto EXIT
    }

    $sock->close;

EXIT:

    XB_Log::log "info", "<- $procname \n";
    return;
}



1;                            # Insure TRUE return if module is interpreted.















