# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# ----------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
# 
#  $RCSfile: xb-overlay-manager.pl,v $
#
# $Revision: 1.39 $
#   $Author: yushunwa $
#     $Date: 2000/10/27 23:41:56 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
#
# Primary Author: Anindo Banerjea
#
#########################################################
#
# Start PERL on any system where perl is in the path
# This is a modified version of the "start perl" script
# provided in the PERL man pages, which starts perl 
# on the remainder of the file regardless of whether it
# is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the
# perl path, rather than requiring it be hard-coded.
#
# setenv PERLDB_OPTS "N f=26"
#
# Must be set before perl starts!
#
# N 	: Nonstop (noninteractive)
# f=26	: frame=26 = 16 + 8 + 2,
#		2 = entry and exit,
#		!4 = don't print args to functions
#		8= enabled overloaded stringify and tied FETCH
#		16= print return values from subroutines
#
############################################################
# the following line finds perl and runs it, in sh, csh, or perl:
#
# note - '-T' (TAINT) switch is not included, because 'which' often
# returns a version of perl that isn't secure. don't worry about it.
#
eval '(exit $?0)' && eval 'PERLDB_OPTS="N f=26";export PERLDB_OPTS;PERL=`which perl5`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERLDB_OPTS "N f=26"; setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

############################################################
# PERL CODE STARTS HERE
############################################################

# XBONE code to set libraries
BEGIN {
  use strict;
  use sigtrap;
  use FindBin;
  delete $ENV{PATH};

  # taint-safe backticks equivalent
  my $pid = open KID, "-|";
  unless(defined $pid) { die "open: $!"; }
  unless($pid) { 
    foreach my $uname ("/usr/bin/uname", "/bin/uname") {
      if(-x $uname) { exec $uname, "-msr" or die "exec: $!"; }
    }
    die "cannot find uname"; 
  }
  my ($osname, $osvers, $arch) = split ' ', lc <KID>;
  close KID or die "close: $!";
  $osvers =~ s/^(\d+\.\d+).*/$1/;

  if($osname eq "sunos") {
    # bang Solaris uname returns into shape for perl usage
    ($osname, $arch) = ("solaris", "sun4");
    $osvers =~ s/5\.(\d+)/2.$1/;
  } elsif($osname eq "linux") {
    # same for Linux
    if($arch eq "i686") { $arch =~ s/i\d(\d+)/i3$1/; }
  } elsif($osname eq "freebsd") {
    if($osvers =~ /cairn/) { $osvers = "2.2"; }
  }

  my $ostag = join "-", ($arch, $osname, $osvers);
  print "Configuring perl for $ostag...\n" if 0;

  my $sdir = "/home/xbone/perl";
  my $ldir = $FindBin::RealBin;
  foreach my $p ($ldir, "$ldir/../lib", "$ldir/../../lib", 
		 "$ldir/../../../lib", 
		 "$sdir/$ostag", "$sdir/$ostag/$arch-$osname") {
    if(-d $p) { unshift @INC, $p; }
  }

  # untaint the resulting include path so "use" works
  foreach my $i (@INC) { if($i =~ /(.*)/) { $i = $1;} }
  
  printf STDERR "\n%s\n\n", join "\n", @INC if 0;
};

##############################################################################
# END OF XBONE PREFIX CODE - PUT YOUR PERL CODE BELOW                        #
##############################################################################

#-------------------------------------------------
# X-Bone: Read overlay manager command-line args.
#-------------------------------------------------
BEGIN {

  use strict;
  use sigtrap;
  use XB_Defs;

  my @usage=(
    "\nX-Bone Overlay Manager (OM) usage:\n",
    "% ./xb-overlay-manager.pl [EXT IP BLK] [INT IP BLK] [SUBNET BITS]\n\n",
    "  [EXT IP BLK] : External IP address blocks for overlays\n",
    "                 Format: *.*.*.0/24 (Ex. 172.27.211.0/24).\n",
    "  [INT IP BLK] : Internal IP address blocks for overlays\n",
    "                 Same format as [EXT IP BLK].\n",
    "  [SUBNET BITS]: Length of subnet bits used in overlays\n",
    "                 Values: 1-6, 3 recommended.\n",
    "\n",
    "Please beware that the IP address blocks chosen SHOULD NOT conflict\n",
    "with any existing addresses used in the environment in which X-Bone\n",
    "Overlays will be deployed. Failuer to do so will cause the routing\n",
    "tables of the X-Bone nodes be incorrectly overwritten!\n\n",
  );

  my ($ext_blk, $int_blk, $sub_len);
  my $error=0;

  if($#ARGV==-1) {
    print "\nWarning!! You did not specify any command-line arguments. \n";
    print "Please read the following usage info of the XBone overlay manager.\n";
    print "The overlay manager will now run with the pre-defined IP address \n";
    print "blocks specified below:\n";
    print "    - External: $XB_Defs::XB_IP_EXTERNAL_BLOCK\n";
    print "    - Internal: $XB_Defs::XB_IP_INTERNAL_BLOCK\n";
    print "    - Overlay Subnet Length = $XB_Defs::XB_OVL_SUBNET_BITS bits\n";
    print "The default IP address blocks comform with IANA's specification for\n";
    print "IP addresses used in private Internets (RFC 1918). \n\n";
    print "If you would like to specify your own IP address blocks for the\n";
    print "overlays, please stop this Overlay Manager (CTRL-c or kill), and\n";
    print "restart it with the following command line format:\n";
    print @usage;
  }else{
    if($#ARGV != 2) { $error=1; }
    else{
      unless($ARGV[0] =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)\/(\d+)/){ $error=1;}
      unless(($1<256) && ($2<256) && ($3<256) && ($4<256) && ($5<30) && ($5>0))
      { $error=1;}
      my $ebits = $5;
      unless($ARGV[1] =~ /(\d+)\.(\d+)\.(\d+)\.(\d+)\/(\d+)/){ $error=1;}
      unless(($1<256) && ($2<256) && ($3<256) && ($4<256) && ($5<30) && ($5>0))
      { $error=1;}
      unless($ARGV[2] < 30 && $ARGV[2]>0){ $error=1; }
      my $subnet_bits = $ARGV[2];
      if ($XB_Defs::IP_ADDRESS_LENGTH - $ebits - $subnet_bits < 2)
      {
	  $error = 1;
	  print "Invalid bit combinations specified.\n";
      }
    }
    if($error == 1){
      print "\nError!! Wrong command-line argument format!\n";
      print "You need to specify ALL 3 command-line arguments with the\n";
      print "format shown below.\n";
      print @usage;
      die "Wrong command-line argument format! ==>xb-om::read_cmd_line_args\n";
    } else {
      $XB_Defs::XB_IP_EXTERNAL_BLOCK=$ARGV[0];
      $XB_Defs::XB_IP_INTERNAL_BLOCK=$ARGV[1];
      $XB_Defs::XB_OVL_SUBNET_BITS=$ARGV[2];
    }
  }

  print "External IP block= ", $XB_Defs::XB_IP_EXTERNAL_BLOCK, "\n";
  print "Internal IP block= ", $XB_Defs::XB_IP_INTERNAL_BLOCK, "\n";
  print "Subnet Mask length=", $XB_Defs::XB_OVL_SUBNET_BITS, "\n";
  
  if ($XB_Defs::DEMO_MODE == 1)
      {
	  print "\nOverlay Manger is running in DEMO mode.\n";
	  print "Read XB_Defs.pm for more on DEMO mode.\n";
      }
};

##############################
# END OF XBONE PREFIX CODE - 
# PUT YOUR PERL CODE BELOW
#
# The following just prints out
# some status info and checks
# the arglist.
##############################

use strict;
use sigtrap;

use POSIX "sys_wait_h";
use IO::Socket;
use Data::Dumper;
use Config;

use XB_MIB;
use XB_Defs;
use XB_RPC;
use XB_IP_Allocation;
use XB_Ping;
use XB_Topology;
use XB_OM_App_Start;
use XB_IPsec;
use XB_Log;
use XB_Utils;

#####################################################################
# This is for compatibility with the old code (Steve's version)
# while Greg's version is maintained as a separate tree. old_code = 1
# makes it compatible to Steve's code. old_code = 0 moves up to the new code.
#####################################################################
my $old_code = 0;

#####################################################################
# This counter is used to generate unique Security IDs for IPsec
#####################################################################
my $spi_counter = 256;  
			


# Each overlay is assigned an index. Indexes start at 1. 
# This data structure is used to get that index using   
# the name of the overlay.                              
my %overlay_index = ();


#########################################################
# We want our overlays to be assigned from one onwards  #
# so we initialize it with a fake entry in index zero   #
#########################################################

# Each element of this list is a reference to a complicated
# hash.  Hash stores information like the overlay name,
# Creator information, Nodes participating and their information
# Security information, etc.
my @overlay_array = ("Overlay Zero Cannot be used");



##############################################################
# This is a parallel structure with the same indices as
# overlay_array, but used to store the ip_addresses returned
# by the ip_allocator. It is used to correctly return the
# set of addresses to the ip_allocator during destroy
##############################################################
my @overlay_allocation = ("");



############################################
#
# remove_overlay_entry ($OverlayName)
#
# Deletes the overlay entry from the MIB   
#
# Input: (OverlayName)
# Output: 1 on success, 0 on failure
#
############################################
sub remove_overlay_entry ($)
{
   my $procname = "remove_overlay_entry";
   XB_Log::log "info", "-> $procname: @_\n";
   my ($OverlayName) = @_;
   if (defined $overlay_index{$OverlayName}) {
       if ($overlay_index{$OverlayName} == $#overlay_array) {
	   pop @overlay_array;
	   pop @overlay_allocation;
	   delete $overlay_index{$OverlayName};
       } else {
           undef $overlay_array[$overlay_index{$OverlayName}];
           undef $overlay_allocation[$overlay_index{$OverlayName}];
	   delete $overlay_index{$OverlayName};
       }
       XB_Log::log "info", "<- $procname: 1\n";
       return 1;
   } else {
       XB_Log::log "err", "overlay $OverlayName does not exist, so cannot remove\n";
       XB_Log::log "info", "<- $procname: 0\n";
       return 0;
   }
}

############################################
#
# include_node_in_ovlstate ($OverlayName, $NodeName)
#
# Marks a specific node as not included in an overlay
#
# Input: (OverlayName, $NodeName)
# Output: 1 on success, 0 on failure
#
############################################
sub include_node_in_ovlstate ($$)
{
    my $procname = "include_node_in_ovlstate";
    my ($OverlayName, $NodeName) = @_;
    XB_Log::log "info", "-> $procname: @_\n";
    my $index =  $overlay_index{$OverlayName};
    if (not defined $index) {
	die "Fatal: $procname: internal error: overlay $OverlayName not defined\n";
    }
    my $reqref = $overlay_array[$index];
    foreach my $key (keys %{$$reqref{XboneNode}}) {
        my $name = $$reqref{XboneNode}{$key};
	if (not defined $$reqref{$name}{Status}) {
	    die "Fatal: $procname: Status of a node should always be defined\n";
	}
	if ($name eq $NodeName) {
	    $$reqref{$name}{Status} = "in";
	    XB_Log::log "info", "<- $procname: 1\n";
	    return 1;
	}
    }
    XB_Log::log "err", "$procname: node did not exist\n";
    XB_Log::log "info", "<- $procname: 0\n";
    return 0;
}

############################################
#
# update_mib ($commands)
#
# Update the details of an overlay in the mib
#
# Input: (commands)
#	commands: A string in the xbone protocol format containing the
#		 the header that describes the command, creator, security
#		 etc. Followed by a list of XboneNode commands listing the
#		 nodes involved.
# Output: NONE
# Side effects: The overlay description is replaced with the new one
#
############################################
sub update_mib ($)
{
   my $procname = "update_mib";
   XB_Log::log "info", "-> $procname: @_\n";
   my ($commands) = @_;
   my @reqrecs = xb_generic_input($commands);
   my $reqref  = xb_text2hash_index(0, @reqrecs);

   my $OverlayName = $$reqref{XboneOverlay}{Name};
   if (defined $overlay_index{$OverlayName}) {
       my $index = $overlay_index{$OverlayName};
       ######################################################
       # Now overwrite the old array entry with the new one #
       ######################################################
       $overlay_array[$index] = $reqref;
   } else {
      die "Fatal: overlay name should already be in mib in update_mib\n";
   }
}

############################################
#
# int get_unique_overlay_number(string)                #
#
# Enter the description of the overlay into the MIB    #
# for future use. This also assigns it a unique index  #
# number, which is used in the numbering of tunnels,   #
# and virtual host ids.                                #
#
# Input: (commands)
#	commands: A string in the xbone protocol format containing the
#		 the header that describes the command, creator, security
#		 etc. Followed by a list of XboneNode commands listing the
#		 nodes involved.
# Output: (overlay_number)
#	overlay_number: Id assigned to the overlay or 0 for failure
#
############################################
sub get_unique_overlay_number ($)
{
   my $procname = "get_unique_overlay_number";
   XB_Log::log "info", "-> $procname: @_\n";

   my ($commands) = @_;

   my @reqrecs = xb_generic_input($commands);
   my $reqref  = xb_text2hash_index(0, @reqrecs);
   my $return_val;

   unless ($#overlay_array >= 0) {die "Fatal: overlay_array size $#overlay_array should always be greater than zero\n@overlay_array\n";}

   my $OverlayName = $$reqref{XboneOverlay}{Name};
   if (defined $overlay_index{$OverlayName}) {
       XB_Log::log "err", "Overlay with name $OverlayName already exists\n";
       XB_Log::log "info", "<- $procname: 0\n";
       return 0;
   }

   for my $loop_index ( 1 .. $#overlay_array) {
     unless (defined $overlay_array[$loop_index]) {
       $overlay_array[$loop_index] = $reqref;
       $overlay_index{$OverlayName} = $loop_index;
       $return_val = $loop_index;
       XB_Log::log "info", "<- $procname: $return_val\n";
       return $return_val;
     }
   }
   push @overlay_array, $reqref;
   push @overlay_allocation, undef;
   $overlay_index{$OverlayName} = $#overlay_array;
   XB_Log::log "info", "<- $procname: " . $#overlay_array . "\n";
   return $#overlay_array;
}

############################################
#
# xb_ovlmgr_listoverlay ()
#
# Dig out the list of overlays from the MIB 
#
# Input: NONE
# Output: (return_string)
#	return_string: Xbone protocol formatted string containing list of
#		       overlay names
#
############################################
sub xb_ovlmgr_listoverlay ()
{
   my $procname = "xb_ovlmgr_listoverlay";
   XB_Log::log "info", "-> $procname: @_\n";
   my $return_string = "Xbone Version=$XB_Defs::XBONE_PROTOCOL\nXboneControl Command=ListOverlays Transaction=Reply\n";
   foreach my $key (keys %overlay_index) {
       $return_string .= "XboneOverlay Name=$key\n";
   }
   $return_string .= "XboneEOC\n";
   XB_Log::log "info", "<- $procname:\n";
   return $return_string;
}

############################################
#
# xb_generate_spi ()
#
# Generates a unique 32 bit number in hex
# Using a counter spi_counter for uniqueness
#
# Input: NONE
# Output: (return_string)
#	return_string: a string containing a formatted 32 bit unique hex number
#		      
#
############################################
sub xb_generate_spi ()
{
   my $procname = "xb_generate_spi";
   XB_Log::log "info", "-> $procname: @_\n";
   my $return_string = sprintf "0x%08x", ++$spi_counter;
   XB_Log::log "info", "<- $procname: $return_string\n";
   return $return_string;
}

############################################
#
# xb_generate_IPsec_parameters ($IPsecAuth, $IPsecEnc)
#
# Generates IPsec parameters based on the requirements
#
# Input: ($IPsecAuth, $IPsecEnc)
#	IPsecAuth: either undef or a valid IPsec Authentication algorithm
#	IPsecEnc: either undef or a valid IPsec Encryption algorithm
# Output: ($Spi_str, $Auth_fwd_str, $Enc_fwd_str, $Auth_back_str, $Enc_back_str)
# 	A list of parameters suitable for passing for XboneNodeCommand add IPsec
#		      
#
############################################
sub xb_generate_IPsec_parameters ($$)
{
  my $procname = "xb_generate_IPsec_parameters";
  XB_Log::log "info", "-> $procname: @_\n";
  my ($IPsecAuth, $IPsecEnc) = @_;

  my $Auth_fwd_str = "undef 0x0";
  my $Enc_fwd_str = "undef 0x0";
  my $Auth_back_str = "undef 0x0";
  my $Enc_back_str = "undef 0x0";
  my $Spi_str = xb_generate_spi();

  if ($IPsecAuth ne "undef")
      {$Auth_fwd_str = $IPsecAuth." ".XB_IPsec::key($IPsecAuth);}
  if ($IPsecEnc ne "undef")
      {$Enc_fwd_str = $IPsecEnc." ".XB_IPsec::key($IPsecEnc);}
  if ($IPsecAuth ne "undef")
      {$Auth_back_str = $IPsecAuth." ".XB_IPsec::key($IPsecAuth);}
  if ($IPsecEnc ne "undef")
      {$Enc_back_str = $IPsecEnc." ".XB_IPsec::key($IPsecEnc);}
  XB_Log::log "info", "<- $procname:\n";
  return ($Spi_str, $Auth_fwd_str, $Enc_fwd_str, $Auth_back_str, $Enc_back_str);
}

############################################
#
# teardown_overlay ($header_string)
#
# For rollback of state on failure during xbone_ovlmgr_create
#
# Input: ($header_string)
#	header_string: the header (creator, overlay, security) section of
#			of the XboneCreate message
# Output: None
#
# Side effect:
#	Creates a OverlayDestroy message and uses xb_ovlmgr_destroy to
#	fake a destroy. That removes all state from all nodes that are
#	part of the overlay, then removes internal state.
#		      
#
############################################
sub teardown_overlay($)
{
    my $procname = "teardown_overlay";
    XB_Log::log "info", "-> $procname: @_\n";
    my ($header_string) = @_;
    my $message = $header_string;
    $message =~ s/\bCreateOverlay\b/DestroyOverlay/;
    $message .= "XboneEOC\n";
    XB_Log::log "debug1", "teardown_overlay:\n";
    XB_Log::log "debug2", "$message\n###########\n";
    xb_ovlmgr_destroy($message);
    XB_Log::log "info", "<- $procname:\n";
}

###############################################
#
# xb_ovlmgr_create ($commands)
#
# Create the overlay
#
# Input: ($commands)
#	commands: Xbone Protocol message containing the description of
#		the overlay to set up
# Output: ($return_string)
#	return_string: XboneProtocol and/or Error message
#
# Side effect:
# Send mcast message to daemons and collect responses.
# Obtain a block of IP addresses from IP_addr_allocator
# Then fire off control messages to the chosen resources to reserve.
#
##############################################
sub xb_ovlmgr_create ($)
{
  my $procname = "xb_ovlmgr_create";
  XB_Log::log "info", "-> $procname: @_\n";
  my ($commands) = @_; 

  my $ix;
  my @responses = ();
  my @lines; my @hostlines; my @routerlines;
  my $line;
  my $class;
  my $router_count = 0;
  my $host_count = 0;
  my $Topology;
  my $IPsecAuth;
  my $IPsecEnc;
  my $Application;
  my $OverlayName;
  my $SearchRadius;
  my $DNS_flag;
  my $release;
  my $overlay_state = "NONE";
  my $header_string = "";      # This will contain the common boiler plate to
                               # be sent to all nodes
  my $dns_header_string ="";   # Header for all DNS messages.
  my $return_string = "";  # This will contain the results to be sent back
			   # to the GUi and stored in the MIB as overlay state
  my ($Spi_string, $Auth_fwd_str, $Enc_fwd_str, $Auth_back_str, $Enc_back_str);

  #########################################################################
  # The following eval defines an error recovery block, i.e., the block
  # within which all fatal exceptions are caught and handled at the bottom
  # of the block
  #########################################################################
  eval {
    #########################################################
    # Parse the input commands to get:
    #  - the overlay name and search radius
    #  - the network topology, IPsec info, and application to deploy
    #  - the router and host counts
    #########################################################
    my @line = split /\n/, $commands;
    while ($_ = $line[0]) {
      shift @line;
      SWITCH: {
        /^XboneControl\s*/ and do {
           if (/Command=(\w\S+)/) {
               if ($1 ne "CreateOverlay") {
                  die "Fatal: Something wrong, Command should always be Create in proc $procname: $1\n";
               }
           }
           last SWITCH;
        };
	/^XboneOverlay\s*/ and do {
	   if (/Name=(\w\S+)/) {
	      $OverlayName=$1;
	   }
	   if (/SearchRadius=(\S+)/) {
	      $SearchRadius=$1;
	   }
	   if (/DNS=(\S+)/) {
	       $DNS_flag=$1;
	   }
	   last SWITCH;
	};
	/^XboneNetwork\s*/ and do {
	   if (/Topology=(\w\S+)/) {
	      $Topology=$1;
	   }
	   if (/IPsecAuthentication=(\w\S+)/) {
	      $IPsecAuth=$1;
	   }
	   if (/IPsecEncryption=(\w\S+)/) {
	      $IPsecEnc=$1;
	   }
	   if (/Application=(\w\S+)/) {
		$Application=$1;
	   }

	   last SWITCH;
	};
        /^XboneNode\s*/ and do {
          /Class=(\w\S+)\s*/ and do {
               $class = $1;
          };
          /NodeCount=(\S+)\s*/ and do {
		if ($class  eq "host") {
		    $host_count = $1;
		} else {
		    $router_count = $1;
		}
	  };
	  last SWITCH;
        };
      }
    }
    unless (defined($OverlayName)) {
      XB_Log::log "err", "$procname: Illegal OverlayName\n";
      XB_Log::log "info", "<- $procname:\n";
      die "Error: Illegal OverlayName in $procname\n";
    }

    ################################################################
    # Do a quick sanity check on the input parameters
    ################################################################
    if (my $errline =
	XB_Topology::sanity_check($Topology, $host_count, $router_count)) {
      XB_Log::log "err", "$procname: $errline\n";
      XB_Log::log "info", "<- $procname:\n";
      die $errline;
    }

    ################################################################
    # Now parse the commands more carefully to produce
    # the new commands that you will multicast to everyone to get
    # the set of nodes that can play in this overlay.
    ################################################################
    my $newcommands = "Xbone Version=$XB_Defs::XBONE_PROTOCOL " .
       "Release=\"$XB_Defs::XBONE_RELEASE $XB_Defs::XBONE_OVLMGR_BACKCOMPAT\"".
       " Seq=" . time . "\nXboneControl Command=ResourceDiscovery " .
       "Transaction=Request\n";

    @line = split /\n/, $commands;
    while ($_ = $line[0]) {
	shift @line;
      SWITCH: {
	  (/\bXboneNetwork\b/ or
	   /\bXboneCreator\b/ or
	   /\bXboneSecurity\b/ or
	   /\bXboneEOC\b/ or
	   /\bXboneMonitor\b/ ) and do {
	       $newcommands .= "$_\n";
	       last SWITCH;
	   };
	  /\bXboneOverlay\b/ and do {
	      $newcommands .= "XboneOverlay Name=$OverlayName\n";
	      last SWITCH;
	  };
	  s/\bXboneNode\b// and do {
	      my $tunnel_count = 0;
	      s/\bClass=(\w\S+)\s*// and do {
		  $class = $1;
	      };
	      s/\bNodeCount=\S+\s*//;
	      $tunnel_count = 
		XB_Topology::tunnel_count($Topology, $class, $host_count, $router_count);
	      if ($tunnel_count == 0) {
		  die "Fatal: tunnel_count should never be zero if sanity check has been performed\n";
	      }

	      $newcommands .= "XboneNode Class=$class InterfaceCount=$tunnel_count";
	      $newcommands .= " $_" unless ($_ eq ""); 
	      # Add the IPsec authentication and Encryption lines
	      
	      # FIX ME: This is a temporary to allow steve;s and greg's daemon to
	      # coexist. old_code should go away and the second branch is the
	      # correct one that you should keep.
	      ##if ($old_code) {
	      ##$newcommands .= " $IPsecAuth" unless ($IPsecAuth eq "undef");
	      ##$newcommands .= " $IPsecEnc" unless ($IPsecEnc eq "undef");
	      ##} else {
		  $newcommands .= " Authentication=$IPsecAuth"
		      unless ($IPsecAuth eq "undef");
		  $newcommands .= " Encryption=$IPsecEnc"
		      unless ($IPsecEnc eq "undef");
	      ##}
	      $newcommands .= "\n";
	  }
      }
    }
    
    
    XB_Log::log "debug1", "Sending multicast:\n";
    XB_Log::log "debug2", "$newcommands###########\n";

    ################################################################
    # multicast out your message 
    ################################################################
    @responses = XB_RPC::XB_request_overlay (\$newcommands, $SearchRadius);

    XB_Log::log "debug1", "Got responses from " . scalar(@responses) . " nodes\n";
    XB_Log::log "debug2", "@responses###########\n";
    
    ##################################################
    # Get the common part of the header messages
    # before the XboneNode commands start
    #################################################
    my @command_header = split  /\n/, $commands;
    @command_header = grep !/XboneNode/,  @command_header;

    my $OverlayNumber;

    ##########################################################################
    # Modify the command header to produce the header for the return message #
    ##########################################################################
    for ($ix = 0;  $ix < scalar (@command_header);  $ix++)
    {
        $_ = $command_header[$ix];
	/\bXboneEOC\b/ and do {last;};
        /\bXboneOverlay\b/ and do {
	    # Remove the DNS line as well.
	    s/SearchRadius=\S+\s*DNS=\S+//;
        };
        $header_string .= "$_\n";
        $_ = $command_header[$ix];
        /\bXboneControl\b/ and do {
	    s/\bRequest\b/Reply/;
        };
        $return_string .= "$_\n";
    }



    # Check each response for missing protocol field or mismatch.
    # Drop message if error after logging.
    my @checked_responses = ();
    my $m;
    foreach $m (@responses)
    {
	$_ = $m;
	/^Xbone Version=(\S+)/;
	if ($1 ne $XB_Defs::XBONE_PROTOCOL) {
	    XB_Log::log "debug1", "Version field missing or a mismatch " .
		"occured for message $m : dropped\n";
	}
	else {
	    XB_Log::log "debug2", "Message : $m\n";
	    push @checked_responses, $m;
	}
    }


    # Separate all messages according to the release field.
    my %message_hash = ();
    foreach $m (@checked_responses) {
	if ($m =~ /\bRelease=(\S+)/) {
	    if (!defined $message_hash{$1}) {
		$message_hash{$1} = [];
	    }
	    push @{$message_hash{$1}}, $m;
	}
	else {
	  XB_Log::log "debug1", "Release field missing from message : $m\n";
	}
    }


    # collect all the different values of release field that we get and sort.
    my @releases = reverse sort keys %message_hash;

    my $r;
    my $flag = 0;
    my @rmessages = ();
    my $errline;
    foreach $r (@releases) {
	# @rmessages contains all messages corresponding to release $release
	@rmessages = @{$message_hash{$r}};


	# now remove noise and separate hosts/routers
	for (my $ix = 0;  $ix < scalar (@rmessages);  $ix++) {
	    my @one_response = grep /XboneNode/, split  /\n/, $rmessages[$ix];
	    $rmessages[$ix] = "$one_response[0]\n";
	}

	@hostlines = grep /Class=host/, @rmessages;
	@routerlines = grep /Class=router/, @rmessages;

	# check if we have enough, if yes then exit after making flag = 1    
	$errline = XB_Topology::response_check($Topology, $host_count,
						  $router_count, 
						  scalar (@hostlines),
						  scalar (@routerlines));
	if ($errline eq "") {
	    $flag = 1;
	    XB_Log::log "info", "Found enough responses for release : $r\n";
	    $release = $r;
	    last ;
	}
    }

    if ($flag == 0) {
        XB_Log::log "err", "$procname: $errline\n";
        XB_Log::log "info", "<- $procname:\n";
	  my $x = $#hostlines + 1;
	  my $y = $#routerlines + 1;
	die "Couldn't find enough nodes. Found $x hosts and $y routers only";
    }
    
    # We're now sure that we have enough responses.
    # If we're doing a demo, then we pick hosts/routers in the reverse order.
    if ($XB_Defs::DEMO_MODE == 1) {
        XB_Log::log "info", "Reversing the order of host/router pick.\n";
	@hostlines = reverse @hostlines;
	@routerlines = reverse @routerlines;
    }


    # Need to add Release field to $header_string because
    # header_string has been constructed from the GUI message which
    # doesn't contain the Release field.
    $dns_header_string = $header_string;
    $header_string =~ s/\bVersion=(\S+)/Version=$1 Release=$release/;

    
    # Here we are going to make an entry into the overlay manager's
    # permanent state. So if we have any errors after successful return
    # of this function, must remember to remove this state using
    # remove_overlay_entry
    if (! ($OverlayNumber = get_unique_overlay_number($commands))) {
            XB_Log::log "err", "$procname: Could not get unique overlay number\n";
            XB_Log::log "info", "<- $procname:\n";
	    die "Error: Could not get unique overlay number. Most possibly an overlay with the same name already exists\n";
    }
    $overlay_state = "INTERNAL";

    ###############################################################
    # In the following two loops we perform the following tasks
    # - Decide which machines to use and which not to use
    # - extract the names of the machines to be used
    # - assign them virtual names, but keep track of the full names
    # - create the header of the message to be sent to each (real) machine
    # - construct the reply message to be set to the GUI
    ###############################################################
    my @router_name = ("") x $router_count;
    my @host_name = ("") x $host_count;
    my %IPaddr = ();
    my %fullName = ();
    my %Class = ();
    my %cmdmsg = ();

    ############################################################
    # First do the above for the routers
    ############################################################
    for ($ix = 0;  $ix < scalar (@routerlines);  $ix++)
    {
	$_ = $routerlines[$ix];

        if ($ix < $router_count) {   # If we are still looking for more routers

	    # Extract the name of the router
	    /ID=(\S*)/ and do {
	        my $temp = my $realName = $1; 
	        $temp =~ s/\..*//;  # chop off anything
				# after the first .
	        my $suffix = "";
	        if (exists $fullName{$temp}) {
	            $suffix = 1;
		    while (exists $fullName{$temp .$suffix}) {$suffix++;}
	        }
	        $router_name[$ix] = $temp . $suffix;
	        $fullName{$router_name[$ix]} = $realName;
	    };

	    # Extract the IP address of the router
	    /\bIPaddr=(\S+)\s*/ and do {
	         $IPaddr{$fullName{$router_name[$ix]}} = $1;
	    };

	    # Set the class of the node
	    $Class{$router_name[$ix]} = "router";

	    # Construct the XboneNode string to send to this node
	    my $node_string = $_;
	    $node_string =~ s/\bInterfaceCount=\d+\s*//;

    	    ###########################################
    	    # Construct the header of the command to send to this node
    	    ###########################################
	    if (exists $cmdmsg{$fullName{$router_name[$ix]}}) {
		die "Fatal: $procname: we should not be trying to create two messages to the same physical machine. $fullName{$router_name[$ix]}";
	    }

    	    $cmdmsg{$fullName{$router_name[$ix]}} = $header_string;
    	    $cmdmsg{$fullName{$router_name[$ix]}} .= $node_string;

	}
	###############################################################
	# Make sure the resource daemon sent a status line and if not
	# put it in
	###############################################################
        unless (/Status=out/) {
            s/$/ Status=out/; 
        }
	$return_string .= $_;
    }

    ############################################################
    # Then do the same for the hosts
    ############################################################
    for ($ix = 0;  $ix < scalar (@hostlines);  $ix++)
    {
	$_ = $hostlines[$ix];

        if ($ix < $host_count) {  # If we are still looking for more hosts
	    # Extract the name of the router
	    /ID=(\S*)/ and do {
	        my $temp = my $realName = $1; 
	        $temp =~ s/\..*//;  # chop off anything
				    # after the first .
	        my $suffix = "";
	        if (exists $fullName{$temp}) {
	            $suffix = 1;
		    while (exists $fullName{$temp . $suffix}) { $suffix++;}
	        }
	        $host_name[$ix] = $temp . $suffix;
	        $fullName{$host_name[$ix]} = $realName;
	    };

	    # Extract the IP address of the router
	    /IPaddr=(\S+)\s*/ and do {
	         $IPaddr{$fullName{$host_name[$ix]}} = $1;
	    };

	    # Set the class of the node
	    $Class{$host_name[$ix]} = "host";

	    # Construct the XboneNode string to send to this node
	    my $node_string = $_;
	    $node_string =~ s/\bInterfaceCount=\d+\s*//;

    	    ###########################################
    	    # Construct the header of the command to send to this node
    	    ###########################################
	    if (exists $cmdmsg{$fullName{$host_name[$ix]}}) {
		die "Fatal: $procname: we should not be trying to create two messages to the same physical machine. $fullName{$host_name[$ix]}";
	    }
    	    $cmdmsg{$fullName{$host_name[$ix]}} = $header_string;
    	    $cmdmsg{$fullName{$host_name[$ix]}} .= $node_string;

	}
	###############################################################
	# Make sure the resource daemon sent a status line and if not
	# put it in
	###############################################################
        unless (/Status=out/) {
            s/$/ Status=out/; 
        }
	$return_string .= $_;
    }

    $return_string .= "XboneEOC\n";

    ###################################################################
    # Here we are entering the string (constructed in the two
    # loops above ) about which nodes responded to the multicast
    # into the overlay managers internal state. Must remember to remove
    # this if we have to abort the setup
    ###################################################################
    update_mib($return_string);

    my $ipAllocRequest = XB_Topology::create_ip_alloc_request($Topology, 
							      \@host_name, 
							      \@router_name);
    $dns_header_string =~ s/\bCreateOverlay\b/DNSUpdate/;

    ###################################################################
    # Obtain IP addresses from IP address allocator.
    ###################################################################
    my $allocation = XB_IP_Allocation::XB_ip_alloc($OverlayName,
						   $dns_header_string,
						   $ipAllocRequest, $DNS_flag);
    unless (defined $allocation) {
	remove_overlay_entry($OverlayName);
        XB_Log::log "err", "$procname: IP allocation module returned invalid allocation\n";
        XB_Log::log "info", "<- $procname:\n";
	die "Error: IP allocation module returned invalid allocation\n";
    }
    $overlay_allocation[$OverlayNumber] = $allocation;

    ###################################################################
    # Now we have external state so must remember to remove it.
    ###################################################################
    $overlay_state = "EXTERNAL";

    # save the release.
    my $tempr = $overlay_array[$OverlayNumber];
    $$tempr{XboneOverlay}{Release} = $release;

    ###################################################################
    # Create the overlay topology structure required by the application
    # deployment (Oscar's code).
    ###################################################################
    my $ovlTopology = XB_Topology::create_ovlTopology($Topology, \@host_name, 
						      \@router_name, 
						      $allocation);


    ########################################################################
    # Now start generating the NodeCommands to send to the individual nodes.
    # For each edge, output commands for the tunnels, the IPsec association,
    # and for the routes 
    ########################################################################
    foreach my $address_pair (@{ $allocation->{'structure'} })
    {

	my $internal0 =
	    XB_IP_Allocation::XB_generate_internal_address($address_pair->[0]);

	my $internal1 =
	    XB_IP_Allocation::XB_generate_internal_address($address_pair->[1]);

	my $node0 = $allocation->{'name'}->{$address_pair->[0]};
	my $node1 = $allocation->{'name'}->{$address_pair->[1]};

	# Two tunnels in the forward direction
	$cmdmsg{$fullName{$node0}} .= "XboneNodeCommand tunnel add $internal0 $internal1 $allocation->{linkmask} $IPaddr{$fullName{$node0}} $IPaddr{$fullName{$node1}}\n";
	$cmdmsg{$fullName{$node0}} .= "XboneNodeCommand tunnel add $address_pair->[0] $address_pair->[1] $allocation->{linkmask} $internal0 $internal1\n";

	# Two tunnels in the other direction
	$cmdmsg{$fullName{$node1}} .= "XboneNodeCommand tunnel add $internal1 $internal0 $allocation->{linkmask} $IPaddr{$fullName{$node1}} $IPaddr{$fullName{$node0}}\n";
	$cmdmsg{$fullName{$node1}} .= "XboneNodeCommand tunnel add $address_pair->[1] $address_pair->[0] $allocation->{linkmask} $internal1 $internal0\n";

	# IPsec entries if IPsec is turned on
	if (($IPsecAuth ne "undef") || ($IPsecEnc ne "undef")) {

	    ($Spi_string,
	     $Auth_fwd_str,
	     $Enc_fwd_str,
	     $Auth_back_str,
	     $Enc_back_str
	    ) = xb_generate_IPsec_parameters($IPsecAuth, $IPsecEnc);

	    $cmdmsg{$fullName{$node0}} .= "XboneNodeCommand IPsec add $internal0 $internal1 $Spi_string $Auth_fwd_str $Enc_fwd_str $Auth_back_str $Enc_back_str\n";
	    $cmdmsg{$fullName{$node1}} .= "XboneNodeCommand IPsec add $internal1 $internal0 $Spi_string $Auth_back_str $Enc_back_str $Auth_fwd_str $Enc_fwd_str\n";

	}

	# Routes if required. Generate_routes will return empty strings
	# if routes are not needed, and multiple lines when needed
	$cmdmsg{$fullName{$node0}} .= XB_Topology::generate_route(
					   $Topology,
					   $address_pair->[0],
					   $address_pair->[1],
					   \%Class,
					   $allocation,
					   $ovlTopology->{'Structure'}
								  );

	$cmdmsg{$fullName{$node1}} .= XB_Topology::generate_route(
					   $Topology,
					   $address_pair->[1],
					   $address_pair->[0],
					   \%Class,
					   $allocation,
					   $ovlTopology->{'Structure'}
								  );

    }

    # Now put in any application deployment commands needed
    if (defined ($Application) and ($Application ne "none")) {
	foreach my $node (@router_name, @host_name)
        {
	    $cmdmsg{$fullName{$node}} .=
	        XB_OM_App_Start::appOvlNodeCommands(
		    $node,
		    "",	# Node OS (don't know)
		    "yes",  # Support Apps
		    "",	# installed packages (don't know)
		    $Application,
		    $ovlTopology);
	}
    }

    # All done. Now send the messages
    foreach my $node (keys %cmdmsg)
    {
	 # Terminate the message correctly
	 $cmdmsg{$node} .= "XboneEOC\n";

	 XB_Log::log "debug1", "sending to $node:\n";
	 XB_Log::log "debug2", "$cmdmsg{$node}###########\n";

	 # send the message
	 $_ = XB_RPC::XB_resource_daemon_rpc (
				$node,
				$XB_Defs::MCAST_PORT,
				$cmdmsg{$node});

	 XB_Log::log "debug1", "Reply from $node:\n";
	 XB_Log::log "debug2", "$_###########\n";

	 include_node_in_ovlstate($OverlayName, $node) or
		XB_Log::log "err", "$procname: Could not change node status to in\n";

         if (/Code=(\S+)\s*/) {
            unless ($1 == 0) {
                my $message = "$node sent back: code $1";
		if (/Msg=\"([^\"]*)\"/) { $message .=  ", Msg=$1"; }
                die $message;
            }
         } else {
             die "Fatal: Must have Code field in return message from resource daemon\n$_\n";
         }
    }
  };

  # Error handling segment
  if ($@) {
    my $errorstring = $@;
    chomp($errorstring);
    if ($errorstring =~ /Fatal/) {
	# Do not catch fatal errors. These are meant to kill the program.
	die "$errorstring";
    }
    if ($overlay_state =~ /NONE/) {
	XB_Log::log "err", "$procname: Error handling (no state to remove): $errorstring\n";
    } elsif ($overlay_state =~ /INTERNAL/i) {
	# If we only have to remove internal state
	remove_overlay_entry($OverlayName);
        XB_Log::log "err", "$procname: Error handling (removing internal state): $errorstring\n";
    } elsif ($overlay_state =~ /EXTERNAL/i) {
	# If we have to roll back node state
        XB_Log::log "err", "$procname: Error handling (removing external state): $errorstring\n";
	teardown_overlay($header_string);
    } else {
      die "Unknown internal state: $overlay_state";
    }
    XB_Log::log "info", "<- $procname:\n";
    return "Xbone Version=$XB_Defs::XBONE_PROTOCOL\nXboneReturn Msg=\"$errorstring\"\nXboneEOC\n";
  }

  XB_Log::log "info", "<- $procname:\n";

  my $index =  $overlay_index{$OverlayName};
  if (not defined $index) {
        die "Overlay should be defined here";
  }
  my $reqref = $overlay_array[$index];
  $return_string = hash2text($reqref);
  return $return_string;
}


###############################################
#
# xb_ovlmgr_destroy ($commands)
#
# Destroy the overlay
#
# Input: ($commands)
#	commands: Xbone Protocol message containing the description of
#		the overlay to destroy
# Output: ($return_string)
#	return_string: XboneProtocol describing results and/or Error message
#
# Side effect:
# Then fire off control messages to associated xbone daemons to
# tear down the state. Remove state from DNS servers. Then tear
# down internal state. 
#
##############################################
sub xb_ovlmgr_destroy ($)
{

 my $procname = "xb_ovlmgr_destroy";
 XB_Log::log "info", "-> $procname: @_\n";
 
 my $line;
 my $OverlayName;
 my $Command;
 my $DNS_flag;
 
 my ($commands) = @_; 
 my $header_string = "Xbone Version=$XB_Defs::XBONE_PROTOCOL\n";
 my $dns_header_string;
 
 # Parse the input commands :
 #  - to get the overlay name 
 #  - to get the Security parameters
 #  - and to construct the header
 my @line = split /\n/, $commands;
 while ($_ = $line[0]) {
     shift @line;
   SWITCH: {
       /^XboneControl\s*/ and do {
	   $header_string .= "$_\n";
	   $Command="DestroyOverlay";
           last SWITCH;
       };
       
       /^XboneOverlay\s*/ and do {
	   $header_string .= "$_\n";
	   if (/Name=(\w\S+)/) {
	       $OverlayName=$1;
	   }
	   last SWITCH;
       };
       /^XboneSecurity\s*/ and do {
	   $header_string .= "$_\n";
	   last SWITCH;
       };
   }
 }
 
 $dns_header_string = $header_string;
 my $message =  ""; # Start composing a return message
 
 
 
 # Look up overlay information
 my $index =  $overlay_index{$OverlayName};
 if (not defined $index) {
     $message = "XboneReturn Code=1 Msg=\"Overlay $OverlayName not " .
	 "defined\"\n";
     goto EXIT;
 }
 
 # here we extract the DNS flag and the Release for this overlay
 # we put the release in the messages that we send to the node.
 my $reqref = $overlay_array[$index];
 my $release = $$reqref{XboneOverlay}{Release};
 $header_string =~ s/\bVersion=(\S+)/Version=$1 Release=$release/;
 $DNS_flag = $$reqref{XboneOverlay}{DNS};

 
 # For each involved node, make a message and send it
 foreach my $key (keys %{$$reqref{XboneNode}}) {
     my $name = $$reqref{XboneNode}{$key};
     if (not defined $$reqref{$name}{Status}) {
	 die "Fatal: $procname: Status of a node should always be defined\n";
     }
     
     
     # If the node is involved in the overlay
     if ($$reqref{$name}{Status} eq "in") {
	 
	 # Construct the message to send to the node
	 my $cmdmsg = $header_string;          # Need to send control
	 # info in the header
	 $cmdmsg .= "XboneNode ID=$name\n";  # Need to send the XboneNode
	 # info for the current node
	 $cmdmsg .= "XboneNodeCommand overlay delete $OverlayName\n";
	 $cmdmsg .= "XboneEOC\n";
	 
       XB_Log::log "debug1", "sending to $name:\n";
       XB_Log::log "debug2", "$cmdmsg###########\n";
	 
	 # Send the message in an eval to catch all dies
	 eval {
	     $_ = XB_RPC::XB_resource_daemon_rpc ($name,
						  $XB_Defs::MCAST_PORT,
						  $cmdmsg);
	     if (/Code=(\S+)\s*/) {
		 if ($1 != 0) {
		     $message .=
			 "XboneReturn Code=1 Msg=\"$name sent back: code $1";
		     if (/Msg=\"([^\"]*)\"/) {
			 $message .=  ", Msg=$1";
		     }
		     $message .= "\"\n";
		 }
	     } 
	     else {
		 die "Fatal: Must have Code field in return message from resource daemon (was $_)\n";
	     }
	   XB_Log::log "debug1", "Reply from $name:\n";
	   XB_Log::log "debug2", "$_###########\n";
	 };
	 if ($@) { 
	     $message .= "XboneReturn Code=1 Msg=\"$@\"\n"; 
	 }
	 
     }
 }
 
 # Construct the message to send to the DNS servers
 $dns_header_string =~ s/DestroyOverlay/DNSUpdate/;
 
 
 # Return the overlay addresses to the IP address allocation modules
 XB_IP_Allocation::XB_ip_dealloc("$OverlayName", $dns_header_string, 
				 $overlay_allocation[$index], $DNS_flag);
 
 # Remove the overlay state from the internal mib
 if (!remove_overlay_entry($OverlayName) ) {
     $message .= "XboneReturn Code=1 Msg=\"Warning: Could not Remove " . 
	 "overlay entry in MIB\"\n";
 }
 
 $message .=  "XboneReturn Code=0 Msg=\"Overlay $OverlayName removed\"\n";
 
 EXIT:
 # Now take all responses and error messages and package it up to
 # send back to the GUI for display
 my $return_string = "Xbone Version=$XB_Defs::XBONE_PROTOCOL\n".
     "XboneControl Command=$Command Transaction=Reply\n";
 $return_string .= $message;
 $return_string .= "XboneEOC\n";
 XB_Log::log "info", "<- $procname:\n";
 return $return_string;
 
}



# xb_ovlmgr_destroyall ($commands)
#
# Destroy the overlay
#
# Input: ($commands)
#	commands: Xbone Protocol message containing the description of
#		the overlay to destroy
# Output: ($return_string)
#	return_string: XboneProtocol describing results and/or Error message
#
# Side effect:
#       Removes all active overlays by sending a multicast message. Sends 
#       messages to DNS servers to that effect. 

sub xb_ovlmgr_destroyall ($)
{
  my $procname = "xb_ovlmgr_destroyall";
  XB_Log::log "info", "-> $procname: @_\n";
  my ($commands) = @_; 
  
  my $return_string;
  my ($releases, $r);
  my $message_hdr = $commands;
  my $message;
  my $seqmes;
  my $dns_header;
  my @release_list;

  $return_string = "Xbone Verison=$XB_Defs::XBONE_PROTOCOL\n";
  $return_string .= "XboneControl Command=DestroyAll Transaction=Reply\n";

  # Clear local data structures. any better way of doing this??
  @overlay_array = ("Overlay Zero Cannot be used");
  %overlay_index = ();
  @overlay_allocation = ("");
  
  # Put in the actual command. Need to do it only once.
  $message_hdr =~ s/\bVersion=(\S+)\b/Version=$1 Release=/;
  $message_hdr =~ s/\bXboneEOC\b/XboneNodeCommand overlay killall\nXboneEOC/;

  # Generate a list of releases.
  $releases = $XB_Defs::XBONE_RELEASE ." ". $XB_Defs::XBONE_OVLMGR_BACKCOMPAT;
  @release_list = split /\s+/, $releases;


  foreach $r (@release_list) {

      # putting in release and the sequence number.
      $seqmes = "Seq=" . time;
      $message = $message_hdr;
      $message =~ s/Release=/Release=$r $seqmes/;

      XB_Log::log("debug1", "Sending multicast message :$message\n");
      eval {
	XB_RPC::XB_multicast_message(\$message, $XB_Defs::MAX_MCAST_RADIUS);
      };
      if ($@) { 
	  XB_Log::log("debug1", "Failed to multicast message : $message\n");
	  $return_string .= "XboneReturn Code=1 Msg=\"$@\"\n"; 
	  goto EXIT;
      }
  }


  $dns_header = $commands;
  $dns_header =~ s/\bDestroyAll\b/DNSUpdate/;
  $dns_header =~ s/\bXboneEOC\b//;

  # now we send to DNS RD's, the XB_ip_dealloc_all routine clears 
  # all IP allocation data structures and also sends messages to DNS RD's
  XB_IP_Allocation::XB_ip_dealloc_all($dns_header);
  $return_string .="XboneReturn Code=0 Msg=\"\"\n";

 EXIT:
  $return_string .= "XboneEOC\n";
  XB_Log::log "info", "<- $procname:\n";
  return $return_string;
}




###############################################
#
# NOT YET IMPLEMENTED
#
# xb_ovlmgr_modify ($commands)
#
# Modify the overlay
#
# Input: ($commands)
#	commands: Xbone Protocol message containing the description of
#		the overlay to change. Nodes to add and nodes to delete.
# Output: ($return_string)
#	return_string: XboneProtocol and/or Error message
#
# Side effect:
# Then fire off control messages to subset of xbone daemons to
# tear down the state and add state to effect the described changes.
#
# NOT YET IMPLEMENTED
#
###############################################
sub xb_ovlmgr_modify ($)
{
    my $procname = "xb_ovlmgr_modify";
    XB_Log::log "info", "-> $procname: @_\n";
    XB_Log::log "err", "Error: Modify not implemented\n";
    XB_Log::log "info", "<- $procname:\n";
    return "Error: Modify not implemented\n";
}

###############################################
#
# xb_ovlmgr_status ($commands)
#
# Collect status on the nodes and tunnels of the overlay
#
# Input: ($commands)
#	commands: Xbone Protocol message containing the description of
#		the overlay to collect status on.
# Output: ($return_string)
#	return_string: XboneProtocol message containing the status of the
#	concerned overlay and/or Error messages
#
###############################################
sub xb_ovlmgr_status($)
{
    my $procname = "xb_ovlmgr_status";
    XB_Log::log "info", "-> $procname: @_\n";
    my $OverlayName;
    my $SearchRadius;
    my $SecurityInfo;

    my ($commands) = @_;

    #########################################################
    # Parse the input commands to get:
    #  - the command (OverlayStatus)
    #  - the overlay name 
    #  - the Security parameters
    #########################################################
    my @line = split /\n/, $commands;
    while ($_ = $line[0]) {
      shift @line;
      SWITCH: {
          /^XboneControl\s*/ and do {
	      if (/Command=(\w\S+)/) {
		  if ($1 ne "OverlayStatus") {
		      die "Fatal: Something wrong, Command should always be OverlayStatus in proc $procname: $1\n";
		  }
	      }
	      last SWITCH;
          };
          /^XboneOverlay\s*/ and do {
	      if (/Name=(\w\S+)/) {
		  $OverlayName=$1;
	      }
	      if (/SearchRadius=(\w\S+)/) {
		  $SearchRadius=$1;
	      }
	      last SWITCH;
          };
          /^XboneSecurity\s*/ and do {
	      $SecurityInfo= "$_\n";
	      last SWITCH;
	  }
      }
    }

    #####################################################################
    # Construct message to send to resource daemons
    #####################################################################
    my $header_string = "Xbone Version=$XB_Defs::XBONE_PROTOCOL\n".
	"XboneControl Command=OverlayStatus Transaction=Request\n";
    $header_string .= "XboneOverlay Name=$OverlayName\n";
    $header_string .= $SecurityInfo;

    #####################################################################
    # start creating the return message
    #####################################################################
    $_ = $header_string;
    s/\bTransaction=Request\b/Transaction=Reply/;
    my $reply_string = $_;

    #####################################################################
    # Look up the overlay information
    #####################################################################
    my $index =  $overlay_index{$OverlayName};
    if (not defined $index) {
        $reply_string .= "XboneReturn Msg=\"Warning: Overlay $OverlayName ".
	    "does not exist\"\nXboneEOC\n";
        XB_Log::log "info", "<- $procname:\n";
	return $reply_string;
    }
    my $reqref = $overlay_array[$index];
   
    # need to put in the Release field in the messages.
    my $release = $$reqref{XboneOverlay}{Release};
    $header_string =~ s/\bVersion=(\S+)/Version=$1 Release=$release/;

    #####################################################################
    # Look up the Creator and Network information for reply message
    #####################################################################
    foreach my $line ("XboneCreator", "XboneNetwork") {
        $reply_string .=  "$line";
	foreach my $element (keys %{ $reqref->{$line} }) {
	      $reply_string .= " ${element}=$reqref->{$line}->{$element}";
	}
	$reply_string .= "\n";
    }

    #####################################################################
    # For each node involved in the overlay, construct a status message
    # send it to the node, parse the reply for errors and complile the
    # replies into a single reply message to GUI.
    #####################################################################
    LOOP: foreach my $key (keys %{$$reqref{XboneNode}}) {
        my $name = $$reqref{XboneNode}{$key};
	if (not defined $$reqref{$name}{Status}) {
	    die "Fatal: $procname: Status of a node should always be defined\n";
	}
	if (not defined $$reqref{$name}{Status}) {
	    die "Fatal: Class of a node should always be defined\n";
	}
	my $class =  $$reqref{$name}{Class};

        #####################################################################
        # If the node is involved
        #####################################################################
        if ($$reqref{$name}{Status} eq "in") {

          #####################################################################
          # Construct a OverlayStatus message to send it
          #####################################################################
          my $cmdmsg = $header_string;          # Need to send control
                                                # info in the header
          $cmdmsg .= "XboneNode ID=$name\n";    # Need to send the XboneNode
                                                # info for the current node
          $cmdmsg .= "XboneNodeCommand ping\n";
          $cmdmsg .= "XboneEOC\n";

          #####################################################################
          # Send the message
          #####################################################################
          XB_Log::log "debug1", "sending to $name:\n";
          XB_Log::log "debug2", "$cmdmsg###########\n";

	  my $router_response = "";
	  eval {
	      $router_response =
			XB_RPC::XB_resource_daemon_rpc ($name,
							$XB_Defs::MCAST_PORT,
							$cmdmsg);
              XB_Log::log "debug1", "Reply from $name:\n";
              XB_Log::log "debug2", "$router_response###########\n";
	  };

          #####################################################################
          # Parse the results for errors and return codes
          #####################################################################
	  if ($@) {
	       $reply_string .= "$@\n";
	       next LOOP;
	  }
          $_ = $router_response;
          if (/XboneReturn Code=(\S+)/) {
               if ($1 != 0) {
                  $reply_string .=  "XboneReturn Msg=\"Router $name sent back: Code=$1";
		  if (/Msg=\"([^\"]*)\"/) {
		      $reply_string .=  ", Msg=$1";
		  }
		  $reply_string .=  "\"\nXboneEOC\n";
                  XB_Log::log "info", "<- $procname:\n";
                  return $reply_string;
               }
          } else {
                 die "Fatal: Must have Code field in return message from resource daemon (was $router_response)\n";
          }

          #####################################################################
          # Compile the relevant lines (tunnel and nodea) into the GUI message
          #####################################################################
	  if (/(XboneNode\s+.*)/) {
	      $reply_string .= $1 . "\n";
	  } else {
	      $reply_string .= "Error: missing XboneNode line in response from node $name\n";
	      $reply_string .= "XboneNode Class=$class ID=$name Status=in\n"
	  }
	  while (/(XboneTunnel\s+.*)/g) {
		$reply_string .= $1 . "\n";
	  }

      }
    }
 
 
    $reply_string .= "XboneEOC\n";
    XB_Log::log "info", "<- $procname:\n";
    return $reply_string;
}

###############################################
#
# NOT USED
# 
# xb_ovlmgr_multicast_status ($commands)
#
# Alternative implementation of collect status. Multicasts the status
# message to all nodes and then puts together the UDP responses to
# collect status. Not completely debugged. Also, xb_daemon does not
# support multicast of status messages. Kept for reference.
#
# Input: ($commands)
#	commands: Xbone Protocol message containing the description of
#		the overlay to collect status on.
# Output: ($return_string)
#	return_string: XboneProtocol message containing the status of the
#	concerned overlay and/or Error messages
#
# NOT USED
#
###############################################
sub xb_ovlmgr_multicast_status ($)
{
    my $procname = "xb_ovlmgr_multicast_status";
    XB_Log::log "info", "-> $procname: @_\n";
    my $OverlayName;
    my $SearchRadius;
    my $SecurityInfo;

    my ($commands) = @_;

    my @line = split /\n/, $commands;
    while ($_ = $line[0]) {
      shift @line;
      SWITCH: {
          /^XboneControl\s*/ and do {
             if (/Command=(\w\S+)/) {
                 if ($1 ne "OverlayStatus") {
                    die "Fatal: Something wrong, Command should always be OverlayStatus in proc $procname: $1\n";
                 }
             }
             last SWITCH;
          };
          /^XboneOverlay\s*/ and do {
             if (/Name=(\w\S+)/) {
                $OverlayName=$1;
             }
             if (/SearchRadius=(\w\S+)/) {
                $SearchRadius=$1;
             }
             last SWITCH;
          };
          /^XboneSecurity\s*/ and do {
	     $SecurityInfo= "$_\n";
             last SWITCH;
	  }
      }
    }
    my $newcommands = "Xbone Version=$XB_Defs::XBONE_PROTOCOL\nXboneControl Command=ResourceDiscovery Transaction=Request\n";
    $newcommands .= "XboneOverlay Name=$OverlayName\n";
    $newcommands .= $SecurityInfo;
    my $index =  $overlay_index{$OverlayName};
    my $reqref = $overlay_array[$index];
    my @carray = hash2text_item($reqref, "XboneNetwork");
    if (@carray) { $newcommands .= $carray[0];};
    @carray = hash2text_item($reqref, "XboneCreator");
    if (@carray) { $newcommands .= $carray[0];};
    $_ = $newcommands;
    s/\bResourceDiscovery\b/OverlayStatus/;
    s/\bTransaction=Request\b/Transaction=Reply/;
    my $reply_string = $_;
    $newcommands .= "XboneEOC\n";
 
    my @responses = ();
    eval {
	my @responses =
	    XB_RPC::XB_request_overlay (\$newcommands,
	   				 $SearchRadius);
    };
    if ($@) {
	my $errorstring = $@;
        XB_Log::log "err", "$procname: $errorstring\n";
        XB_Log::log "info", "<- $procname:\n";
	return $errorstring;
    }
 
    XB_Log::log "debug1", "Got responses from " . scalar(@responses) .  " nodes\n";
    XB_Log::log "debug2", "@responses###########\n";
    for (my $ix = 0;  $ix < scalar (@responses);  $ix++)
    {
        my @one_response = grep /XboneNode/, split  /\n/, $responses[$ix];
        $reply_string .= "$one_response[0]\n";
    }
    $reply_string .= "XboneEOC\n";
    XB_Log::log "info", "<- $procname:\n";
    return $reply_string;
}

################
# Main program #
################

STDERR->autoflush(1);
STDOUT->autoflush(1);

my $return_string = "";

if ($Config{osname} eq "linux") {
  $XB_Defs::IP_MULTICAST_IF = 32;
  $XB_Defs::IP_MULTICAST_TTL = 33;
  $XB_Defs::IP_MULTICAST_LOOP = 34;
  $XB_Defs::IP_ADD_MEMBERSHIP = 35;
  $XB_Defs::IP_DROP_MEMBERSHIP = 36;
  $XB_Defs::GATED_PID_PATH = "/var/run/";
};

####################################
# Initialize debug and error logging
# and print a banner line.
#
# See log::open() comments in /lib/XB_Log.pm
# Logging can go to four destinations:
#       XB_Defs::SYSLOG, XB_Defs::ERR,
#       XB_Defs::OUT and XB_Defs::FILE
####################################

if ($XB_Defs::LOG_MASK)
{
    print "\nOpening system log with mask: $XB_Defs::LOG_MASK\n\n";


    XB_Log::open ($XB_Defs::LOG_DEST, $XB_Defs::LOG_MASK, $XB_Defs::LOG_FILE);

    # For debugging, send everything to STDERR.
    # XB_Log::open (XB_Log::ERR, "err,warn,info,debug0,debug1,debug2,debug3");

}
else
{
    print "\nSystem logging defaulted to STDOUT.\n";
}


XB_banner_msg ();


####################################
# Check whether or not this version
# is obsolete and issue warning or
# die if it is very out of date.
####################################

$_ = XB_Utils::XB_obsolete();

if (!$_)
{
    my $tmp;

    $tmp = "****************************\n";
    $tmp .= "Xbone system is out of date more than XBONE_OBSOLETE_LIMIT.\n";
    $tmp .= "****************************\n\n";
    XB_Log::log ("warning", $tmp);
}
elsif ($_ == -1)
{
    my $tmp;

    $tmp =  "****************************\n";
    $tmp .= "Xbone system out of date well beyond XBONE_OBSOLETE_LIMIT.\n";
    $tmp .= "This version of the Xbone system is fatally obsolete.\n";
    $tmp .= "****************************\n\n";
    XB_Log::log ("err", $tmp);

    die;
}

# no longer needed to init srand, is done by IPsec::init and SSL::init
 
XB_IPsec::init();

# does SSL init:
XB_RPC::XB_init_network();

# Create a new socket to listen from commands from the GUI from

my ($inputflags, $errorflags, $iflg, $eflg) = ("", "", "", "");

my $sock = IO::Socket::INET->new(
	 Proto     => 'tcp',
	 LocalAddr => $XB_Defs::XBONE_OVERLAY_MANAGER,
         LocalPort => $XB_Defs::OVLMGR_PORT,
         Listen => SOMAXCONN,
	 Reuse => 1) or
   die "Fatal: Cannot open listen socket";
XB_Log::log ("info", "Opened TCP listen on $XB_Defs::XBONE_OVERLAY_MANAGER : $XB_Defs::OVLMGR_PORT\n");

vec ($inputflags, fileno ($sock), 1) = 1;
$errorflags = $inputflags;


#######################################
# Wait for a GUI command and process it
#######################################

LOOP: while (1)
{
    my ($peer, $port, $iaddr, $Command);
    my ($nfound, $timeleft, $timebefore, $timeout, $ping_pid, $child_pid);

    do
    {
	$timebefore = time();

	($nfound, $timeleft) = 
	    select ($iflg = $inputflags, undef, $eflg = $errorflags,
		    	$timeout = $XB_Defs::OVLMGR_PING_INTERVAL );

    } while (($nfound == 0) &&
	     ((time() - $timebefore) < $XB_Defs::OVLMGR_PING_INTERVAL) );


    ###############################
    # Reap any dead child processes
    # Background heartbeat (ping)
    # processes must be reaped.
    ###############################

    while (1)
    {
	$child_pid = waitpid (-1, WNOHANG);
	if (($child_pid == 0) || ($child_pid == -1))  {  last;  }

        XB_Log::log ("info", "Child process reaped: $child_pid\n");
    }

    if (vec ($eflg, fileno ($sock), 1))
    { 
      XB_Log::log ("err", "I/O error on GUI socket: $!."); 
	die ""; 
    }

    if ((time() - $timebefore) >= $XB_Defs::OVLMGR_PING_INTERVAL)
    {

	##################################
        # Fork the background pinging task
        ##################################

        XB_Log::log "debug1", "Starting ping task.\n";

	if ($ping_pid = fork())
	{				# Parent process returns here.
	}
	elsif (defined ($ping_pid))
	{				# Child ping task returns here.

	    my ($oidxptr, $oallocptr) =
		(\%overlay_index, \@overlay_allocation);

	    XB_ping_overlays (\%overlay_index, \@overlay_array,
			      \@overlay_allocation);

            XB_Log::log ("debug1", "Ping task exiting.\n");
	    exit (0);
	}
	else
	{
	  XB_Log::log ("err",
	    "Ping task could not be started.  Pinging is disabled.\n");
	}
    }


    if (vec ($iflg, fileno ($sock), 1))
    {
	my $client =
	    $sock->accept or die "Fatal: Failed in accept socket to GUI\n";

	$client->autoflush(1);

	my $other = inet_ntoa($client->peeraddr);
	unless ($other eq $XB_Defs::LOOPBACK_IP_ADDRESS)
	{
	  XB_Log::log "err",
	    "Attempted connection from $other " . gethostbyaddr($client->peeraddr, PF_INET) . "REJECTED\n";

	    $client->close or XB_Log::log "err", "Close failed\n";
	    next LOOP;
	}

	my $buf = "";
	my $commands = "";
	my $got_EOC = 0;
	my $byte_read;
	while (defined($byte_read = sysread($client, $buf, 5000))){
	  if($byte_read == 0){ last; }
	  $commands .= $buf;
	  if ($commands =~ /\bXboneEOC\b/) { 
	    $got_EOC = 1;
	    last;
	  }
        };
	if(not defined ($byte_read)){
	  # could be (1) socket close (2) more to come (3) unknown error
	  # but for the purpose of this loop, go to next iteration
	  next;
	}elsif($byte_read == 0){
	  XB_Log::log "warning", "socket closed before command is received";
	  next;
        }
	#while ($buf = <$client>)
	#{
	#    # if (ord($buf) == 4 ) { last;};
	#    $commands .= $buf;   
	#    $_ = $buf;
	#    if (/\bXboneEOC\b/) { last;};
	#};


	#######################################
	# Parse the commands
	#######################################

	my @line = split /\n/, $commands;
	$_ = $line[0];
	$return_string = "";

	if (/^Xbone Version=(\S+)/) {
	    if ($1 ne $XB_Defs::XBONE_PROTOCOL) { 
		$return_string = "Xbone Version=$XB_Defs::XBONE_PROTOCOL\n" .
		"XboneControl Command=CreateOverlay Transaction=Reply\n" .
		"XboneReturn Code=1 " .
		"Msg=\"Version mismatch in message from " .
		"GUI.\"\nXboneEOC";
		goto EXIT;
#		die "Fatal: version mismatch in message from GUI: $1\n"; 
	    }
	}
	else {
	    die "Fatal: Message from GUI does not have Xbone Version\n$commands\n";
	};
	
	$_ = $line[1];
	if (/^XboneControl\s*/)
	{
	    if (/Command=(\w\S+)/)
	    {
		$_=$1;
		$Command = $1;
              XB_Log::log "debug1", "$1 from GUI to Overlay Manager:\n";
              XB_Log::log "debug2", "$commands##########\n";

	      SWITCH:
		{
		    /^CreateOverlay$/ and do
		    {
			$return_string = xb_ovlmgr_create($commands);
			last SWITCH;
		    };

		    /^DestroyOverlay$/ and do
		    {
			$return_string = xb_ovlmgr_destroy($commands);
			last SWITCH;
		    };

		    /^DestroyAll$/ and do
		    {
			$return_string = xb_ovlmgr_destroyall($commands);
			last SWITCH;
		    };

		    /^ModifyOverlay$/ and do
		    {
			$return_string = xb_ovlmgr_modify($commands);
			last SWITCH;
		    };

		    /^ListOverlays$/ and do
		    {
			$return_string = xb_ovlmgr_listoverlay();
			last SWITCH;
		    };

		    /^OverlayStatus$/ and do
		    {
			$return_string = xb_ovlmgr_status($commands);
			last SWITCH;
		    };
		} 
	    }
	    else
	    {
		die "Fatal: Command attribute missing from XboneControl\n$line[0]\n";
	    };
	}
	else
	{
	    die "Fatal: Command string does not have XboneControl as first command\n$commands\n";
	};

      XB_Log::log "debug1", "$Command-Reply from OverlayManager to GUI\n";
      XB_Log::log "debug2", "$return_string##########\n";


      EXIT:
	print $client $return_string;

      XB_Log::log "debug3",  "About to close socket\n";
	$client->close or XB_Log::log "err", "$!: Close failed\n";
      XB_Log::log "debug3", "Socket closed\n";

    }


} # End of LOOP

XB_Log::log "err", "Did not expect to get here. There should be way out of that loop\n";
$sock->close or XB_Log::log "err", "$!: Close failed\n";




















