# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
# 
#
# ----------------------------------------------------------------------------
#  $RCSfile: XB_App_Start.pm,v $
#
# $Revision: 1.7 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:35 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Author: Oscar Ardaiz

package	XB_App_Start;
require 	Exporter;
@ISA 		= qw(Exporter);
@EXPORT 	= qw();
@EXPORT_OK	= qw(installapp deinstallapp executeapp killapp);

# XXX oscar's code does not work with these! fix.
# use strict;
# use sigtrap;

####################################################
# sub installapp
# Subroutine that installs apps at nodes (to set up an app-bone)
#
# input:
# $app_name name of the package 
# $app_version application-version
# $app_url url of the package $app_name-$app_version
# $app_os operating system of that package
output:
# ok if OK
# string_error if any error 
####################################################

$appinstallationpath = '/usr/local';

#####starting code...now

sub installapp (@)
{
    my @input =  @_;

    $app_type = shift @input;
    unless (@input)
    { return "Command application name needed."; }

    if ($app_type !~ m/^([a-zA-Z])+$/)
    { return "Command application name: $app_type is invalid"; }


    $app_version = shift @input;
    unless (@input)
    { return "Command application version needed."; }

    if ($app_version !~ m/^\d.\d$/)
    { return "Command application version: $app_version is invalid"; }

    $app_package_name = "$app_type-$app_version";

    # check if packages exists (pkg_add does so after fetching it,
    # we do not want it)
    # (we could check for other conditions when pkg_add fails,
    # and try that pkg_add is only called if it can succeed)

    if (!(system("/usr/sbin/pkg_info -e $app_package_name")))
    { return "ok"; }

    $app_package_url = shift @input;
    unless (@input)
    { return "Command application url needed" ; }

    if (@input)
    {
	$app_package_os = shift @input;

	open(OS, "uname -s|") ||   return "Can't exec uname -s $!";
	chop($node_sys = <OS>);
	close(OS);
	open(REL, "uname -r|") || return "Can't exec uname -r $!";
	chop($node_rel = <REL>);
	close(REL);
	$node_OS = "$node_sys-$node_rel";

	unless ($app_package_os =~ m/^$node_OS$/)
	{
	    return "Node OS: $node_OS differs from requested: $app_package_os";
	}
    }

###############
#exec pkg_add
# pkg_add is a program that inputs a URL and
# retrives a tar file, untars it to a tmp directory
# checks if it has been installed (installed programs are saved 
# in /var/db/pkg/)
# reads PLIST file in tar file for dependent packages that should be
# installed, and installs them
# and install the program according to PLIST directives and
# enviromental variables xx,yy, ...
###############

# bug of pkg_add (or not), if PKG_ADD_BASE not set it fails.

    $ENV{'PKG_ADD_BASE'} = '.';

    chdir "$appinstallationpath" ||
	die "couldn't chdir to installation dir: $appinstallationpath"; 

    if (system("/usr/sbin/pkg_add -I $app_package_url"))
    { 
	`/usr/sbin/pkg_delete $app_package_name`;
	return "could not be installed properly";
    }

    return "ok";
}




####################################################
# sub executeapp
# Subroutine that runs apps at nodes (to set up an app-bone)
#input:
# $app_type: application type, "squid", "anetd",...
# $app_id: unique app_id so that diferent application instances 
# in this node do not clash
# $app_binding_to: incoming and outgoing binding address for this
# application instance
# $squid_parent_node,$squid_sibling_node....:application type specific configurations
#return:
# \@app_pid: array of pid spawned and any children it may spawned
# of string_error if some error
##########################################################

# installapp will have to tell us,$appbinarypath $appbinary, 
# $appdefaultconfigfilepath $appdefaultconfigfile,
# but does it know which file is the binary? (~the only executable?),
# the config file?(->executeapp creates it all on the fly).

$appbinarypath = '/usr/local/sbin/';

$appdefaultconfigfilepath = '/usr/local/etc';

#appdatapath depends on local system
# anetd seems to take as its data path $HOME...

$appdatapath = '/usr/local/ovlappdata';

#IF KAME
#$netstat = '/usr/local/v6/bin/netstat';
#IF CAIRN
$netstat = '/usr/bin/netstat';

#####starting code...now

sub executeapp (@)
{
	my @input = @_;		

	unless (@input)
	{ return "Command has no application type"; }
	$app_type = shift @input;
	
	if ($app_type !~ m/^([a-zA-Z])+$/)
	{ return "Command application type: $app_type is invalid"; }

	unless (@input)
	{ return "Command has no application id"; }
	$app_id = shift @input;

	unless (@input)
	{ return "Command has no application binding"; }
	$app_binding_to = shift @input;

	@squid_sibling_node = ();
	$squid_parent_node = ();
	$i = 0;

	while (@input)
	{
	    if ($input[0] =~ m/^-app_neighbour_type_1$/)
	    {
		shift @input;
		unless (@input)
		{ return  "Command premature end of input"; }

		$squid_sibling_node[$i] = shift @input;
		$i += 1;
	    }
	    elsif ($input[0] =~ /^-app_neighbour_type_2$/)
	    {
		shift @input;
		unless (@input)
		{
		    return "Command premature end of input";
		}

		$squid_parent_node = shift @input;
	    } 
	    else
	    { return "Command Bad parameter $input[0]";	}	
	}


########################
###config execution enviroment
#####create some directory if necessary
###### change enviroment variables, 
#####or system conf files
##### and initialize some application data
#############################
#1############
#path for data directory of this application instance,
#we name the data directory as app_id/app_type,
#where app_id is usually ip_local_address

	$UID = "nobody";
	$GID = "nogroup";

	if (!(-s "$appdatapath"))
	{
	    (mkdir "$appdatapath", 040775)  ||
		return "Can't mkdir $appdatapath $!";

	    if (system("chmod 777 $appdatapath"))
	    { return "can chmod 777 $appdatapath $!"; }
	}
 
	if (!(-s "$appdatapath/$app_id"))
	{
	    (mkdir "$appdatapath/$app_id", 040775)  ||
		return "Can't mkdir $appdatapath/$app_id $!";

	    if (system("chmod 777 $appdatapath/$app_id"))
	    { return "can chmod 777 $appdatapath/$app_id/ $!"; }
	}

	chdir "$appdatapath/$app_id"  ||
	    return "Can't cddir $appdatapath/$app_id $!";

	if (!(-s "$app_type"))
	{ 
	    mkdir "$app_type",040775 ||
		return "Can't mkdir $app_type $!";

	    if (system("chmod 777 $appdatapath/$app_id/$app_type"))
	    { return "can chmod 777 $appdatapath$app_id/$app_type $!"; }
	}

        # squid requires that the cache and logs directories exists

	if ($app_type=~/^squid$/)
	{
	    chdir "$appdatapath/$app_id/$app_type"  ||
		return "Can't cddir $appdatapath/$app_id/$app_type $!";

	    if (!(-s "cache"))
	    { 
		(mkdir "cache", 040775) ||
		    return "Can't mkdir cache $!";

		if (system("chmod 777 $appdatapath/$app_id/$app_type/cache"))
		{ return "can chmod 777 $appdatapath$app_id/$app_type/cache $!"; }
	    }

	    if (!(-s "logs"))
	    { 
		(mkdir "logs", 040775) ||
		    return "Can't mkdir logs $!";

		if (system("chmod 777 $appdatapath/$app_id/$app_type/logs"))
		{ return "can chmod 777 $appdatapath$app_id/$app_type/logs $!";	}
	    }
	}

################

	elsif ($app_type=~/^anetd$/)
	{

# currently ad binds to IP address returned by
# gethostbyname (gethostname()), we have hack it to get the IP address we
# want this way: gethostbyname looks up where host.conf file tells it,
# so we change host.conf to look first to hosts file, and in hosts file,
# we write that localhostname=gethostname() has the IP address we want
# it to have.  #change host.conf file to look first at hosts file

	    open(ETCHOSTCONF,">/etc/host.conf") ||
		return "Can't open /etc/host.conf $!";
	    print ETCHOSTCONF "hosts\n";
	    print ETCHOSTCONF "bind\n";
	    close(ETCHOSTCONF);

	    use Sys::Hostname;

	    open(ETCHOSTS,">/etc/hosts") ||
		return "Can't open /etc/hosts $!";
#get hostname;
	    $hostname=hostname(); 
#add local ip address
	    print ETCHOSTS "127.0.0.1 localhost\n";
	    print ETCHOSTS "$app_binding_to $hostname\n";
	    print ETCHOSTS "$app_binding_to $hostname\.\n";
	    close(ETCHOSTS);
	}

#################

	if ($app_type=~/^squid$/)
	{
	    $appdefaultconffile="squid/squid.conf.default";

            #read squid.conf.default

	    open(INPUT,"<$appdefaultconfigfilepath/$appdefaultconffile") ||
		return "Can't input $appdefaultconfigfilepath/$appdefaultconffile $!";

	    $appconffile="$app_type.$app_id.conf";

            #create outputfile

	    open(OUTPUT,">$appdatapath/$app_id/$app_type/$appconffile") ||
		return "Can't output $appdatapath/$app_id/$app_type/$appconffile $!";

	    $appdefaultmimeconffile="squid/mime.conf.default";

            #read mime.conf.default

	    open(INPUT2,"<$appdefaultconfigfilepath/$appdefaultmimeconffile") ||
		return "Can't input $appdefaultconfigfilepath/$appdefaultmimeconffile $!";

	    $appmimeconffile="$app_type.$app_id.mime.conf";

            #create outputfile

	    open(OUTPUT2,">$appdatapath/$app_id/$app_type/$appmimeconffile") ||
		return "Can't output $appdatapath/$app_id/$app_type/$appmimeconffile $!";
	    close OUTPUT2;
	    close INPUT2;

            #print config info to file

            #add incomming_address outgoing address $ip_local_address  

	    print OUTPUT "tcp_incoming_address $app_binding_to\n";
	    print OUTPUT "tcp_outgoing_address $app_binding_to\n";

            #if child of, add cache_peer $parent_ip_address line

	    if ($squid_parent_node)
	    {	
		print OUTPUT "cache_peer $squid_parent_node parent 3128 3130 no-query \n";
	    }

            #if sibling of, add cache_peer $sibling_ip_address line	

	    if (@squid_sibling_node)
	    {
		foreach   (@squid_sibling_node)
		{ print OUTPUT "cache_peer $_ sibling 3128 3130 no-query \n"; }
	    }

            #add some filepaths,....

	    print OUTPUT "cache_mem 8 MB\n";
	    print OUTPUT "cache_dir $appdatapath/$app_id/squid/cache\t10\t4\t1\n";
	    print OUTPUT "cache_access_log $appdatapath/$app_id/squid/logs/access.log\n";
	    print OUTPUT "cache_log $appdatapath/$app_id/squid/logs/cache.log\n";
	    print OUTPUT "cache_store_log  $appdatapath/$app_id/squid/logs/store.log\n";
	    print OUTPUT "pid_filename $appdatapath/$app_id/squid/logs/squid.pid\n";
	    print OUTPUT "mime_table $appdatapath/$app_id/squid/$appmimeconffile\n";
	    print OUTPUT "icon_directory $appdefaultconfigfilepath/squid/icons\n";

            # print OUTPUT "cache_effective_user root\n";
            # print OUTPUT "cache_effective_group wheel\n";

	    print OUTPUT <INPUT>;
	    close OUTPUT;
	    close INPUT;
	}

        # elsif ($app_type=~/^anetd$/) {
        # when anetd accepts a binding address, it shall be input here as,
        # print OUTPUT "anetd_binding_address $app_binding_node\n";	 
        # }

##################
#finally squid needs to be executed with -z option,
#so that it creates its swaping space

	if ($app_type =~ /^squid$/)
	{
            # create cache swap directories by executing squid with -z option

	    $program = "$appbinarypath/squid";
	    @arg = ();
	    $args[0] =
		"-f$appdatapath/$app_id/$app_type/$app_type.$app_id.conf";
	    $args[1] = "-z";
	    (!(system("$program $args[0] $args[1]"))) or return "squid could not create swap directories";
	}

########################
###exec app
######################

	if  ($app_type=~/^squid$/)
	{
	    @args = ();
	    $args[0] =
		"-f$appdatapath/$app_id/$app_type/$app_type.$app_id.conf";
	    $program = "$appbinarypath/squid";
	}
	elsif ($app_type =~ /^anetd$/)
	{
	    # ad needs to be execute from its directory

	    (chdir "$appbinarypath") ||
		return "Can't chdir $appbinarypath $!";

            # anetd has to take as configuration enviromental variables, sig!
            # $ENV{'HOME'} = '$appdatapath/$app_id/$app_type';	

	    $program="./ad.bsd44";
	}   
  
	if (!defined($pid = fork))
	{ return "cannot fork: $!"; } 
	elsif ($pid)
	{   # parent

            # wait till app starts,	
            # find out pid of any process this child may spawn, 
            # find if it listing on the correct port,
            # and return ok, or kill possible running processes spawned.

	    sleep 1;	
	    $i = 0;
	    @app_pid = ();

	    open (PS, "ps -axjww|") or return "couldn't exec ps";

	    my $line;

	    while (defined($line = <PS>))
	    {
		my @pdata = split(" ", $line);

                #obtain pid of main program

		if ($line =~ /$program/)
		{
		    $app_pid[$i] = $pdata[1];
		    $i += 1;
		}

                # recursively find out pid of any program invoked from pid

		foreach (@app_pid)       
		{
		    if ($pdata[2] =~ /^$_$/)	
		    {
			$app_pid[$i]=$pdata[1];
			$i += 1;
		    }
		}
	    }

	    close PS;

	    use Socket;
	    unless ($app_binding_to =~ m/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/)
	    {
		($name,$aliases,$addrtype,$length,@addrs) =gethostbyaddr(inet_aton($app_binding_to),AF_INET);
	      $app_binding_to = inet_ntoa($addrs[0]);
	    }

	    $portsactive = "$netstat -n |grep -c $app_binding_to";

	    if ($app_type =~ /^anetd$/)
	    {
           
# we are going to wait till the programs has bind to the port it have to, 
# both tcp and udp ports,
# we find it out from from netstat -n |grep $app_binding_to.3322,
# the -n option is used since $app_binding_to are numeric IP address,	
# if during 6 sec netstat didnt found nothing we assume an error, 
# for anetd we will try to kill the process which might be running 

		$i=0;
		for (;;)
		{
		    sleep 2;
		    $i += 1;

		    if ((`$portsactive.3322`) > 1)
		    {last;}

		    if ($i > 3)
		    {
			foreach (@app_pid)
			{
			    if ($_ =~ /(\D)/)   # Taint mode fixup
			    { next; }
			    $_ = $1;

			    if (kill ("KILL", $_))
			    { return "could not kill misbehave anetd"; }
			}
			return
	     "app execution not recognized $i sec (killed if necesary)";
		    }	
		}

# change /etc/host.conf back, since some programs may misfunction in some cases,
# it is not very important, since first we should need to have made that hack,
# since programs should bind to the IP address that we should tell them.
	
		open(ETCHOSTCONF,">/etc/host.conf") || return "Can't open /etc/host.conf $!";
		print ETCHOSTCONF "bind\n";
		print ETCHOSTCONF "hosts\n";
		close(ETCHOSTCONF);
	    }
	    elsif ($app_type=~/^squid$/)
	    {

# we are going to wait till the programs has bind to the port it have to, 
# we find it out from from netstat -n |grep $app_binding_to.3128,	
# if during 6 sec netstat didn't find anything we assume an error

		$i = 0;
		for (;;)
		{
		    sleep 2;
		    $i += 1;

		    if ((`$portsactive.3128`) > 0)
		    {last;}

		    if ($i > 3)
		    {
			foreach (@app_pid)
			{
			    if ($_ =~ /(\D)/)   # Taint mode fixup
			    { next; }
			    $_ = $1;

			    if (kill ("KILL", $_))
			    { return "could not kill misbehaving squid"; }
			}
			return
    "app execution not recognized in $i seconds (killed if necessary)";
		    }	
		}
	    }

	    return ("ok", @app_pid);

	} #parent end.
       	    
	exec ($program, @options, @args)
	    || return "Can't execute program: $!";

	# NOTREACHED
       
}



####################################################
# sub killapp
# Subroutine that kills apps processes at nodes (to set up an app-bone)
#input:
# $app_type: application type, "squid", "anetd",...
# @app_pid_list: array of pids to be killed 
# or
# $app_id: unique app_id (then this packages mantains state)
# not desired...
#return
#"ok" if all went well
#string_error if some error
##########################################################
#do we want to delete its data?
# appdatapath depends on local system
# anetd seems to take as its data path $HOME...

$appdatapath='/usr/local/ovlappdata';

#####starting code...now

sub killapp (@)
{
    my @input = @_;

    if ($input[0] =~ m/^-app_name$/)
    {
	shift @input;
	unless (@input)
	{ return "Command premature end of input"; }

	$app_type = shift @input;
	if ($app_type =~ m/^([a-zA-Z])+$/)
	{ }
	else
	{ return "Command ApplicationType $app_type invalid"; }
    }
    else
    { return "Command has no app type";	}

    if ($input[0] =~ m/^-app_pid$/)
    {
	shift @input;
	unless (@input)
	{ return "Command premature end of input"; }

	$app_pid_list = shift @input;
    }	
    else
    { return "Command has no app id"; }

    @app_pid = split(" ", $app_pid_list);
    print("$app_pid_list,@app_pid");


    foreach (@app_pid)
    {
	if ($_ =~ /(\D)/)   # Taint mode fixup
	{ next; }
	$_ = $1;

	if (kill ("KILL", $_))
	{ return "kill of application process $_ failed"; }
    }
    return "ok";
}


####################################################
# sub deinstallapp
# Subroutine that deinstalls apps at nodes (to set up an app-bone)
#
# input:
# $app_name name of the package 
output:
# ok if OK
# error_string if any error 
####################################################

$appinstallationpath='/usr/local';

#####starting code...now

sub deinstallapp (@)
{
    my @input =  @_;

    if ($input[0] =~ /^-app_name$/)
    {
	shift @input;
	unless (@input)
	{ return "Command premature end of input"; }

	$app_type = shift @input;
	if ($app_type =~ m/^([a-zA-Z])+$/)
	{ }
	else
	{ return "Command applicationName $app_name is invalid"; }
    }
    else
    { return "Command has no app name";	}

    open (VERSION, "/usr/sbin/pkg_info |grep $app_type|") 
	or return "couldn't exec /usr/sbin/pkg_info";

    my $line;

    while (defined($line = <VERSION>))
    {
	print($line);
	($app_package_name,@nothing)=split(" ",$line);
    }

    close VERSION;
    if ((system("/usr/sbin/pkg_delete $app_package_name")))
    { return "package deinstallation failed"; }

    return "ok";
}

1;

