# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
# 
#
# ----------------------------------------------------------------------------
#  $RCSfile: XB_Commands.pm,v $
#
# $Revision: 1.24 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:36 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
#############################################
#
# Xbone host node command execution routines
#
# Principal author: Gregory Finn
#
#############################################

package XB_Commands;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(XB_ResourceDiscovery XB_CreateOverlay XB_IPv4_Allocation
             XB_IPv4_Deallocation XB_ListOverlays
	     XB_OverlayStatus XB_ModifyOverly XB_DestroyOverlay
	     XB_DestroyAll XB_dispatch_on_msg
	     XB_dispatch_on_nodecommand);
@EXPORT_OK = qw();

use strict;
use sigtrap;

use Data::Dumper;
use Net::Ping;

use XB_Defs;
use XB_Node_DB;
use XB_DNS;
use XB_App_Start;
use XB_Log;






###################################################################
#
# XB_dispatch_on_msg ( nodehref, mhref, canonical_peer )
#
# The arguments nodehref and mhref are the reference to the daemon
# state hash and a reference to the hash of the current message
# received from the overlay manager respectively.  The canonical
# peer name is the name of the overlay manager that issued the
# current message.
#
# The message must be that with XboneControl Command=CreateOverlay.
# The actions taken are those consistent with overly creation.
# The value returned is either the list (<code>, <command>, <msg>)
# used to construct a reply message that is sent back to the
# overlay manager, or the list (<msg>), in which case the entire
# correct reply message is already formatted as text.
###################################################################

sub XB_dispatch_on_msg ($$$)
{
    my ($nodehref, $mhref, $canonical_peer) = @_;
    my ($cmd, @result);

    XB_Log::log ("info", "->XB_dispatch_on_msg (@_)");

    $cmd = $mhref->{'XboneControl'}{'Command'};
    @result = (1, "XboneControl command: $cmd not recognized.");

    ###################################
    # Case dispatch on 'Command' field
    #
    # ResourceDiscovery command handled
    # by multicast reception code.
    ###################################

    if ($cmd eq 'CreateOverlay')
    {
	@result = XB_CreateOverlay ($nodehref, $mhref, $canonical_peer);
	goto EXIT;
    }
    elsif ($cmd eq 'IPv4_Allocation')
    {
	@result = XB_IPv4_Allocation ($nodehref, $mhref, $canonical_peer);
	goto EXIT;
    }
    elsif ($cmd eq 'IPv4_Deallocation')
    {
	@result = XB_IPv4_Deallocation ($nodehref, $mhref, $canonical_peer);
	goto EXIT;
    }
    elsif ($cmd eq 'ListOverlays')
    {
	@result = XB_ListOverlays ($nodehref, $mhref, $canonical_peer);
	goto EXIT;
    }
    elsif ($cmd eq 'OverlayStatus')
    {
	@result = XB_OverlayStatus ($nodehref, $mhref, $canonical_peer);
	goto EXIT;
    }
    elsif ($cmd eq 'ModifyOverlay')
    {
	@result = XB_ModifyOverlay ($nodehref, $mhref, $canonical_peer);
	goto EXIT;
    }
    elsif ($cmd eq 'DestroyOverlay')
    {
	@result = XB_DestroyOverlay ($nodehref, $mhref, $canonical_peer);
	goto EXIT;
    }
    elsif ($cmd eq 'DestroyAll')
    {
	@result = XB_DestroyAll ($nodehref, $mhref, $canonical_peer);
	goto EXIT;
    }
    elsif ($cmd eq 'DNSUpdate')
    {
	@result = XB_DNSUpdate ($nodehref, $mhref);
	goto EXIT;
    }

EXIT:
    XB_Log::log ("info", "<-XB_dispatch_on_msg (@result)");
    return @result;
}





###################################################################
#
# XB_ResourceDiscovery ( nodehref, mhref )
#
# The arguments nodehref and mhref are the reference to the daemon
# state hash and a reference to the hash of the current message
# received from the overlay manager respectively.
#
# The message must be that with XboneControl Command=ResourceDiscovery
# The returned value is the list form (<code>, <message>).
###################################################################

sub XB_ResourceDiscovery ($$)
{
    my ($nodehref, $mhref) = @_;
    my $ifacecnt;
    my (@result) = (0, "");

    XB_Log::log ("info", "->XB_ResourceDiscovery (@_)");

    XB_Log::log ("info", "<-XB_ResourceDiscovery (@result)");
    return @result;
}





###################################################################
#
# XB_CreateOverlay ( nodehref, mhref, canonical_peer )
#
# The arguments nodehref and mhref are the reference to the daemon
# state hash and a reference to the hash of the current message
# received from the overlay manager respectively.  The canonical
# peer name is the name of the overlay manager that issued the
# current message.
#
# The message must be that with XboneControl Command=CreateOverlay.
#
# The returned value must be acceptable to XB_dispatch_on_msg().
###################################################################

sub XB_CreateOverlay ($$$)
{
    my ($nodehref, $mhref, $canonical_peer) = @_;

    my ($ovlname, %ovlhash, @result, $aclhref, $ovlhref, @retval);

    XB_Log::log ("info", "->XB_CreateOverlay (@_)");

    ########################
    # Check to see if named
    # overlay already exists
    ########################

    XB_Log::log ("debug0", "Processing XboneControl command ...");

    $ovlname = $mhref->{'XboneOverlay'}{'Name'};

    ($aclhref, $ovlhref) = (XB_find_overlay ($nodehref,
					     $ovlname, $canonical_peer));
    if ($aclhref)
    {
	@retval = (1, "Xbone overlay $ovlname already exists.");
	goto EXIT;
    };

    ######################
    # Check access control
    # Find matching ACL
    ######################

    @result = XB_check_access ($nodehref, $mhref);
    if (!$result[0])
    {
	@retval = (1, $result[1]);
	goto EXIT;
    }
    else { $aclhref = $result[1]; };

    ######################
    # Check message syntax
    ######################

    if (!defined ($mhref->{'XboneOverlay'}{'Name'}))
    {
	@retval = (1, "XboneOverlay Name missing from message.");
	goto EXIT;
    }

    if (!defined ($mhref->{'XboneCreator'}{'Name'}))
    {
	@retval = (1, "XboneCreator Name missing from message.");
	goto EXIT;
    }

    if (!defined ($mhref->{'XboneCreator'}{'Email'}))
    {
	@retval = (1, "XboneCreator Email missing from message.");
	goto EXIT;
    }

    #######################
    # Check creation limits
    #######################

    if ($nodehref->{'OverlayCount'} >= $nodehref->{'MaxOverlayCount'})
    {
	@retval = (1, "Creator will exceed maximum node overlay count.");
	goto EXIT;
    }

    if ($nodehref->{'TunnelCount'} >= $nodehref->{'MaxTunnelCount'})
    {
	@retval = (1, "Creator will exceed maximum node tunnel count.");
	goto EXIT;
    }

    if ($aclhref->{'OverlayCount'} >= $aclhref->{'MaxOverlayCount'})
    {
	@retval = (1, "Creator will exceed maximum ACL overlay count.");
	goto EXIT;
    }

    if ($aclhref->{'TunnelCount'} >= $aclhref->{'MaxTunnelCount'})
    {
	@retval = (1, "Creator will exceed maximum ACL tunnel count.");
	goto EXIT;
    }

    if ($aclhref->{'AccessLevel'} !~ /deploy/i)
    {
	@retval = (1, "Creator exceeded ACL access level.");
	goto EXIT;
    }

    ####################
    # Create the overlay
    ####################

    XB_init_nodeovl (\%ovlhash);

    $ovlhash{'OverlayName'} = $mhref->{'XboneOverlay'}{'Name'};
    $ovlhash{'OverlayManager'} = $canonical_peer;
    $ovlhash{'CreatorName'} = $mhref->{'XboneCreator'}{'Name'};
    $ovlhash{'Email'} = $mhref->{'XboneCreator'}{'Email'};

    XB_increment_overlaycount ($nodehref, $aclhref);

    if (!defined (@{$aclhref->{'Overlays'}}))   # Create empty array if none there.
    { $aclhref->{'Overlays'} = []; };

    push (@{$aclhref->{'Overlays'}}, \%ovlhash);

    ########################################
    # Execute XboneNodeCommand sub-commands.
    # If the nodecommands fail we delete
    # this overlay to restore state back to
    # what it was.
    ########################################

    @retval = XB_dispatch_on_nodecommand ($nodehref, $mhref, $aclhref, \%ovlhash,
			     [ 'tunnel', 'route', 'ipsec' ] );

    if ($retval[0])
    {
	XB_overlay_delete ( $nodehref, $aclhref, \%ovlhash );
    }
    else
    {
	##########################
	# We have a new overlay.
	# Update the daemon state.
	##########################

	my $dmpmsg = Dumper ($nodehref);
        XB_Log::log ("debug0", "FINAL STATE:\n$dmpmsg");

	XB_write_state ($nodehref);
    };

EXIT:
    XB_Log::log ("info", "<-XB_CreateOverlay (@retval)");
    return @retval;
}





###################################################################
#
# XB_IPv4_Allocation ( nodehref, mhref, canonical_peer )
#
# The arguments nodehref and mhref are the reference to the daemon
# state hash and a reference to the hash of the current message
# received from the overlay manager respectively.  The canonical
# peer name is the name of the overlay manager that issued the
# current message.
#
# The message must be that with XboneControl Command=IPv4_Allocation.
#
# The returned value must be acceptable to XB_dispatch_on_msg().
###################################################################

sub XB_IPv4_Allocation ($$$)
{
    my ($nodehref, $mhref, $canonical_peer) = @_;
    my (@result);

    my $args = "$nodehref, $mhref";

    XB_Log::log ("info", "->XB_IPv4_Allocation (@_)");


#### Ankur's action routine


    XB_Log::log ("info", "<-XB_IPv4_Allocation (@result)");
    return @result;
}





###################################################################
#
# XB_IPv4_Deallocation ( nodehref, mhref, canonical_peer )
#
# The arguments nodehref and mhref are the reference to the daemon
# state hash and a reference to the hash of the current message
# received from the overlay manager respectively.  The canonical
# peer name is the name of the overlay manager that issued the
# current message.
#
# The message must be that with XboneControl Command=IPv4_Deallocation.
#
# The returned value must be acceptable to XB_dispatch_on_msg().
###################################################################

sub XB_IPv4_Deallocation ($$$)
{
    my ($nodehref, $mhref, $canonical_peer) = @_;
    my (@result);

    XB_Log::log ("info", "->XB_IPv4_Deallocation (@_)");


#### Ankur's action routine


    XB_Log::log ("info", "<-XB_IPv4_Deallocation (@result)");
    return @result;
}




###################################################################
#
# XB_ListOverlays ( nodehref, mhref, canonical_peer )
#
# The arguments nodehref and mhref are the reference to the daemon
# state hash and a reference to the hash of the current message
# received from the overlay manager respectively.  The canonical
# peer name is the name of the overlay manager that issued the
# current message.
#
# The message must be that with XboneControl Command=ListOverlays.
#
# The returned value must be acceptable to XB_dispatch_on_msg().
###################################################################

sub XB_ListOverlays ($$$)
{
    my ($nodehref, $mhref, $canonical_peer) = @_;
    my (@result, $namelref, $args);

    $args = "$nodehref, $mhref";
    if (defined ($canonical_peer))  { $args .= ", $canonical_peer"; }

    XB_Log::log ("info", "->XB_ListOverlays ($args)");

    @result = (0, "Xbone Version=$XB_Defs::XBONE_PROTOCOL\n");

    $result[1] .= "XboneControl Command=ListOverlays Transaction=Reply\n";

    foreach $namelref ( XB_overlay_names(\%XB_Defs::DAEMON_STATE) )
    {
	if ($canonical_peer && ($namelref->[1] eq $canonical_peer))
	{  $result[1] .= "XboneOverlay Name=<$namelref->[0], $namelref->[1]>\n";  }
	else
	{  $result[1] .= "XboneOverlay Name=<$namelref->[0], $namelref->[1]>\n";  }
    };

    $result[1] .= "XboneEOC\n";

    XB_Log::log ("info", "<-XB_ListOverlays (@result)");
    return @result;
}





###################################################################
#
# XB_OverlayStatus ( nodehref, mhref, canonical_peer )
#
# The arguments nodehref and mhref are the reference to the daemon
# state hash and a reference to the hash of the current message
# received from the overlay manager respectively.  The canonical
# peer name is the authenticated name of the overlay manager that
# issued the current message.
#
# The message must be that with XboneControl Command=OverlayStatus.
#
# The returned value must be acceptable to XB_dispatch_on_msg().
###################################################################

sub XB_OverlayStatus ($$$)
{
    my ($nodehref, $mhref, $canonical_peer) = @_;
    my ($msg, $ovlname, $aclhref, $ovlhref, $ifaces, @retval);


    XB_Log::log ("info", "->XB_OverlayStatus (@_)");

    XB_Log::log ("debug0", "Executing XboneControl command ...\n");


    $ovlname = $mhref->{'XboneOverlay'}{'Name'};

    ($aclhref, $ovlhref) = XB_find_overlay ( $nodehref,
					    $ovlname, $canonical_peer );
    if (!$aclhref)
    {
	@retval = (1,
	  "Overlay $ovlname that was created by $canonical_peer was not found.");

        XB_Log::log ("info", "<-XB_OverlayStatus (@retval)");
	return @retval;
    }
    else
    {
	my @tdata = $ovlhref->{'Tunnel_Tags'};

	$msg = "Xbone Version=$XB_Defs::XBONE_PROTOCOL ";
	$msg .= "Release=$XB_Defs::XBONE_RELEASE";
	$msg .= "\n";

	$msg .= "XboneControl Command=OverlayStatus Transaction=Reply";
	$msg .= "\n";

	$msg .= "XboneOverlay Name=$ovlname";
	$msg .= "\n";

	$msg .= "XboneSecurity UserId=$mhref->{'XboneSecurity'}{'UserId'} ";
	$msg .= "AuthType=$mhref->{'XboneSecurity'}{'AuthType'}";
	$msg .= "\n";

	$msg .= "XboneNode Class=$nodehref->{'Class'} ";
	$msg .= "OS=$nodehref->{'OS'} ";
	$ifaces = $nodehref->{'MaxTunnelCount'} - $nodehref->{'TunnelCount'};
	$msg .= "InterfaceCount=$ifaces ID=$nodehref->{'ID'} ";
	$msg .= "IPaddr=$nodehref->{'IPaddr'} ";
	$msg .= "Status=in";
	$msg .= "\n";

	if (scalar (@tdata))
	{
	    #######################################
	    # Execute XboneNodeCommand sub-commands
	    #######################################

	    @retval = XB_dispatch_on_nodecommand ($nodehref, $mhref,
						  $aclhref, $ovlhref,
						  [ 'ping' ] ); 
	
	    if (!$retval[0])	   # Error should not happen
	    { $msg .= "$retval[1]"; }
	}
   };

    $msg .= "XboneReturn Code=0 Msg=\"\"\n";

    $msg .= "XboneEOC\n";

    XB_Log::log ("info", "<-XB_OverlayStatus ($msg)");
    return ($msg);
}





###################################################################
#
# XB_ModifyOverlay ( nodehref, mhref, canonical_peer )
#
# The arguments nodehref and mhref are the reference to the daemon
# state hash and a reference to the hash of the current message
# received from the overlay manager respectively.  The canonical
# peer name is the name of the overlay manager that issued the
# current message.
#
# The message must be that with XboneControl Command=ModifyOverlay.
#
# The returned value must be acceptable to XB_dispatch_on_msg().
###################################################################

sub XB_ModifyOverlay ($$$)
{
    my ($nodehref, $mhref, $canonical_peer) = @_;

    my @result = (1, "XboneControl Command=ModifyOverlay not implemented.");

    XB_Log::log ("info", "->XB_ModifyOverlay (@_)");

    XB_Log::log ("info", "<-XB_ModifyOverlay (@result)");
    return @result;
}





###################################################################
#
# XB_DestroyOverlay ( nodehref, mhref, canonical_peer )
#
# The arguments nodehref and mhref are the reference to the daemon
# state hash and a reference to the hash of the current message
# received from the overlay manager respectively.  The canonical
# peer name is the name of the overlay manager that issued the
# current message.
#
# The message must be that with XboneControl Command=DestroyOverlay.
#
# The returned value must be acceptable to XB_dispatch_on_msg().
###################################################################

sub XB_DestroyOverlay ($$$)
{
    my ($nodehref, $mhref, $canonical_peer) = @_;
    my ($aclhref, $ovlhref, $ovlname);
    my @result = (0, "");

    XB_Log::log ("info", "->XB_DestroyOverlay (@_)");

    XB_Log::log ("debug0", "Executing XboneControl command ...");

    ##############################
    # Find designated overlay hash
    ##############################

    $ovlname = $mhref->{'XboneOverlay'}{'Name'};
    ($aclhref, $ovlhref) = XB_find_overlay ($nodehref, $ovlname,
					    $canonical_peer);
    if (!$aclhref)
    {
	$result[0] = 1;
	$result[1] = "Overlay <$ovlname, $canonical_peer> not found";
	goto EXIT;
    };

    ####################
    # Check access level
    ####################

    if ($aclhref->{'AccessLevel'} !~ /deploy/i)
    {
	$result[0] = 1;
	$result[2] = "Command exceeded user's ACL access level.";
	goto EXIT;
    }

    #######################################
    # Execute XboneNodeCommand sub-commands
    #######################################

    @result = XB_dispatch_on_nodecommand ($nodehref, $mhref, $aclhref,
					  $ovlhref, [ 'overlay' ] );

    if ($result[0])		# No state change occurred.
    { goto EXIT; };

    #######################
    # Update internal state
    #######################

    XB_write_state ($nodehref);

EXIT:
    XB_Log::log ("info", "<-XB_DestroyOverlay (@result)");
    return @result;
}




###################################################################
#
# XB_DestroyAll ( nodehref, mhref, canonical_peer )
#
# The arguments nodehref and mhref are the reference to the daemon
# state hash and a reference to the hash of the current message
# received from the overlay manager respectively.  The canonical
# peer name is the name of the overlay manager that issued the
# current message.
#
# The message must be that with XboneControl Command=DestroyAll.
#
# The returned value must be acceptable to XB_dispatch_on_msg().
###################################################################

sub XB_DestroyAll ($$$)
{
    my ($nodehref, $mhref, $canonical_peer) = @_;
    my @result = (0, "");

    XB_Log::log ("info", "->XB_DestroyAll (@_)");

    XB_Log::log ("debug0", "Executing XboneControl command ...");

    #######################################
    # Execute XboneNodeCommand sub-commands
    #######################################

    @result = XB_dispatch_on_nodecommand ($nodehref, $mhref, $canonical_peer,
					  undef, [ 'overlay' ] );

    if ($result[0])		# No state change occurred.
    { goto EXIT; };

    #######################
    # Update internal state
    #######################

    XB_write_state ($nodehref);

EXIT:
    XB_Log::log ("info", "<-XB_DestroyAll (@result)");
    return @result;
}





###################################################################
#
# XB_DNSUpdate ( nodehref, mhref )
#
# The arguments nodehref and mhref are the reference to the daemon
# state hash and a reference to the hash of the current message
# received from the overlay manager respectively.
#
# The message must be that with XboneControl Command=DestroyAll.
#
# The returned value must be acceptable to XB_dispatch_on_msg().
###################################################################

sub XB_DNSUpdate ($$)
{
    my ($nodehref, $mhref) = @_;
    my @result = (0, "");

    XB_Log::log ("info", "->XB_DNSUpdate (@_)");

    XB_Log::log ("debug0", "Executing XboneControl command ...\n");

    #######################################
    # Execute XboneNodeCommand sub-commands
    #######################################

    @result = XB_dispatch_on_nodecommand ($nodehref, $mhref, undef,
					  undef, [ 'dns' ] );

    if ($result[0])		# No state change occurred.
    { goto EXIT; };

    XB_write_state ($nodehref);

EXIT:
    XB_Log::log ("info", "<-XB_DNSUpdate (@result)");
    return @result;
}







###################################################################
#
# XB_dispatch_on_nodecommand ( nodehref, mhref, aclhref, ovlhref, vlref )
#
# This routine executes the XboneNodeCommand lines in a command message
# received from the overlay manager.
#
# A normal return is (0, "").  If an error occured, the return will
# be (1, message).
###################################################################

sub XB_dispatch_on_nodecommand ($$$;$$)
{
    my ($nodehref, $mhref, $aclhref, $ovlhref, $vlref) = @_;

    my ($args, $vlist, $line) = ("", "", "");
    my @retval = (0, "");


    $args = "$nodehref, $mhref";
    if (!defined $aclhref)
    {
	$args .= ", undef";
    };
    if (!defined $ovlhref)
    {
	$args .= ", undef";
    };
    if (!defined $vlref)
    {
	$vlref = "";
	$args .= ", undef";
    };

    XB_Log::log ("info", "->XB_dispatch_on_nodecommand ($args)");

    if ($vlref) { $vlist = "@$vlref"; };     # List of allowed verbs

    foreach $line (@{$mhref->{'XboneNodeCommands'}})
    {
################
# IPSEC
################
	if ($line =~ s/^\s*ipsec\s*//i)
	{
	    if ($vlref && ($vlist !~ /ipsec/i))
	    {
		@retval = (1,
    "ipsec verb not allowed in $mhref->{'XboneControl'}{'Command'} command.");
		last;
	    };

	    @retval = XB_ncmd_ipsec ($nodehref, $mhref, $aclhref, $ovlhref, $line);
	    if ($retval[0])  { last; };
	}
################
# OVERLAY
################
	elsif ($line =~ s/^\s*overlay\s*//i)
	{
	    if ($vlref && ($vlist !~ /overlay/i))
	    {
		@retval = (1,
    "overlay verb not allowed in $mhref->{'XboneControl'}{'Command'} command.");
		last;
	    };

	    @retval = XB_ncmd_overlay ($nodehref, $mhref, $aclhref,
				       $ovlhref, $line);
	    if ($retval[0])  { last; };
	}
################
# ROUTE
################
	elsif ($line =~ s/^\s*route\s*//i)
	{
	    if ($vlref && ($vlist !~ /route/i))
	    {
		@retval = (1,
    "route verb not allowed in $mhref->{'XboneControl'}{'Command'} command.");
		last;
	    };

	    @retval = XB_ncmd_route ($nodehref, $mhref, $aclhref, $ovlhref, $line);
	    if ($retval[0])  { last; };
	}
################
# DNS
################
	elsif ($line =~ s/^\s*dns\s*//i)
	{
	    if ($vlref && ($vlist !~ /dns/i))
	    {
		@retval = (1,
    "dns verb not allowed in $mhref->{'XboneControl'}{'Command'} command.");
		last;
	    };

	    @retval = XB_ncmd_dns ($nodehref, $mhref, $aclhref, $line);
	    if ($retval[0])  { last; };
	}
################
# TUNNEL
################
	elsif ($line =~ s/^\s*tunnel\s*//i)
	{
	    if ($vlref && ($vlist !~ /tunnel/i))
	    {
		@retval = (1,
    "tunnel verb not allowed in $mhref->{'XboneControl'}{'Command'} command.");
		last;
	    };

	    @retval = XB_ncmd_tunnel ($nodehref, $mhref, $aclhref, $ovlhref,
				      $line);
	    if ($retval[0])  { last; };
	}
################
# APPLICATION
################
	elsif ($line =~ s/^\s*application\s*//i)
	{
	    if ($vlref && ($vlist !~ /application/i))
	    {
		@retval = (1,
"application verb not allowed in $mhref->{'XboneControl'}{'Command'} command.");
		last;
	    };

	    @retval = XB_ncmd_application ($nodehref, $mhref, $aclhref,
					   $ovlhref, $line);
	    if ($retval[0])  { last; };
	}
################
# PING
################
	elsif ($line =~ s/^\s*ping\s*//i)
	{
	    if ($vlref && ($vlist !~ /ping/i))
	    {
		@retval = (1,
    "ping verb not allowed in $mhref->{'XboneControl'}{'Command'} command.");
		last;
	    };

	    @retval = XB_ncmd_ping ($nodehref, $mhref, $aclhref, $ovlhref);
	    if ($retval[0])  { last; };
	}
################
# HEARTBEAT
################
	elsif ($line =~ s/^\s*heartbeat\s*//i)
	{
	    if ($vlref && ($vlist !~ /heartbeat/i))
	    {
		@retval = (1,
    "ping verb not allowed in $mhref->{'XboneControl'}{'Command'} command.");
		last;
	    };

	    @retval = XB_ncmd_heartbeat ($nodehref, $mhref, $aclhref, $ovlhref);
	    if ($retval[0])  { last; };
	}
	else
	{
	    @retval = (1, "Unrecognized command: XboneNodeCommand $line.");
	    goto EXIT;
	}
    };


EXIT:
    XB_Log::log ("info", "<-XB_dispatch_on_nodecommand (@retval)");
    return @retval;
}





###################################################################
#
# XB_ncmd_ipsec ( nodehref, mhref, aclhref, ovlhref, line )
#
###################################################################

sub XB_ncmd_ipsec ($$$$$)
{
    my ($nodehref, $mhref, $aclhref, $ovlhref, $line) = @_;

    my @args = ();
    my ($tmp, $ix, @callargs);
    my @retval = (0, "");


    XB_Log::log ("info", "->XB_ncmd_ipsec (@_)");

    XB_Log::log ("debug0", "command=IPsec");

    if ($line =~ s/^\s*add\s*//)
    {
	#####################
	# Command:  IPsec add
	#####################

	XB_Log::log ("debug0", "command=IPsec add $line");

	@args = split /\s/, $line;

	####################################################
	# We activate IPsec in both forward and reverse
	# directions.  Args passed to us are:
	# ( src ip, dst ip, spi,
	#   fa, fa_key, fe, fe_key, ba, ba_key, be, be_key )
	#
	# Calls to IPsec::add() are:
	# ( src ip, dst ip, "out", spi, fa, fa_key, fe, fe_key ) and
	# ( dst ip, src ip, "in", spi, ba, ba_key, be, be_key )
	# where spi, keys must be "0x..." hex strings and we must
	# substitute for any "undef" string arg the Perl undef value.
	#####################################################

        #################################
	# Forward tunnel is OUT direction
	#################################

	@callargs = ($args[0], $args[1], "out",
		     $args[2], $args[3], $args[4], $args[5], $args[6]);

	XB_push_overlay_state ($nodehref, $ovlhref, 'IPsec',
           "$args[0] $args[1] out $args[2] $args[3] $args[4] $args[5] $args[6]");

	XB_Log::log ("debug0", "XB_IPsec::add (@callargs)");

	for ($ix = 3;  $ix <= 7;  $ix++)   # "undef" substitution.
	{
	    if ($callargs[$ix] =~ /undef/i)
	    { undef ($callargs[$ix]); };
	};

	eval { $tmp = XB_IPsec::add ($callargs[0], $callargs[1], $callargs[2],
				     $callargs[3], $callargs[4], $callargs[5],
				     $callargs[6], $callargs[7]) };
	if ($@ || (!$tmp))
	{
            XB_Log::log ("debug0", "**** FAILURE.");

	    return (1, "IPsec command XB_IPsec::add call failure");
	};

        ###########################################
	# Now reverse tunnel, which is IN direction
	###########################################

	@callargs = ($args[1], $args[0], "in",
		     $args[2], $args[7], $args[8], $args[9], $args[10]);

	XB_push_overlay_state ($nodehref, $ovlhref, 'IPsec',
          "$args[1] $args[0] in $args[2] $args[7] $args[8] $args[9] $args[10]");

	XB_Log::log ("debug0", "XB_IPsec::add (@callargs)");

	for ($ix = 3;  $ix <= 7;  $ix++)   # "undef" substitution.
	{
	    if ($callargs[$ix] =~ /undef/i)
	    { undef ($callargs[$ix]); };
	};

	eval { $tmp = XB_IPsec::add ($callargs[0], $callargs[1], $callargs[2],
				     $callargs[3], $callargs[4], $callargs[5],
				     $callargs[6], $callargs[7]) };
	if ($@ || (!$tmp))
	{
	    XB_Log::log ("debug0", "**** FAILURE.");
    
	    @retval = (1, "IPsec command XB_IPsec::add call failure");
	    goto EXIT;
	};
    }
    elsif ($line =~ s/^\s*delete\s*//)
    {
	########################
	# Command:  IPsec delete
	########################

	XB_Log::log ("debug0", "command=IPsec delete $line");

	@args = split /\s/, $line;

	##################################################
	# Args passed to us are: ( src ip, dst ip, spi )
	#
	# Calls to IPsec:delete() are:
	# ( src ip, dst ip, "out", spi ) and 
	# ( dst ip, src ip, "in", spi )
	#
	# HOWEVER: Cannot assume particular src ip, dst ip
	# ordering from OM.  Must use spi to match with
        # previous IPsec::add() calls to determine that
	# to make correct "in" and "out" associations.
	##################################################

	foreach $tmp (@{$ovlhref->{'IPsec'}})
	{
	    @callargs = split /\s+/, $tmp;

	    if ($args[2] != $callargs[3])
	    {
	        XB_Log::log ("debug0", "**** FAILURE.");

		@retval = (1, "IPsec delete ccommand failure: spi mismatch.");
		goto EXIT;
	    };

	    eval { $tmp = XB_IPsec::delete ($callargs[0], $callargs[1],
					    $callargs[2], $callargs[3]) };

	    if ($@ || (!$tmp))
	    {
	        XB_Log::log ("debug0", "**** FAILURE.");

		@retval = (1, "IPsec command XB_IPsec::delete call failure");
		goto EXIT;
	    };
	}

      }

EXIT:
    XB_Log::log ("info", "<-XB_ncmd_ipsec (@retval)");
    return @retval;
}





###################################################################
#
# XB_ncmd_overlay ( nodehref, mhref, canonical_peer, ovlhref, line )
#
###################################################################

sub XB_ncmd_overlay ($$$$$)
{
    my ($nodehref, $mhref, $canonical_peer, $ovlhref, $line) = @_;
    my @retval = (0, "");  my (@args, $args, $aclhref);

    $args = "$nodehref, $mhref";
    if (defined ($ovlhref)) { $args .= ", $ovlhref"; };
    if (defined ($line)) { $args .= ", $line"; };
    XB_Log::log ("info", "->XB_ncmd_overlay ($args)");

    XB_Log::log ("debug0", "command=overlay\n");

    @args = split /\s+/, $line;

    if ($args[0] =~ /delete/i)
    {
	###########################
	# Command:  overlay delete
	###########################

	XB_Log::log ("debug0", "Command=overlay delete $line\n");

	($aclhref, $ovlhref) = XB_find_overlay ($nodehref, $args[1],
						$canonical_peer);

	if (!$ovlhref)
	{
	    XB_Log::log ("warning",
			 "Overlay <$args[1], $canonical_peer> not found.\n");
	    @retval = (1, "Overlay delete command: Overlay not found.");
	    goto EXIT;
	};

	if (!XB_overlay_delete ($nodehref, $aclhref, $ovlhref))
	{
	    @retval = (1, "Overlay delete command: Deletion failure.");
	    goto EXIT;
	};
    }
    elsif ($args[0] =~ /killall/i)
    {
	##################################
	# Command:  overlay killall
	#
	# For this command, the argument
	# aclhref holds the canonical
	# overlay manager name.
	##################################

	XB_Log::log ("debug0", "Command=overlay killall $line\n");

	XB_delete_overlays ($nodehref, $canonical_peer);
    }

EXIT:
    XB_Log::log ("info", "<-XB_ncmd_overlay (@retval)");
    return @retval;
}





###################################################################
#
# XB_ncmd_route ( nodehref, mhref, aclhref, ovlhref, line )
#
###################################################################

sub XB_ncmd_route ($$$$$)
{
    my ($nodehref, $mhref, $aclhref, $ovlhref, $line) = @_;

    my (@args, $ovlname);
    my @retval = (0, "");


    XB_Log::log ("info", "->XB_ncmd_route (@_)");
    XB_Log::log ("debug0", "Executing route command (@args)");

    $ovlname = $mhref->{'XboneOverlay'}{'Name'};

    if ($line =~ s/^\s*add\s+//i)   # Specific "route add"
    {
	#####################
	# Command:  route add
	#####################

	@args = split /\s+/, $line;

	XB_Log::log ("debug0", "add_route (@args)");

	if (!XB_Route::add_route (@args))
	{
	      XB_Log::log ("debug0", "**** FAILURE: route add @args");

	      @retval = (1, "XB_Route::add_route() in XB_ncmd_route().");
	      goto EXIT;
	};

	######################################
	# "route add" commands for this node are
	# turned into a string of corresponding
	# "route delete" commands for later takedown.
	######################################
	
	XB_push_overlay_state ($nodehref, $ovlhref, 'Routes', "@args");

	XB_Log::log ("debug0", "SAVING delete: $ovlname @args <XXX>");

    }
    elsif ($line =~ s/^\s*delete\s+//i)   # Specific "route delete"
    {
	########################
	# Command:  route delete
	########################

	@args = split /\s+/, $line;

	XB_Log::log ("debug0", "delete_route @args");

	if (!XB_Route::get_route (@args))
	{
	    XB_Log::log ("debug0", "Route @args already deleted.");
	}
	else
	{
	    if (!XB_Route::delete_route (@args))
	    {
		  XB_Log::log ("debug0", "**** FAILURE: route delete @args");

		  @retval = (1, "XB_Route::delete_route() in XB_ncmd_route().");
		  goto EXIT;
	    }
	}
    };

EXIT:
    XB_Log::log ("info", "->XB_ncmd_route @retval");
    return @retval;
}




###################################################################
#
# XB_ncmd_dns ( nodehref, mhref, aclhref, line )
#
###################################################################

sub XB_ncmd_dns ($$$$)
{
    my ($nodehref, $mhref, $aclhref, $line) = @_;
    my ($retval, $args);
    my @result = (0, "");


    
    $args = "$nodehref, $mhref";
    if (!defined ($aclhref))
    {  $args .= ", undef";  };

    XB_Log::log ("info", "->XB_ncmd_dns ($args)");

    XB_Log::log ("debug0", "command=dns");

    if ($line =~ s/^\s*add\s*//)
    {
	###################
	# Command:  dns add
	###################

	XB_Log::log ("debug0", "command=dns add");

	push (@XB_Defs::DNS_ADD_FWD_PARMS, $line);
    }
    elsif ($line =~ s/^\s*delete\w*\s*//)
    {
	######################
	# Command:  dns delete
	######################

	XB_Log::log ("debug0", "command=dns delete\n");

	push (@XB_Defs::DNS_DEL_FWD_PARMS, $line);
    }
    elsif ($line =~ s/^\s*commit\s*//)
    {
	#######################
	# Command:  dns  commit
	#######################

	XB_Log::log ("debug0", "command=dns commit");

	$retval = XB_commit_dns_database ();

	@XB_Defs::DNS_ADD_FWD_PARMS = ();
	@XB_Defs::DNS_DEL_FWD_PARMS = ();

	if ($retval)
	{
	    @result = (1, "DNS commit failure: $retval");
	    goto EXIT;
	};
    };

EXIT:
    XB_Log::log ("info", "<-XB_ncmd_dns (@result)");
    return @result;
}





###################################################################
#
# XB_ncmd_tunnel ( nodehref, mhref, aclhref, ovlhref, line )
#
###################################################################

sub XB_ncmd_tunnel ($$$$$)
{
    my ($nodehref, $mhref, $aclhref, $ovlhref, $line) = @_;

    my (@args, $tag);
    my @result = (0, "");

    XB_Log::log ("info", "->XB_ncmd_tunnel (@_)");

    if ($line =~ s/^\s*add\s*//)
    {
	######################
	# Command:  tunnel add
	######################

	@args = split /\s+/, $line;

	XB_Log::log ("debug0",
           "XB_Tunnel::up ( $args[0], $args[1], $args[2], $args[3], $args[4] )\n");

	eval { $tag = XB_Tunnel::up ($args[0], $args[1], $args[2],
				  $args[3], $args[4]) };
	if ($@)
	{
	    @result = (1, "XB_Tunnel::up() failure in XB_ncmd_tunnel()");
	    goto EXIT;
	};

	XB_push_overlay_state ( $nodehref, $ovlhref, 'Tunnel_Tags',
		      "$tag $args[0] $args[1] $args[2] $args[3] $args[4]" );

	XB_increment_tunnelcount ($nodehref, $aclhref);
    };

EXIT:
    XB_Log::log ("info", "<-XB_ncmd_tunnel (@result)");
    return @result;
}





###################################################################
#
# XB_ncmd_ping ( nodehref, mhref, aclhref, ovlhref )
#
# Performs pings protocol probes of the other ends of the tunnels
# associated with the overlay hash pointed to by ovlhref.
#
# Returns (0, msg) where msg is:
# XboneTunnel LocalAddr=<addr> RemoteAddr=<addr> Status= {up, down}
#
# There will be one msg line for each tunnel in the overlay.
###################################################################

sub XB_ncmd_ping ($$$$)
{
    my ($nodehref, $mhref, $aclhref, $ovlhref) = @_;
    my ($msg, $ttaref, @tdline, $tdline, $p);

    XB_Log::log ("info", "->XB_ncmd_ping (@_)");

    ###############################
    # A ping resets overlay timeout
    ###############################

    XB_Log::log ("debug0",
	      "Alive_Time reset for overlay: $ovlhref->{'OverlayName'}");

    $ovlhref->{'Alive_Time'} = time();

    #############################
    # For now only a tunnel's
    # Xbone addresses are ping'ed
    #############################

    $ttaref = $ovlhref->{'Tunnel_Tags'};

    $msg = "";
    foreach $tdline (@$ttaref)
    {
	@tdline = split /\s+/, $tdline;

	$msg .= "XboneTunnel LocalAddr=$tdline[1]";
	$msg .= " RemoteAddr=$tdline[2]";
	$msg .= " Status=";

	$p = Net::Ping->new("icmp");
	if ( $p->ping($tdline[2]) )
	{ $msg .= "up"; }
	else
	{ $msg .= "down"; }
	
	$msg .= "\n";
    };

    XB_Log::log ("info", "<-XB_ncmd_ping (0 $msg)");
    return (0, $msg);
}





###################################################################
#
# XB_ncmd_heartbeat ( nodehref, mhref, aclhref, ovlhref )
#
###################################################################

sub XB_ncmd_heartbeat ($$$$)
{
    my ($nodehref, $mhref, $aclhref, $ovlhref) = @_;

    XB_Log::log ("info",
	      "->XB_ncmd_heartbeat (@_)");

    #####################################
    # A heartbeat resets overlay timeout.
    # This requires saving state so that
    # it restores properly after a crash.
    #####################################

    XB_Log::log ("debug0",
	      "Alive_Time reset for overlay: $ovlhref->{'OverlayName'}");

    $ovlhref->{'Alive_Time'} = time();

    XB_write_state ($nodehref);

    XB_Log::log ("info", "<-XB_ncmd_heartbeat (0 \"\")");
    return (0, "");
}





###################################################################
#
# XB_ncmd_application ( nodehref, mhref, aclhref, ovlhref, line )
#
###################################################################

sub XB_ncmd_application ($$$$$)
{
    my ($nodehref, $mhref, $aclhref, $ovlhref, $line) = @_;

    my (@args, $retval, $ovlname);
    my @retval = (0, "");


    XB_Log::log ("info",
	      "->XB_ncmd_application (@_)");

    $ovlname = $mhref->{'XboneOverlay'}{'Name'};

    if ($line =~ s/^\s*install\s*//)
    {
	###############################
	# Command:  application install
	###############################

	@args = split /\s+/, $line;
		
	XB_Log::log ("debug0", "command=application install");

	#################################################
	# Command has the format:
	# XboneNodeCommand application install $app_name
	#     $app_version $app_url $app_package_os
	#
	# For now check to make sure that $app_package_os
	# matches this node's OS.  If it does not, do not
	# execute the command.
	#################################################

	if ($args[3] !~ /$nodehref->{'OS'}/i)
	{
	    XB_Log::log ("debug0",
		      "*** No installapp() -- OS: $nodehref->{'OS'} but app OS: $args[3]");

	    next LINE;
	};

	$retval = XB_App_Start::installapp (@args);

	XB_Log::log ("debug0", "installapp() ---> $retval");

	if ($retval !~ /^ok$/i)
	{
	      XB_Log::log ("debug0", "**** FAILURE: application install: @args");

	      @retval = (1, "Application install $retval");
	      goto EXIT;
	};

    }
    elsif ($line =~ s/^\s*execute\s*//)
    {
	###############################
	# Command:  application execute
	###############################

	@args = split /\s+/, $line;

	XB_Log::log ("debug0", "command=application execute");

	@retval = XB_App_Start::executeapp (@args);

	XB_Log::log ("debug0", "executeapp() ---> @retval");

	$retval = shift @retval;
	if ($retval !~ /^ok$/i)	     # String error message
	{
	    XB_Log::log ("debug0", "**** FAILURE: application execute: $retval");

	    @retval = (1, "Application execute $retval");
	    goto EXIT;
	}
	elsif (scalar (@retval))
	{
	    ##############################
	    # Application is executing, so
	    # store PIDs as overlay state.
	    ##############################

	    XB_push_overlay_state ($nodehref, $ovlhref, 'App_PIDs', "@retval");
	}
	else
	{
	   XB_Log::log ("debug0", "**** FAILURE: executeapp() returned no PIDs.");

           @retval = (1, "Application execute executeapp() returned no PIDs");
           goto EXIT;
	};
    };

EXIT:
    XB_Log::log ("info", "<-XB_ncmd_application (@retval)");
    return @retval;

}



1;                            # Insure TRUE return if module is interpreted.
