# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# ----------------------------------------------------------------------------
#
#  $RCSfile: xb-config.pl,v $
#
# $Revision: 1.35 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:37 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Author: Greg Finn
#
############################################################
#
# Start PERL on any system where perl is in the path
# This is a modified version of the "start perl" script
# provided in the PERL man pages, which starts perl 
# on the remainder of the file regardless of whether it
# is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the
# perl path, rather than requiring it be hard-coded.
#
# setenv PERLDB_OPTS "N f=26"
#
# Must be set before perl starts!
#
# N 	: Nonstop (noninteractive)
# f=26	: frame=26 = 16 + 8 + 2,
#		2 = entry and exit,
#		!4 = don't print args to functions
#		8= enabled overloaded stringify and tied FETCH
#		16= print return values from subroutines
#
############################################################
# the following line finds perl and runs it, in sh, csh, or perl:
#
# note - '-T' (TAINT) switch is not included, because 'which' often
# returns a version of perl that isn't secure. don't worry about it.
#
eval '(exit $?0)' && eval 'PERLDB_OPTS="N f=26";export PERLDB_OPTS;PERL=`which perl5`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERLDB_OPTS "N f=26"; setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

############################################################
# PERL CODE STARTS HERE
############################################################

# XBONE code to set libraries
BEGIN {
  use strict;
  use sigtrap;
  use FindBin;
  delete $ENV{PATH};

  # taint-safe backticks equivalent
  my $pid = open KID, "-|";
  unless(defined $pid) { die "open: $!"; }
  unless($pid) { 
    foreach my $uname ("/usr/bin/uname", "/bin/uname") {
      if(-x $uname) { exec $uname, "-msr" or die "exec: $!"; }
    }
    die "cannot find uname"; 
  }
  my ($osname, $osvers, $arch) = split ' ', lc <KID>;
  close KID or die "close: $!";
  $osvers =~ s/^(\d+\.\d+).*/$1/;

  if($osname eq "sunos") {
    # bang Solaris uname returns into shape for perl usage
    ($osname, $arch) = ("solaris", "sun4");
    $osvers =~ s/5\.(\d+)/2.$1/;
  } elsif($osname eq "linux") {
    # same for Linux
    if($arch eq "i686") { $arch =~ s/i\d(\d+)/i3$1/; }
  } elsif($osname eq "freebsd") {
    if($osvers =~ /cairn/) { $osvers = "2.2"; }
  }

  my $ostag = join "-", ($arch, $osname, $osvers);
  print "Configuring perl for $ostag...\n" if 0;

  my $sdir = "/home/xbone/perl";
  my $ldir = $FindBin::RealBin;
  foreach my $p ($ldir, "$ldir/../lib", "$ldir/../../lib", 
		 "$ldir/../../../lib", 
		 "$sdir/$ostag", "$sdir/$ostag/$arch-$osname") {
    if(-d $p) { unshift @INC, $p; }
  }

  # untaint the resulting include path so "use" works
  foreach my $i (@INC) { if($i =~ /(.*)/) { $i = $1;} }
  
  printf STDERR "\n%s\n\n", join "\n", @INC if 0;
};

##############################################################################
# END OF XBONE PREFIX CODE - PUT YOUR PERL CODE BELOW                        #
##############################################################################

use strict;
use sigtrap;

use Data::Dumper;
use Sys::Hostname;
use Term::ReadLine;
use IO::Socket;
use Socket;
use FileHandle;
use Config;

use XB_Defs;



##########################################
# Global declaration initialization.
##########################################



sub init_nodedb ($) {
  my ($href) = @_;
  my $os = $Config{osname};
  
  %$href = ( ID => hostname(),
	     Class => 'host',
	     IPversion => 'IPv4',
	     OverlayCount => 0x0,
	     MaxOverlayCount => 0x0,
	     TunnelCount => 0x0,
	     MaxTunnelCount => 0x0,
	     IPaddr => '0.0.0.0',
	     Authentication => 'none',
	     Encryption => 'none',
	   );
  
  if($os eq "freebsd") {
    # is it CAIRN?
    if(!system("sysctl net.inet.ip.iti_count > /dev/null 2>&1")) { 
      $os = "cairn";
    }
    # is it KAME?
    if(!system("sysctl net.inet.ipsec.def_policy > /dev/null 2>&1")) {
      my $osreldate = `sysctl -n kern.osreldate`;
      # IPsec on FreeBSD 4.0 is broken, revert back to "freebsd".
      if($osreldate =~ /^40/){ 
        $os = "freebsd";
      }else{
        $os = "kame";
      } 
    }
  }
  
  if($os eq "linux") {
    if(!system("modprobe -l | grep -s ipsec_mod > /dev/null 2>&1")) {
      $os = "nist";
    }
  }
  $href->{'OS'} = $os;
}
		 



sub init_acl ($$)
{
    my ($href, $nodehref) = @_;

    %$href = (
		 UserAuth => '\S*',
		 ID => $nodehref->{'ID'},
		 AccessLevel => 'deploy',
		 OverlayCount => 0x0,
		 MaxOverlayCount => 0x0,
		 TunnelCount => 0x0,
		 MaxTunnelCount => 0x0
             );
}




sub get_integer ($$)
{
    my ($prompt, $lref) = @_;
    my ($str, $pat);

    while (1)
    {
	print "$prompt: (range $lref->[0] to $lref->[1]) ";

	$str = <STDIN>;
	chomp $str;

	if ($str !~ /^[0-9]+\Z/)
	{
	   print "\n\aNot an integer, retry: (range $lref->[0] to $lref->[1]) ";
	   next;
        };

	if ($str < $lref->[0])
	{
	   print "\n\aToo small, retry: (range $lref->[0] to $lref->[1]) ";
	   next;
        }
	elsif ($str > $lref->[1])
	{
	   print "\n\aToo large, retry: (range $lref->[0] to $lref->[1]) ";
	   next;
        };

	last;
    }

    return ($str);
}




sub get_dotted_decimal ($;$)
{
    my ($prompt, $default) = @_;
    my ($str);

    while (1)
    {
	print "$prompt: [dotted decimal x.x.x.x] ";

	if ($default)
	{ print "($default) "; }

	$str = <STDIN>;
	chomp $str;

	if (!$str)
	{
	    $str = $default;
	    last;
	}
	elsif ($str !~ /^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\Z/)
	{
	   print "\n\aIncorrect dotted decimal address, retry: ";
	   next;
        };

	last;
    }

    return ($str);
}




sub get_string ($;$)
{
    my ($prompt, $default) = @_;
    my ($str, $pat);

GETSTR:
    while (1)
    {
	if (defined ($default))
	{ print "$prompt: ($default) "; }
	else
	{ print "$prompt: "; }

	$str = <STDIN>;
	chomp $str;
	if (!$str) { $str = $default; };

	last;
    }

    return ($str);
}




sub get_oneof ($$)
{
    my ($prompt, $lref) = @_;
    my ($str, $pat);

GETSTR:
    while (1)
    {
	print "$prompt: ($lref->[0]) ";

	$str = <STDIN>;
	chomp $str;
	if (!$str) { $str = $lref->[0];  last GETSTR; };

        foreach $pat (@$lref)
	{ if ($str =~ /$pat/i) { last GETSTR; }; }

	print "\n\aNot acceptable, retry: ($lref->[0]) ";
    }

    return ($str);
}




sub get_someof ($$)
{
    my ($prompt, $lref) = @_;
    my (@str, $str, $match, $pat);

    $match = 0;

SOMEOF:
    while (1)
    {
	print "$prompt: ($lref->[0]) ";

	$_ = <STDIN>;
	chomp $_;
	if (!$_) { @str = ($lref->[0]);  last SOMEOF; };

	@str = split /[, \t\n\r\f]+/;

	foreach $str (@str)
	{
	    $match = 0;

	    foreach $pat (@$lref)
	    { if ($str =~ /$pat/i) { $match = 1;  last; }; }

	    if (!$match)
	    {
		print "Choice of $str is unacceptable, retry: ($lref->[0]) ";
		next SOMEOF;
	    }
	}

	last SOMEOF;
    }

    return "@str";
}




sub fillin_nodedb ($)
{
    my ($href) = @_;
    my @candidates;

    init_nodedb ($href);		# Fill in some global defaults.

    @candidates = ("host", "dns", "router");
    $href->{'Class'} =
	get_oneof ("\nClass of node. One of {host, router, dns}:".
	"\n    dns:    the host is the DNS server for X-Bone overlays".
	"\n    router: the host acts as a router in the overlay".
	"\n    host:   the host acts as a normal host (not router)\n",
		   \@candidates);

    @candidates = ("IPv4");
    $href->{'IPversion'} =
	get_oneof ("IP protocol version.\nOne of {IPv4}", \@candidates);

    $href->{'OverlayManagers'} =
	get_string ("Domain names where Overlay Managers reside.\nFor example: www.foo.net", "www.xbone.net");
    $href->{'OverlayManagers'} =~ s/[,]?\s/, /g;
    $href->{'OverlayManagers'} = "{ $href->{'OverlayManagers'} }";

    @candidates = ("none", "md5", "sha1");
    $href->{'Authentication'} =
	get_someof ("Link authentication algorithms supported.\nOne or more of {none, md5, sha1}", \@candidates);
    $href->{'Authentication'} =~ s/\s/, /g;
    $href->{'Authentication'} = "{ $href->{'Authentication'} }";

    @candidates = ("none", "des", "3des");
    $href->{'Encryption'} =
	get_someof ("Link encryption algorithms supported.\nOne or more of {none, des, 3des}", \@candidates);
    $href->{'Encryption'} =~ s/\s/, /g;
    $href->{'Encryption'} = "{ $href->{'Encryption'} }";

    @candidates = (1, 1000);
    $href->{'MaxOverlayCount'} = 
	get_integer ("Maximum number of overlays supported", \@candidates);

    @candidates = (1, 1000);
    $href->{'MaxTunnelCount'} = 
	get_integer ("Maximum number of tunnels supported", \@candidates);

    {
	my ($b0, $b1, $b2, $b3);

	($b0, $b1, $b2, $b3) = unpack ('C4', gethostbyname (hostname()));

	$href->{'IPaddr'} = get_dotted_decimal ("Node IPv4 address",
						"$b0.$b1.$b2.$b3");
    }

    $href->{'ID'} =
	get_string ("Certificate name for this node - usually hostname",
		    $href->{'ID'});

    $href->{'SSLcert'} =
	get_string ("Complete file name for SSL certificate for this node\nrelative to $XB_Defs::XBONE_HOME/rd/",
		    "cert/$href->{'ID'}.cert.pem");

    $href->{'SSLkey'} =
	get_string ("Complete file name for SSL key for this node\nrelative to $XB_Defs::XBONE_HOME/rd/",
		    "cert/$href->{'ID'}.key.pem");
}




sub fillin_acl ($)
{
    my ($nodehref) = @_;
    my %acl = ();
    my @candidates;

    init_acl (\%acl, $nodehref);		# Fill in some global defaults.


    $acl{'UserAuth'} =
	get_string ("Define a user set.  This is a Perl pattern string that\nmatches user email addresses", $acl{'UserAuth'});

    @candidates = ("$acl{'AccessLevel'}", "manage");
    $acl{'AccessLevel'} =
	get_oneof ("Define the access control for this user set.\nOne of {deploy, manage}",
		   \@candidates);

    @candidates = (1, $nodehref->{'MaxOverlayCount'});
    $acl{'MaxOverlayCount'} = 
	get_integer ("Maximum number of overlays supported for this user set",
		     \@candidates);

    @candidates = (1, $nodehref->{'MaxTunnelCount'});
    $acl{'MaxTunnelCount'} = 
	get_integer ("Maximum number of tunnels supported", \@candidates);

    return \%acl;
}




sub write_node ($$)
{
    my ($fh, $nodehref) = @_;
    my (@keys, $key);

    print $fh "XboneNode   \\\n";

    @keys = keys (%$nodehref);

    while (1)
    {
	$key = shift (@keys);
	if (scalar (@keys))
	{
	    print $fh "\t$key=$nodehref->{$key}\t\\\n";
	}
	else
	{
	    print $fh "\t$key=$nodehref->{$key}\n";
	    last;
	}
    }
}





sub write_acl ($$)
{
    my ($fh, $aclhref) = @_;
    my (@keys, $key);

    print $fh "XboneAclList    \\\n";

    @keys = keys (%$aclhref);

    while (1)
    {
	$key = shift (@keys);
	if (scalar (@keys))
	{
	    print $fh "\t$key=$aclhref->{$key}\t\\\n";
	}
	else
	{
	    print $fh "\t$key=$aclhref->{$key}\n";
	    last;
	}
    }
}



#####################################################################
#####################################################################
##
## MAIN ()
##
#####################################################################
#####################################################################

STDERR->autoflush(1);
STDOUT->autoflush(1);

my %nodedb = ();

my (@candidates, $result, $nodehref, $aclhref);
my ($fh, $configuration, @aclhrefs);


$result = $XB_Defs::XBONE_PROTOCOL;   # Get rid of irritating PERL warnings
                                      # regarding only appears once in program.
$nodehref = \%nodedb;
fillin_nodedb ($nodehref);

print "\nAt least one Access Control List (ACL) must be created to define\n";
print "a set of users who are allowed to create/modify an overlay.\n\n";

@aclhrefs = ();
while (1)
{
    $aclhref = fillin_acl ($nodehref);
    push (@aclhrefs, $aclhref);

    print "\nThat completes the definition of one ACL set.\n";

    @candidates = ('No', 'Yes');
    $result = get_oneof ("Do you wish to define another ACL set?", \@candidates);
    if ($result =~ /no/i) { last; };
}


###############################
# Print out the data structures
# for a visual check.
###############################

$Data::Dumper::Purity = 1;

$configuration = Data::Dumper->Dump ([$nodehref], [qw(*XboneNode)]);

print "\nThe node configuration file is:\n\n$configuration";

foreach $aclhref (@aclhrefs)
{
    $configuration = Data::Dumper->Dump ([$aclhref], [qw(*XboneAclList)]);    
    print "\n$configuration";
}

###############################
# Determine where to write it.
###############################

while (1)
{
    $result = get_string ("\n\nFile for this configuration: ",
			  "$XB_Defs::DAEMON_CONF_FILE");

    $fh = new FileHandle "> $result";
    if (!defined ($fh))
    {
	print "\n\aCould not open: $result.";
	next;
    }

    last;
}

#####################
# Write the XboneNode
# portion.
#####################

write_node ($fh, $nodehref);
print $fh "\n";

#########################
# Sequentially write out
# each ACL defined.
#########################

while ($aclhref = shift (@aclhrefs))
{
    write_acl ($fh, $aclhref);
    print $fh "\n";
}

$fh->close;

print "\nFinished writing: $result\n\n";

exit (0);
