# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
# 
#  $RCSfile: xb-overlay-list.pl,v $
#
# $Revision: 1.19 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:38 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Author: Greg Finn
#
############################################################
#
# Start PERL on any system where perl is in the path
# This is a modified version of the "start perl" script
# provided in the PERL man pages, which starts perl 
# on the remainder of the file regardless of whether it
# is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the
# perl path, rather than requiring it be hard-coded.
#
# setenv PERLDB_OPTS "N f=26"
#
# Must be set before perl starts!
#
# N 	: Nonstop (noninteractive)
# f=26	: frame=26 = 16 + 8 + 2,
#		2 = entry and exit,
#		!4 = don't print args to functions
#		8= enabled overloaded stringify and tied FETCH
#		16= print return values from subroutines
#
############################################################
# the following line finds perl and runs it, in sh, csh, or perl:
#
# note - '-T' (TAINT) switch is not included, because 'which' often
# returns a version of perl that isn't secure. don't worry about it.
#
eval '(exit $?0)' && eval 'PERLDB_OPTS="N f=26";export PERLDB_OPTS;PERL=`which perl5`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERLDB_OPTS "N f=26"; setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

############################################################
# PERL CODE STARTS HERE
############################################################


# XBONE code to set libraries
BEGIN {
  use strict;
  use sigtrap;
  use FindBin;
  delete $ENV{PATH};

  # taint-safe backticks equivalent
  my $pid = open KID, "-|";
  unless(defined $pid) { die "open: $!"; }
  unless($pid) { 
    foreach my $uname ("/usr/bin/uname", "/bin/uname") {
      if(-x $uname) { exec $uname, "-msr" or die "exec: $!"; }
    }
    die "cannot find uname"; 
  }
  my ($osname, $osvers, $arch) = split ' ', lc <KID>;
  close KID or die "close: $!";
  $osvers =~ s/^(\d+\.\d+).*/$1/;

  if($osname eq "sunos") {
    # bang Solaris uname returns into shape for perl usage
    ($osname, $arch) = ("solaris", "sun4");
    $osvers =~ s/5\.(\d+)/2.$1/;
  } elsif($osname eq "linux") {
    # same for Linux
    if($arch eq "i686") { $arch =~ s/i\d(\d+)/i3$1/; }
  } elsif($osname eq "freebsd") {
    if($osvers =~ /cairn/) { $osvers = "2.2"; }
  }

  my $ostag = join "-", ($arch, $osname, $osvers);
  print "Configuring perl for $ostag...\n" if 0;

  my $sdir = "/home/xbone/perl";
  my $ldir = $FindBin::RealBin;
  foreach my $p ($ldir, "$ldir/../lib", "$ldir/../../lib", 
		 "$ldir/../../../lib", 
		 "$ldir/../rd", "$ldir/../../rd", 
		 "$ldir/../../../rd", 
		 "$sdir/$ostag", "$sdir/$ostag/$arch-$osname") {
    if(-d $p) { unshift @INC, $p; }
  }

  # untaint the resulting include path so "use" works
  foreach my $i (@INC) { if($i =~ /(.*)/) { $i = $1;} }
  
  printf STDERR "\n%s\n\n", join "\n", @INC if 0;
};

##############################################################################
# END OF XBONE PREFIX CODE - PUT YOUR PERL CODE BELOW                        #
##############################################################################

use strict;
use sigtrap;

use XB_Overlay_List;
use XB_Node_DB;


my ($callok, $lref, @rest);

# first shut off some output
open(OLDOUT,">&STDOUT");
open(STDOUT,">&/dev/null");
open(STDERR,">&/dev/null");

# grab enough state to know who we're talking to, notably
# the place where the rd is. this moved from 127.0.0.1,
# because there may be more than one rd...
%XB_Defs::DAEMON_STATE = ();
XB_read_config_file ($XB_Defs::DAEMON_CONF_FILE, \%XB_Defs::DAEMON_STATE);

print "----- done opening config file; now get overlay list\n";

# returns a list of pairs - tell it who to talk to 

($callok, @rest) = XB_overlaylst($XB_Defs::DAEMON_STATE{IPaddr});
  
############################
# Send proper data to STDOUT
############################

open(STDOUT,">&OLDOUT");

if (!$callok)
{ print STDOUT "Error\n"; }
else
{ print STDOUT "Overlays are:\n"; };

foreach $lref (@rest) {
    print "\t@$lref\n";
}


exit (0);
