# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
#
#  $RCSfile: xb-daemon.pl,v $
#
# $Revision: 1.50 $
#   $Author: touch $
#     $Date: 2000/10/25 22:52:50 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
#############################################
#
# Xbone host resource daemon main program
#
# Principal author: Gregory Finn
#
#############################################
#
# Start PERL on any system where perl is in the path
# This is a modified version of the "start perl" script
# provided in the PERL man pages, which starts perl 
# on the remainder of the file regardless of whether it
# is run under sh, csh, or perl.
#
# This version is modified to dynamically locate the
# perl path, rather than requiring it be hard-coded.
#
# setenv PERLDB_OPTS "N f=26"
#
# Must be set before perl starts!
#
# N 	: Nonstop (noninteractive)
# f=26	: frame=26 = 16 + 8 + 2,
#		2 = entry and exit,
#		!4 = don't print args to functions
#		8= enabled overloaded stringify and tied FETCH
#		16= print return values from subroutines
#
############################################################
# the following line finds perl and runs it, in sh, csh, or perl:
#
# note - '-T' (TAINT) switch is not included, because 'which' often
# returns a version of perl that isn't secure. don't worry about it.
#
eval '(exit $?0)' && eval 'PERLDB_OPTS="N f=26";export PERLDB_OPTS;PERL=`which perl5`; exec $PERL -wS $0 ${1+"$@"}'
    & eval 'setenv PERLDB_OPTS "N f=26"; setenv PERL `which perl5`; exec $PERL -wS $0 $argv:q'
    if 0;

############################################################
# PERL CODE STARTS HERE
############################################################

# XBONE code to set libraries
BEGIN {
  use strict;
  use sigtrap;
  use FindBin;
  delete $ENV{PATH};

  # taint-safe backticks equivalent
  my $pid = open KID, "-|";
  unless(defined $pid) { die "open: $!"; }
  unless($pid) { 
    foreach my $uname ("/usr/bin/uname", "/bin/uname") {
      if(-x $uname) { exec $uname, "-msr" or die "exec: $!"; }
    }
    die "cannot find uname"; 
  }
  my ($osname, $osvers, $arch) = split ' ', lc <KID>;
  close KID or die "close: $!";
  $osvers =~ s/^(\d+\.\d+).*/$1/;

  if($osname eq "sunos") {
    # bang Solaris uname returns into shape for perl usage
    ($osname, $arch) = ("solaris", "sun4");
    $osvers =~ s/5\.(\d+)/2.$1/;
  } elsif($osname eq "linux") {
    # same for Linux
    if($arch eq "i686") { $arch =~ s/i\d(\d+)/i3$1/; }
  } elsif($osname eq "freebsd") {
    if($osvers =~ /cairn/) { $osvers = "2.2"; }
  }

  my $ostag = join "-", ($arch, $osname, $osvers);
  print "Configuring perl for $ostag...\n" if 0;

  my $sdir = "/home/xbone/perl";
  my $ldir = $FindBin::RealBin;
  foreach my $p ($ldir, "$ldir/../lib", "$ldir/../../lib", 
		 "$ldir/../../../lib", 
		 "$sdir/$ostag", "$sdir/$ostag/$arch-$osname") {
    if(-d $p) { unshift @INC, $p; }
  }

  # untaint the resulting include path so "use" works
  foreach my $i (@INC) { if($i =~ /(.*)/) { $i = $1;} }
  
  printf STDERR "\n%s\n\n", join "\n", @INC if 0;
};

##############################################################################
# END OF XBONE PREFIX CODE - PUT YOUR PERL CODE BELOW                        #
##############################################################################

#############################################
#
# Xbone resource daemon main program
#
# Principal authors: Gregory Finn, Steve Hotz
#
#############################################

use strict;
use sigtrap;

use Data::Dumper;
use IO::Socket;
use Getopt::Long;
use Socket;
use FileHandle;
use Sys::Hostname;

use XB_Defs;
use XB_Utils;
use XB_Commands;
use XB_Node_DB;
use XB_DNS;
use XB_Log;
use XB_Route;
use XB_Tunnel;
use XB_SSL;			# SSL transport-layer encryption
use XB_IPsec;			# IPsec link encryption


my $XB_debug_flag = 1;		# Set TRUE to enable debug printing.


##########################################
# Lexically scoped declarations.
##########################################

my $XB_MULTICAST_FLAG = 1;	# If FALSE, only TCP command processing
				# is enabled.


my $mcsocket;
my ($inputflags, $errorflags, $iflg, $eflg) = ("", "", "", "");




#########################################################################
# XB_multicast_request ( \fh, dest_port, \msg )
#
# The resource records are scanned for those that the daemon can satisfy.
# The overlay: line plus resource lines that can be satisfied are
# sent across the UDP socket passed in \fh to the destination address
# specified by the address string in dest_addr.
#########################################################################

sub XB_multicast_request ($$$)
{
    my ($udph, $dest_port, $msgref) = @_;
    my ($ifacecnt, $record, $cmd, $compat_versions);
    my ($replymsg, $nodehref, %msghash, @result) =
	( "", \%XB_Defs::DAEMON_STATE, (), () );

    my ($tport, $taddr);
    ($tport, $taddr) = sockaddr_in($dest_port);
    $taddr = inet_ntoa($taddr);
    XB_Log::log ("info", "->XB_multicast_request ($udph, $taddr, $msgref)");

    XB_Log::log ("debug0", "######################\n");
    XB_Log::log ("debug0", "Incoming multicast UDP message:\n$$msgref");
    XB_Log::log ("debug0", "######################\n");

    %msghash = XB_hash_msg ($msgref);

    ##################################################
    # Implement the XboneControl Command=DestroyAll
    # as a multicast sendable command.
    #
    # Note: If this command is seen it terminates all
    # further processing of this message.
    #
    # Note: No release version checking is done for
    # this message.
    ##################################################

    if ($msghash{'XboneControl'}{'Command'} eq 'DestroyAll')
    {
        XB_Log::log ("debug0", "Multicast XboneControl Command=DestroyAll.");

	XB_delete_overlays (\%XB_Defs::DAEMON_STATE, undef);

	XB_write_state (\%XB_Defs::DAEMON_STATE);

        XB_Log::log ("debug0",
		     "Multicast XboneControl Command=DestroyAll finished.");
	goto EXIT;
    }

    ##################################################
    # Scan Xbone line for syntax and version matching.
    ##################################################

    @_ = XB_scan_xbone_line (\%msghash);
    if (!$_[0])
    {  goto EXIT; }

    #################################
    # Determine version compatibility
    #################################

    $compat_versions = XB_check_release (\%msghash);
    if (!$compat_versions)
    {
        XB_Log::log ("warning", "Received incompatible release number from OM.");
	goto EXIT;
    }


    ##############################################################
    # Here we implement normal multicast commands sent from an OM.
    ##############################################################

    ######################################
    # Handle ResourceDiscovery message.
    # Send no reply if something is wrong.
    ######################################

    if ($msghash{'XboneControl'}{'Command'} eq 'ResourceDiscovery')
    {
	@result = XB_ResourceDiscovery ($nodehref, \%msghash);
	if ($result[0])
	{
	    XB_Log::log ("debug0", "Access check failure: $result[1].");
	    goto EXIT;;
	};

	@result = XB_check_access ($nodehref, \%msghash);
	if (!$result[0])
	{
	    XB_Log::log ("debug0", "Access check failure: $result[1].");
	    goto EXIT;
	}

	goto REPLY;
    };

    #######################################
    # At this point a Command was seen that
    # wasn't recognized.  Issue a warning.
    #######################################

    XB_Log::log ("warning", "Multicast message contained unrecognized command.");
    goto EXIT;

    ####################################
    # Construct reply message to ovlmgr.
    # Insert Seq=xxx into XboneNode response
    ####################################

REPLY:

    $replymsg = "Xbone Version=$XB_Defs::XBONE_PROTOCOL ";
    $replymsg .= "Release=$compat_versions ";
    $replymsg .= "Seq=$msghash{'Xbone'}{'Seq'}";
    $replymsg .= "\n"; 
    $replymsg .= "XboneNode Class=$nodehref->{'Class'} ";
    $replymsg .= "OS=$nodehref->{'OS'} ";
    $ifacecnt = $nodehref->{'MaxTunnelCount'} - $nodehref->{'TunnelCount'};
    $replymsg .= "InterfaceCount=$ifacecnt ID=$nodehref->{'ID'} "; 
    $replymsg .= "IPaddr=$nodehref->{'IPaddr'}"; 
    $replymsg .= "\n"; 

    $replymsg .= "XboneEOC";    # Trailing \n is bugfix for ovlmgr
    $replymsg .= "\n"; 

    ####################
    # Send reply message
    ####################

    XB_Log::log ("debug0", "Sending:\n######\n$replymsg######");

    unless (send ($udph, $replymsg, 0, $dest_port))
    {
	XB_Log::log ("err", "cannot send UDP to manager: $!.");
	die "$!";
    };

    XB_Log::log ("debug0", "Multicast request succeeded.");

EXIT:
    XB_Log::log ("info", "<-XB_multicast_request ()");
    return;
}





##############################################
# XB_multicast_input (mch)
#
# This routine is called to process an incoming multicast datagram that
# has arrived on the socket passed by mch. The datagram is processed,
# which may result in a reply back to the actuator.
##############################################

# holds copies of all udp messages received in the last 2*MSL, to
# prevent against duplicate messages (XXX this should probably go
# somewhere persistent, in case the RD crashes/restarts)
my %udp_msg_cache;

sub XB_multicast_input ($) {
  my $mch = shift;

  # print trace message
  XB_Log::log ("info", "->XB_multicast_input $mch");

  # purge messages older than our duplicate detection interval from the cache
  my $current_time = time;
  my %updated_udp_msg_cache;
  while(my ($msg, $time) = each %udp_msg_cache) {
    # we MUST NOT delete from the hash during iterating, see "perldoc -f each"
    if($time + $XB_Defs::DUPLICATE_DETECTION_INTERVAL >= $current_time) {
      # this message is younger than the detection interval, keep it
      $updated_udp_msg_cache{$msg} = $time;
      XB_Log::log "debug", "kept message received %d seconds ago in ".
	"duplicate detection cache", $current_time - $time;
    } else {
      # this message has expired, do not keep it in the cache
      XB_Log::log "notice", "message timed out from the duplicate ".
	"detection cache, was received %d seconds ago", 
	$current_time - $time;
    }
  }
  # move updated cache into place
  %udp_msg_cache = %updated_udp_msg_cache;

  # grab the multicast message
  my ($msg, $srcport);
  unless($srcport = recv $mch, $msg, $XB_Defs::ACTUATOR_MSG_LIMIT, 0) {
    XB_Log::log "err", "cannot receive on multicast socket: $!"
      and die "XB_multicast_input";
  }

  # update current time in case recv took a while
  $current_time = time;

  # check if the message we just received is a duplicate
  if(exists $udp_msg_cache{$msg}) {
    # yes, we have received this message in the recent past
    XB_Log::log "warning", "received a duplicate of a message already ".
      "received %d seconds ago, ignoring it", 
      $current_time - $udp_msg_cache{$msg};
  } else {
    # this is a fresh message, cache and handle it
    XB_Log::log "debug", "fresh message in duplicate detection cache";
    $udp_msg_cache{$msg} = $current_time;
    my $udpsocket = new FileHandle;
    XB_udp_socket $udpsocket, 0, 0;
    # Prevent local write buffering.
    $udpsocket->autoflush(1); 
    if($msg) { XB_multicast_request $udpsocket, $srcport, \$msg; };
  }
  
  # print trace message
  XB_Log::log "info", "<-XB_multicast_input $mch";
}





################################################################
# XB_udp_input ( udph )
#
# This routine is called to process an incoming UDP datagram
# that has arrived on the socket passed by udph.  The datagram
# is processed.
################################################################

sub XB_udp_input ($)
{
    my ($udph) = @_;
    my ($msg) = "";   my %msghash = ();
    my ($error, $errcmd, $errmsg, $ovlname, $ovlmgrname);
    my ($srcport, $aclhref, $ovlhref, @result);

    XB_Log::log ("info", "->XB_udp_input (@_)");

    unless ($srcport = recv ($udph, $msg,
			     $XB_Defs::ACTUATOR_MSG_LIMIT, 0))
    {
      XB_Log::log ("err", "Reception error on UDP socket: $!.");
	die "$!";
    }


    XB_Log::log ("debug0", "######################\n");
    XB_Log::log ("debug0", "Incoming UDP message:\n$msg");
    XB_Log::log ("debug0", "######################\n");


    %msghash = XB_hash_msg (\$msg);


    ########################################################
    # Discard any Xbone multicast messages.  The way sockets
    # work they appear on this socket as well as the Mcast
    # specific socket.
    ########################################################

    {
	if (defined ($msghash{'XboneControl'}{'Command'}))
	{
	    if ( ($msghash{'XboneControl'}{'Command'} =~ /\bDestroyAll\b/i) ||
		 ($msghash{'XboneControl'}{'Command'} =~ /\bResourceDiscovery\b/i) )
	    {
	      XB_Log::log ("info",
			   "Skipping duplicate UDP reception of Mcast message.");
		
		goto EXIT;
	    }
	}
    }


    ###########################
    # Discard any messages that
    # do not meet basic syntax
    ###########################

    ($error, $errcmd, $errmsg) = XB_scan_msg_syntax (\%msghash);
    if ($error)  {  goto EXIT; }

    ######################
    # Check to see that
    # named overlay exists
    ######################

    if (!defined ($msghash{'XboneOverlay'}{'Name'}))
    {  goto EXIT;  }
    $ovlname = $msghash{'XboneOverlay'}{'Name'};

    if (!defined ($msghash{'XboneOverlay'}{'OverlayManager'}))
    {  goto EXIT;  }
    $ovlmgrname = $msghash{'XboneOverlay'}{'OverlayManager'};

    ($aclhref, $ovlhref) = (XB_find_overlay (\%XB_Defs::DAEMON_STATE,
					     $ovlname, $ovlmgrname));
    if (!$aclhref)
    {  goto EXIT;  }


    #####################################
    # NOTE: Only enable commands here   #
    #       that can safely be executed #
    #       WITHOUT security of SSL.    #
    #####################################

    ################################
    # Currently, we only implement
    # the XboneNodeCommand heartbeat
    ################################

    @result = XB_dispatch_on_nodecommand (\%XB_Defs::DAEMON_STATE, \%msghash,
					  $aclhref, $ovlhref,
					  [ 'heartbeat' ] );

EXIT:
    XB_Log::log ("info", "<-XB_udp_input ()");
    return;
}





########################################################################
# XB_get_all_commands ( tcph )
#
# This routine is called to accept any incoming TCP connections for
# the XBONE MCAST_PORT.  These connections are each passed to a routine
# that serially gathers the commands and acts on them.
########################################################################

sub XB_get_all_commands ($)
{
    my ($tcph) = @_;
    my ($sslh, $canonical_peer, $issuer, $msg, $cmdblock, $peer_authentication);
    my $subrname = "XB_get_all_commands";

    XB_Log::log ("info", "->$subrname ($tcph)");

    ############################
    # Accept the new connection.
    ############################

    XB_Log::log ("debug0", "Accepting TCP connection.\n");

    my $newtcph = $tcph->accept;

    # cannot simply use $newtcph->peeraddr, this causes a die
    # somewhere in the perl system libraries (larse)

    my $peer_hostname = gethostbyaddr ($newtcph->peeraddr, PF_INET);

    XB_Log::log ("debug0", "Got connection from $peer_hostname.");

    ####################################
    # Try to authenticate the peer host.
    ####################################

    XB_Log::log ("debug0", "Authenticating TCP peer.");

    eval { ($sslh, $canonical_peer, $issuer) =
	     XB_SSL::accept ($newtcph->fileno) };

    if ($@)	# SSL authentication failed.
    {
	XB_Log::log ("debug0", "SSL authentication failed.");

        goto CLOSE_TCP;
    }
    elsif ( !($canonical_peer && $issuer) )
    {
	###################################
	# We have a non-authenticated peer.
	# Pass the command only if the peer
	# address is the LOOPBACK address.
	###################################

	$peer_authentication = 0;

	if ($peer_hostname !~ /(LOCALHOST|loopback)/i)
	{
	    warn "Unauthenticated SSL from $peer_hostname, ignoring request.";
	    
	    #############
	    # Send reply.
	    #############

	    XB_transaction_reply ( $sslh,
				  "#### MESSAGE NOT READ ####",
				  "#### MESSAGE NOT READ ####", 1,
				  "Unauthenticated SSL from $peer_hostname." );

	    goto CLOSE_SSL;
	}
    }
    else
    {
	###################################
	# We have an authenticated peer.
	# Check to make sure that we agree
	# that this peer is known to us.
	###################################

	my ($match, $testpeer);

	$peer_authentication = 1;

	$match = 0;
	$_ = $XB_Defs::DAEMON_STATE{'OverlayManagers'};
	foreach $testpeer (split())
	{
	    if ($canonical_peer eq $testpeer) { $match = 1;  last; };
	};

	if (!$match)
	{ 
	    XB_Log::log ("warning",
	      "Unrecognized overlay manager: $canonical_peer, ignoring request.");

	    #############
	    # Send reply.
	    #############

	    XB_transaction_reply ( $sslh,
				  "#### MESSAGE NOT READ ####",
				  "#### MESSAGE NOT READ ####", 1,
			  "Unrecognized overlay manager: $canonical_peer." );

	    goto CLOSE_SSL;
	  }

	if ($issuer !~ /The X-Bone Certification Authority/)
	{
	    XB_Log::log ("warning",
		      "Unrecognized certification authority, ignoring request.");

	    #############
	    # Send reply.
	    #############

	    XB_transaction_reply ( $sslh,
				  "#### MESSAGE NOT READ ####",
				  "#### MESSAGE NOT READ ####", 1,
				  "Unrecognized certification authority." );

	    goto CLOSE_SSL;
	  }
    }

    #############################################
    # At this point authentication succeeded.
    # Progress with reading in the command lines.
    #############################################

    XB_Log::log ("debug0", "Authenticated TCP peer as Overlay Manager.\n");

    $cmdblock = "";
    do
    {
	XB_Log::log ("debug0", "XB_SSL::read().");

	eval { $msg = XB_SSL::read ($sslh) };   # Nothing to do on failure.

	$cmdblock = $cmdblock . $msg;

	XB_Log::log ("debug0", "XB_SSL::read() $msg");

	if ($cmdblock =~ /\bXboneEOC\b/ )
	{
	    XB_process_commands ($sslh, $cmdblock,
				 $canonical_peer, $peer_authentication);

	    ######################################
	    # We have processed our command block.
	    # We are done with this connection.
	    ######################################

	    goto CLOSE_SSL;
	};
      }
    while (length ($msg));


    #############################################
    # If we got here, the connection was remotely
    # closed without our having seen a complete
    # command block.  Close down this connection.
    #############################################

    XB_Log::log ("warning", "In $subrname: No command received.");

CLOSE_SSL:
    eval { XB_SSL::close ($sslh) };   # Nothing to do on failure.

CLOSE_TCP:
    $newtcph->close or warn "In $subrname: $!: Close failed";

    XB_Log::log ("info", "<-$subrname ()");
    return;
}





#####################################################################
# XB_create_command_reply ( cmd, overlay_name, code_value, message )
#
# Creates the reply message to an overlay manager command that
# has been received and procesed.  The message text is:
#
#    XboneControl  Command=$cmd  Transaction=Reply
#    XboneOverlay  Name=$overlay_name
#    XboneReturn  Code=$code_value  Msg=$message
#    XboneEOC
#
# The reply message is returned as a text string.
######################################################################

sub XB_create_command_reply ($$$$)
{
   my ($cmd, $overlay_name, $code_value, $message) = @_;
   my $msg;

   XB_Log::log ("info",
	"->XB_create_command_reply ($cmd, $overlay_name, $code_value, $message)");

   $msg = "Xbone Version=$XB_Defs::XBONE_PROTOCOL ";
   $msg .= "Release=$XB_Defs::XBONE_RELEASE";
   $msg .= "\n";
   $msg .= "XboneControl Command=$cmd Transaction=Reply";
   $msg .= "\n";
   $msg .= "XboneOverlay Name=$overlay_name";
   $msg .= "\n";
   $msg .= "XboneReturn Code=$code_value Msg=\"$message\"";
   $msg .= "\n";
   $msg .= "XboneEOC";
   $msg .= "\n";

   XB_Log::log ("info", "<-XB_create_command_reply $msg");
   return ($msg);
}





#######################################################################
# XB_transaction_reply ( sslh cmd ovlname code msg )
#
# This routine builds a Transaction=Reply message that is sent back
# using SSL across the TCP handle sslh.  There are two calling
# conventions.
#
# Under the first convention command overlay name, reply code and
# message are passed and used to create a typical reply message.
#
# Under the second convention the ovlname argument is "undef".  In this
# the cmd argument contains the complete message that is sent as is.
########################################################################

sub XB_transaction_reply ($$;$$$)
{
    my ($sslh, $cmd, $ovlname, $code, $msg) = @_;

    if (!defined $ovlname) { $ovlname = ""; };
    if (!defined $code) { $code = ""; };
    if (!defined $msg) { $msg = ""; };

    XB_Log::log ("info",
	      "->XB_transaction_reply ($sslh, $cmd, $ovlname, $code, $msg)");

    XB_Log::log ("debug0", "Sending reply message\n######");

    if ($ovlname)
    {
	$cmd = XB_create_command_reply ( $cmd, $ovlname, $code, $msg );
    };

    XB_Log::log ("debug0", "$cmd######");

    eval { XB_SSL::write ($sslh, $cmd) };   # Nothing to do on failure.

    XB_Log::log ("info", "<-XB_transaction_reply ()");
    return;
}





######################################################################
# XB_process_commands ( sslh cmds canonical_peer peerauthp )
#
# This routine is called to process a list of commands lines that
# is passed as a scalar in cmds.  The command lines have been
# received over the SSL socket passed in sslh.
#
# If peerauthp is false, the peer was not been authenticated.
# In this case we only allow 127.0.0.1 loopback as the peer's
# address and only allow the "XboneNodeCommand overlay list"
# to be executed.
#
# A FALSE value is returned if we have an access control failure.
# Otherwise, TRUE is returned.
######################################################################

sub XB_process_commands ($$$$)
{
    my ($sslh, $cmds, $canonical_peer, $peerauthp) = @_;

    my ($result, $line) = (1, "");
    my @lines = split /\n/, $cmds;
    my (@args, @retval, $aclhref, %msghash, $ovlname, $cmd, $reply_msg);

    XB_Log::log ("info",
	      "->XB_process_commands ($sslh, $cmds, $canonical_peer, $peerauthp)");

    XB_Log::log ("debug0", "Printing command\n######");
    XB_Log::log ("debug0", "$cmds######");

    %msghash = XB_hash_msg (\$cmds);

    $ovlname = "***UNKNOWN***";
    if (defined ($msghash{'XboneOverlay'}{'Name'}))
    { $ovlname = $msghash{'XboneOverlay'}{'Name'}; };

    @retval = XB_scan_msg_syntax (\%msghash);
    if ($retval[0])
    {
	####################
	# Bad message syntax
	####################

	XB_transaction_reply ($sslh, $retval[1], $ovlname, 1, "$retval[2]" );

	$result = 0;
	goto EXIT;
    };

    if (!$peerauthp)
    {
        ################################
        # Process non-peer-authenticated
	# messages that we receive
	################################

	if ( $msghash{'XboneControl'}{'Command'} =~ /\bListOverlays\b/i )
	{
	    @retval = XB_ListOverlays (\%XB_Defs::DAEMON_STATE, \%msghash,
				       $canonical_peer);
	    $reply_msg = $retval[1];
	};

	#######################
	# Send reply and return
	#######################

	XB_transaction_reply ($sslh, $reply_msg);

	goto EXIT;
    };


    ##########################
    # At this point we have an
    # authenticated message
    ##########################

    $cmd = $msghash{'XboneControl'}{'Command'};

    if (!defined ( inet_aton ($canonical_peer) ) )
    {
	XB_transaction_reply ($sslh, $cmd, $ovlname, 1,
			      "Cannot resolve hostname: $canonical_peer." );

	$result = 0;
	goto EXIT;
    };

    #####################
    # Process the message
    #####################

    @retval = XB_dispatch_on_msg (\%XB_Defs::DAEMON_STATE, \%msghash,
				  $canonical_peer);

    ###########################
    # Send a reply if generated
    ###########################

    if (scalar (@retval) > 1)	   # Generic request/response
    {
	XB_transaction_reply ($sslh, $cmd, $ovlname, $retval[0], $retval[1]);
    }
    elsif (scalar (@retval) == 1)   # Specific request/response
    {
	XB_transaction_reply ($sslh, $retval[0]);
    };


EXIT:
    XB_Log::log ("info", "<-XB_process_commands ($result)");
    return ($result);
}






#####################################################################
#####################################################################
##
## MAIN ()
##
#####################################################################
#####################################################################

my %nodedb = ();



STDERR->autoflush(1);
STDOUT->autoflush(1);



####################################
# Initialize debug and error logging
# and print a banner line.
#
# See log::open() comments in /lib/XB_Log.pm
# Logging can go to four destinations:
#       XB_Defs::SYSLOG, XB_Defs::ERR,
#       XB_Defs::OUT and XB_Defs::FILE
####################################

if ($XB_Defs::LOG_MASK)
{
    print "\nOpening system log with mask: $XB_Defs::LOG_MASK\n\n";

    XB_Log::open ($XB_Defs::LOG_DEST, $XB_Defs::LOG_MASK, $XB_Defs::LOG_FILE );

    # For debugging, send everything to STDERR.
    # XB_Log::open (XB_Log::ERR, "err,warning,info,debug0,debug1");

}
else
{
    print "\nSystem logging defaulted to STDOUT.\n";
}


XB_banner_msg ();


##############################
# Check whether or not this is
# running as root (superuser).
# Die if it is not.
##############################

if ($<)
{
    XB_Log::log ("err", "****************************\n");
    XB_Log::log ("err",
	    "The Resource Daemon program must be run as root (superuser).\n");
    XB_Log::log ("err", "****************************\n\n");

    die;
};


####################################
# Check whether or not this version
# is obsolete and issue warning or
# die if it is very out of date.
####################################

$_ = XB_obsolete();

if (!$_)
{
    my $tmp;

    $tmp = "****************************\n";
    $tmp .= "Xbone system is out of date more than XBONE_OBSOLETE_LIMIT.\n";
    $tmp .= "****************************\n\n";
    XB_Log::log ("warning", $tmp);
}
elsif ($_ == -1)
{
    my $tmp;

    $tmp =  "****************************\n";
    $tmp .= "Xbone system out of date well beyond XBONE_OBSOLETE_LIMIT.\n";
    $tmp .= "This version of the Xbone system is fatally obsolete.\n";
    $tmp .= "****************************\n\n";
    XB_Log::log ("err", $tmp);

    die;
}


##########################################
# Process program switches:
#
#    -config <file>   ; Redefines config
#                     ; file used
##########################################

GetOptions ( "config=s" => \$XB_Defs::DAEMON_CONF_FILE );

#################################################
# Check for mrtd and ask for password if required
#################################################

XB_Log::log ("debug0", "Initializing mrtd.");

eval { XB_Route::mrtd_init(); };

if ($@) { XB_Log::log ("err", "XB_Route::mrtd_init failed with $@"); die ""; }

##########################################
# Initialize from this host's config file
##########################################

XB_Log::log ("warning", "Using config file: $XB_Defs::DAEMON_CONF_FILE\n");

unless (-r "$XB_Defs::DAEMON_CONF_FILE")
{
    XB_Log::log ("err", "Cannot find config file: $XB_Defs::DAEMON_CONF_FILE.");
    die "";
}


$XB_Defs::DAEMON_STATE_TIME = time ();

XB_read_state (\%XB_Defs::DAEMON_STATE, $XB_Defs::DAEMON_CONF_FILE);

XB_write_state (\%XB_Defs::DAEMON_STATE);

{
    my $state = Dumper (\%XB_Defs::DAEMON_STATE);
    XB_Log::log ("info", "########## STARTUP STATE ##########\n");
    XB_Log::log ("info", "\n$state\n");
    XB_Log::log ("info", "########## END STARTUP STATE ##########\n");
}

##################################
# Check OS type and redeclare
# OS-dependent variables
##################################

XB_Log::log ("debug0", "Node OS = $XB_Defs::DAEMON_STATE{'OS'}\n");

{
    my $match = 0;  my $os;

    foreach $os (@XB_Defs::ALLOWED_NODEOS)
    {
	if ($os =~ /^$XB_Defs::DAEMON_STATE{'OS'}/i)
	{ $match = 1;  last; }
    };

    if (!$match)
    {
      XB_Log::log ("err",
		"In configuration file XboneNode OS property is unrecognized.");
	die "";
    };
}

if ($XB_Defs::DAEMON_STATE{'OS'} =~ /linux|nist/i)
{
  $XB_Defs::IP_MULTICAST_IF = 32;
  $XB_Defs::IP_MULTICAST_TTL = 33;
  $XB_Defs::IP_MULTICAST_LOOP = 34;
  $XB_Defs::IP_ADD_MEMBERSHIP = 35;
  $XB_Defs::IP_DROP_MEMBERSHIP = 36;
  $XB_Defs::GATED_PID_PATH = "/var/run/";
};


##############################################
# Determine if this daemon is running
# the DNS for this Xbone domain
#
# NOTE: If a daemon has Class=dns, it will not
#       respond to XBONE multicast messages.
##############################################

if ($XB_Defs::DAEMON_STATE{'Class'} =~ /dns/i)
{
    my $match = 0;  my $os;

    $XB_MULTICAST_FLAG = 0;

    foreach $os (@XB_Defs::ALLOWED_DNS_NODEOS)
    {
	if ($os =~ /^$XB_Defs::DAEMON_STATE{'OS'}/i)
	{ $match = 1;  last; }
    };

    if (!$match)
    {
      XB_Log::log ("err", "****************************\n");
      XB_Log::log ("err",
             "A Resource Daemon whose Class=DNS is currently forbidden from\n");
      XB_Log::log ("err",
             "having its operating system be $XB_Defs::DAEMON_STATE{'OS'}.\n");
      XB_Log::log ("err", "****************************\n\n");

	die "";
    };
}


####################################################
# Open an XBONE multicast socket and join its group.
####################################################

XB_Log::log ("debug0", "MULTICAST? $XB_MULTICAST_FLAG");

if ($XB_MULTICAST_FLAG)
{
    $mcsocket = new FileHandle;

    XB_Log::log ("debug0", "Getting multicast socket.");

    XB_mcast_recv_socket ($mcsocket, $XB_Defs::DAEMON_STATE{IPaddr}, 
			  $XB_Defs::MCAST_ADDR, $XB_Defs::MCAST_PORT);

    vec ($inputflags, fileno ($mcsocket), 1) = 1;
};

#########################
# Initialize SSL security
#########################

XB_Log::log ("debug0", "Initializing SSL.");

eval { XB_SSL::init ( $XB_Defs::SECURITY_CA_CERT,
	      $XB_Defs::XBONE_HOME . "/rd/" . $XB_Defs::DAEMON_STATE{SSLcert},
	      $XB_Defs::XBONE_HOME . "/rd/" . $XB_Defs::DAEMON_STATE{SSLkey} ) };

if ($@)
{
    XB_Log::log ("err", "XB_SSL::init failed with $@");
    die "";
};

#############################################################
# Open unspecific TCP listening socket to XBONE assigned port
#############################################################

XB_Log::log ("debug0", "Opening TCP listen socket.");

my $tcpsocket = IO::Socket::INET->new (LocalPort => $XB_Defs::MCAST_PORT,
				       LocalAddr => $XB_Defs::DAEMON_STATE{IPaddr},
				       Listen => SOMAXCONN, Reuse => 1);
if (!$tcpsocket)
{
    XB_Log::log ("err", "Cannot open TCP listen socket.");
    die "";
};

$tcpsocket->autoflush(1);

vec ($inputflags, fileno ($tcpsocket), 1) = 1;

#########################################################
# Open unspecific UDP input socket to XBONE assigned port
#########################################################

XB_Log::log ("debug0", "Opening UDP input socket.");

my $udpsocket = IO::Socket::INET->new (LocalPort => $XB_Defs::MCAST_PORT,
				       LocalAddr => $XB_Defs::DAEMON_STATE{IPaddr},
				       Proto => "udp",
				       Reuse => 1);
if (!$udpsocket)
{
    XB_Log::log ("err", "Cannot open UDP input socket.");
    die "";
};

$udpsocket->autoflush(1);

vec ($inputflags, fileno ($udpsocket), 1) = 1;

$errorflags = $inputflags;

#############################################################
# Issue an unlimited select on input from an overlay manager.
#############################################################

XB_Log::log ("debug0", "Entering while ....");

while (1)
{
    my ($nfound, $timeleft, $timebefore, $timeout);

#  warn "BEFORE CRITICAL SELECT, DUMPING:";

    do
    {

#    warn "inputflags = " . unpack("b*", $inputflags) . ", errorflags = " .
#      unpack("b*", $errorflags);


	$timebefore = time();

	($nfound, $timeleft) = 
	    select ($iflg = $inputflags, undef, $eflg = $errorflags,
		    	$timeout = $XB_Defs::OVERLAY_SWEEP_INTERVAL );

#	warn "nfound = $nfound, timeleft = $timeleft";

    } while (($nfound == 0) &&
	     ((time() - $timebefore) < $XB_Defs::OVERLAY_SWEEP_INTERVAL) );

    if ($XB_MULTICAST_FLAG)
    {
	if (vec ($eflg, fileno ($mcsocket), 1))
	{ 
	    XB_Log::log ("err", "I/O error on multicast socket: $!."); 
	    die ""; 
	}

	if (vec ($iflg, fileno ($mcsocket), 1))
	{
	    XB_Log::log ("debug0", "Got MCAST input event.");
	    XB_multicast_input ($mcsocket);         # Process manager requests
	}
    }

    if (vec ($eflg, fileno ($tcpsocket), 1))
    { 
	XB_Log::log ("err", "I/O error on TCP socket: $!."); 
	die ""; 
    }

    if (vec ($iflg, fileno ($tcpsocket), 1))
    {
	XB_Log::log ("debug0", "Got TCP connect event.");
	XB_get_all_commands ($tcpsocket);           # Process manager commands
    }

    if (vec ($eflg, fileno ($udpsocket), 1))
    { 
	XB_Log::log ("err", "I/O error on UDP socket: $!."); 
	die ""; 
    }

    if (vec ($iflg, fileno ($udpsocket), 1))
    {
	XB_Log::log ("debug0", "UDP datagram arrival event.");

	XB_udp_input ($udpsocket);                  # Process UDP datagram
    }

    if ((time() - $timebefore) >= $XB_Defs::OVERLAY_SWEEP_INTERVAL)
    {
	XB_Log::log ("debug0", "Timeout occurred, sweeping overlays");

	XB_sweep_overlays (\%XB_Defs::DAEMON_STATE);
    }
}

exit (0);
