#! /usr/local/bin/perl5 -w
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
#  $RCSfile: index.pl,v $
#
# $Revision: 1.5 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:40 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Author: Lars Eggert

use strict;
use sigtrap;

my $arch;
BEGIN {
  use Config;
  my $osvers = $Config{osvers};
  $osvers =~ s/(\d+\.\d+).*/$1/;
  $arch = "$Config{archname}-$osvers";
};
use lib ("../lib", "/home/xbone/perl/$arch");

use CGI qw(:standard :html3);
use CGI::Carp qw(fatalsToBrowser);
use XB_Defs;

my $title = "Welcome to the X-Bone!";
print header, 
  start_html(-title => $title, -style => { -src =>"/xbone.css" }),
  h1($title),
  p("Follow the steps below to get started using the X-Bone. You need to do",
    "this on your first visit only."),
  dl(dt(h2(a({-href => "/cgi-bin/xb-get-ca-cert.pl"},
	  "1. Download CA Certificate (*)"))),
     dd("This will retrieve the CA's certificate and instruct your browser",
	"to install it. You should confirm the authenticity of this",
	"certificate by verifying its fingerprint. You may do so by",
	"contacting the CA by phone or authenticated e-mail.",
	i("In the future, the CA certificate will be distributed from ISI's",
	  "secure web server.")), p,

     dt(h2(a({-href => "/cgi-bin/xb-req-user-cert.pl"}, 
	  "2. Request X-Bone User Certificate (*)"))),
     dd("Before you may execute any X-Bone operations, you need to obtain a",
	"user certificate. Use this form to request one. The CA will contact",
	"you to confirm your identity before issuing a certificate."), p,

     dt(h2(a({-href => "https://" . virtual_host . 
	      "/cgi-bin/xb-req-host-cert.pl"}, 
	  "3. Request X-Bone Host Certificates (Optional) (*)"))),
     dd("If you are installing X-Bone on new machines, you will need to",
	"request an X-Bone host certifiacet for each of these machines.",
	"Use this form to request one. You must own a signed X-Bone user",
	"certificate to do this!"), p,

     dt(h2(a({-href => "https://" . virtual_host . "/"},
	  "4. Access the Overlay Manager"))),
     dd("Now you can access the X-Bone Overlay Manager over a secure,",
	"authenticated connection. (You should bookmark the next page for",
	"future visits.)")), p,

  dl(dt(h2("(*) ". span({-class => "redbg"}, "Information about the X-Bone".
           " X.509 Certificates:"))),
     dd("The X-Bone project signs X.509 certificates only for collaborators ",
        "of our project. If this is an independent installation at your local",
        "site, you will need to either setup your own certification authority",
        "(see instructions in the ". a({-href => "http://www.openssl.org"},
        "OpenSSL") . " package) or use a commercial service (e.g., Verisign)."
       )), p,

  h1("Information about the X-Bone Project"),
  dl(dt(h2(a({-href => "http://www.isi.edu/xbone/"},
          "X-Bone Project Page"))),
     dd("Project information, software download, FAQ, publications, etc.")), p,
  
  i("If you encounter any problems with these steps, ",
    a({-href => "mailto:$XB_Defs::CA_EMAIL"}, "please contact us.")),
  end_html;
