# -*- perl -*- 
#
# DO NOT MOVE THE FIRST LINE
# It identifies the rest of the file as PERL for EMACS autoformatting
# put perl options at the end of that line, e.g., -p
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# -------------------------------------------------------------------------
# 
#
# ----------------------------------------------------------------------------
#  $RCSfile: XB_Interface.pm,v $
#
# $Revision: 1.26 $
#   $Author: yushunwa $
#     $Date: 2000/10/27 23:09:23 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Author: Lars Eggert

package XB_Interface;

require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw();
@EXPORT_OK = qw(OverlayManagerInterface);

use strict;
use sigtrap;

my $arch;
BEGIN {
  use Config;
  my $osvers = $Config{osvers};
  $osvers =~ s/(\d+\.\d+).*/$1/;
  $arch = "$Config{archname}-$osvers";
};
use lib ("../lib", "/home/xbone/perl/$arch");

use Text::ParseWords;
use CGI qw(:standard :html3);
use CGI::Carp qw(fatalsToBrowser);
use XB_Common;

use IO::Socket;

use FileHandle;
use POSIX qw(:errno_h);

use XB_Defs;

my $simple_read = 0;
sub OverlayManagerInterface;

my $XB_proc_name = "XBONE Interface";
sub OverlayManagerInterface
{
  my ($str) = @_;
  my $subrname = "OverlayManagerInterface";
  ###########################################################
  ## Open a connection to the overlay manager, send it the ##
  ## prepared string, and get the response string back     ##
  ###########################################################
  my $buf = "";
  my $return_string = "";
  my $routeraddr;

  my $tcph;
  $tcph = IO::Socket::INET->new(   LocalAddr => $XB_Defs::LOOPBACK_IP_ADDRESS,
				   PeerAddr => $XB_Defs::XBONE_OVERLAY_MANAGER,
				   PeerPort => $XB_Defs::OVLMGR_PORT,
  				   Proto => 'tcp',			
				   Reuse => 1);

  unless ($tcph) { return "Xbone Version=$XB_Defs::XBONE_PROTOCOL\nXboneReturn Msg=\"Error: Cannot obtain socket: $!. I suspect the Overlay Manager is down.\"\nXboneEOC\n"; };
   
  $tcph->autoflush(1);

  print $tcph $str;

  my $start_time = time();

  ########################################################
  # Select to wait until the socket becomes ready to read
  ########################################################
  my ($inputflags, $errorflags, $iflg, $eflg) = ("") x 4;
  my ($events, $timeleft, $timelimit) = (0, $XB_Defs::CGI_CONNECTION_TIMEOUT,
                                     $XB_Defs::CGI_CONNECTION_TIMEOUT);
  vec ($inputflags, $tcph->fileno, 1) = 1;
  $errorflags = $inputflags;
  while(1){
    ($events, $timeleft) =
    select ( $iflg = $inputflags, undef, $eflg = $errorflags,
             $timelimit );

    ###########################################################
    # When something (either the socket or the timer goes off
    ###########################################################
    if (vec ($iflg, $tcph->fileno, 1))
    {
      ########################################################
      # Make the read call non-blocking so we can avoid getting
      # blocked if the other end crashes
      ########################################################
      fcntl($tcph, F_SETFL, O_NDELAY);
      my ($buf, $byte_read);
      # do sysread until it returns either 0 (EOF) or undef (error)
      if(defined($byte_read = sysread($tcph, $buf, 1000))){
        unless($byte_read == 0){ $return_string .= $buf; }
      }
      if((defined $return_string)&&($return_string =~ /\bXboneEOC\b/)){last;}
      # check for error condition of sysread
      if(not defined ($byte_read)){
        if(defined $!){
	  if($! == EWOULDBLOCK){
	    undef $!;	# clean up error flag for the next round
	    next;	# since we loop back to select, no need to sleep
	  }elsif($! != 0){
	    # some other error
            my $errno = $! + 0; # Force numeric context
            $return_string .= "Xbone Version=$XB_Defs::XBONE_PROTOCOL\n".
	                      "XboneReturn Msg=\"Error: $errno:  $!. ".
			      "This is wierd, I don't expect anything except".
			      " EWOULDBLOCK here.\"\nXboneEOC\n";
	    last;
          }else{
	    # $!=0 => EOF
	    last;
	  }  
	}else{
	  # undefined $! => EOF
	  last;
	}  
      }elsif($byte_read == 0){
        # byte_read = 0 => EOF
	last;
      }
      my $current_time = time();
      if($current_time > $start_time + $XB_Defs::CGI_CONNECTION_TIMEOUT){
	 return "Xbone Version=$XB_Defs::XBONE_PROTOCOL\nXboneReturn ".
		"Msg=\"Error: Non-blocking read from Overlay Manager timed".
		" out.\"\nXboneEOC\n" ;
      }
      #while (1) {
      #    my ($buf);
      #    $tcph->recv($buf, 1000, 0);
      #    if ($buf) {
      #        $return_string .= $buf;
      #        if ($buf =~ /\bXboneEOC\b/) { last;};
      #    } else {
      #        my $current_time = time();
      #        if ($current_time >
      #                $start_time + $XB_Defs::CGI_CONNECTION_TIMEOUT) {
      #                return "Xbone Version=$XB_Defs::XBONE_PROTOCOL\nXboneReturn Msg=\"Error: Non-blocking read from Overlay Manager timed out.\"\nXboneEOC\n";
      #            last;
      #        } else {
      #            if ((not defined($!)) or ($! == 0)) {
      #                # No error, but null return. This is EOF
      #                last;
      #            } elsif ($! == EWOULDBLOCK ) {
      #                undef $!; # so that next time around unless there is
      #                          # a new error we get the a clean slate
      #                sleep(1);
      #                next;
      #            } else {
      #                # Some other error
      #                my $errno = $! + 0; # Force numeric context
      #                $return_string .= "Xbone Version=$XB_Defs::XBONE_PROTOCOL\nXboneReturn Msg=\"Error: $errno:  $!. This is wierd, I don't expect anything except EWOULDBLOCK here.\"\nXboneEOC\n";
      #                last;
      #            }
      #        }
      #    }
      #}
    } elsif (vec ($eflg, $tcph->fileno, 1)) {
        return "Xbone Version=$XB_Defs::XBONE_PROTOCOL\nXboneReturn Msg=\"Error: $XB_proc_name: $subrname(): I/O error on TCP socket. $!.\"\nXboneEOC\n";
    } else {
        # Must be due to time limit.
        return "Xbone Version=$XB_Defs::XBONE_PROTOCOL\nXboneReturn Msg=\"Error: Select to overlay manager timed out.\"\nXboneEOC\n";
    }
  } #while(1)

  $tcph->close();
  if ($return_string eq "") {
      $return_string = "Xbone Version=$XB_Defs::XBONE_PROTOCOL\nXboneReturn Msg=\"Error: Null message from Overlay Manager. Looks like the Overlay Manager crashed\"\nXboneEOC\n";
  }
  return $return_string;
}  



sub hash ($) {
  my %href;
  # use Text::ParseWords to surgically disect the string, taking care of quotes
  foreach my $pair (parse_line '\s+', 0, shift) {
    if(defined $pair) {
      my ($key, $value) = split "=", $pair, 2;
      if(defined $key) { $href{$key} = $value; }
    }
  }
  return %href;
}
    

sub create_teardown_section (@) {
  my @lines = @_;
  while(my $line = shift @lines) {
    if($line =~ /\bXboneReturn\s+(.*)/) { 
      my %h = hash $1;
      #if(exists $h{Msg}) { $h{Msg} =~ s/\"(.*)\"/$1/; }
      return p($h{Msg}); 
    }
  }
}


sub create_creation_section (@) {
  my @lines = @_;
  my %ins;
  my %outs;
  my $otbl = Tr(th({-colspan => 2}, "Overlay Parameters"));
  while(my $line = shift @lines) {
    if($line =~ /\bXboneOverlay\s(.*)/) {
      my %h = hash $1;
      $otbl .= Tr(th("Name"), td($h{Name})) . 
	Tr(th("Search Radius"), td($h{SearchRadius}));
    } elsif($line =~ /\bXboneNetwork\s+(.*)/) {
      my %h = hash $1;
      $otbl .= Tr(th("Topology"), 
		  td($h{Topology}, img({-align => "absmiddle", 
					-src => "/images/$h{Topology}.gif"})));
    } elsif($line =~ /\bXboneCreator\s+(.*)/) {
      my %h = hash $1;
      $otbl .= Tr(th("Creator"), td($h{Name}, br, $h{Email}));
    } elsif($line =~ /\bXboneSecurity\s+(.*)/) {
      my %h = hash $1;
      $otbl .= Tr(th("Authentication"), td("$h{UserId} ($h{AuthType})"));
    } elsif($line =~ /\bXboneNode\s+(.*)/) {
      my %h = hash $1;
      my $row = Tr(td({-class => "topcenter"},
		      [$h{Class}, $h{Status}, $h{InterfaceCount}, 
		       $h{ID}.br.$h{IPaddr}, $XB_Defs::NODEOS_LABEL{$h{OS}}]));
      if($h{Status} eq "in") { $ins{$h{ID}} = $row; }
      else { $outs{$h{ID}} = $row; }
    } elsif($line =~ /\bXboneEOC\b/) { 
      last;
    } else {
      $otbl .= Tr(th("UNKNOWN LINE"), td($line));
    }
  }
  my $hdr = Tr(th(["Role", "Status", "Links", "Host", "Operating System"]));
  my $in;
  foreach my $node (keys %ins) { $in .= $ins{$node}; }
  my $out;
  foreach my $node (keys %outs) { $out .= $outs{$node}; }
  return table($otbl, Tr(th("Overlay Nodes"), td(table($hdr, $in))), 
	       Tr(th("Skipped Candidates"), td(table($hdr, $out))));
}


sub create_monitoring_section (@) {
  my @lines = @_;
  my $current_node;
  my %tunnels;
  my %tunnel_count;
  my %node_info;
  my $requestor;
  my $otbl = Tr(th({-colspan => 2}, "Overlay Parameters"));
  while(my $line = shift @lines) {
    if($line =~ /\bXboneOverlay\s(.*)/) {
      my %h = hash $1;
      $otbl .= Tr(th("Name"), td($h{Name}));
    } elsif($line =~ /\bXboneNetwork\s+(.*)/) {
      my %h = hash $1;
      $otbl .= Tr(th("Topology"), 
		  td($h{Topology}." ", 
		     img({-align => "absmiddle", 
			  -src => "/images/$h{Topology}.gif"})));
    } elsif($line =~ /\bXboneCreator\s+(.*)/) {
      my %h = hash $1;
      $otbl .= Tr(th("Creator"), td($h{Name}, br, $h{Email}));
    } elsif($line =~ /\bXboneSecurity\s+(.*)/) {
      my %h = hash $1;
      $requestor = "$h{UserId} ($h{AuthType})";
    } elsif($line =~ /\bXboneNode\s+(.*)/) {
      my %h = hash $1;
      $current_node = $h{ID};
      # interfacecount is wrong, class will be added below (with image),
      # and status is always in anyway - omot all these
      $node_info{$current_node} .= 
	td({-class => "topcenter"}, [$h{ID}.br.$h{IPaddr}, 
				     $XB_Defs::NODEOS_LABEL{$h{OS}}]);
    } elsif($line =~ /\bXboneTunnel\s+(.*)/) {
      my %h = hash $1;
      $tunnels{$current_node} .= 
	Tr(td({-class => "topcenter"}, 
	      [$h{LocalAddr}, $h{RemoteAddr}, $h{Status}]));
      $tunnel_count{$current_node}++;
    } elsif($line =~ /\bXboneEOC\b/) { 
      last;
    } else {
      $otbl .= Tr(th("UNKNOWN LINE"), td($line));
    }
  }
  my $mtbl = 
    Tr(th(["Role", "Host", "Operating System", "Tunnels"]));
  foreach my $node (keys %node_info) {
    my $role;
    if($tunnel_count{$node} / 2 == 1) {
      # host
      $role = td({-class => "topcenter"}, "Host", br, 
		 img({-src => "/images/h1.gif"}));
    } else {
      #router
      my $links = $tunnel_count{$node} / 2;
      if($links > 8) { $links = 8; }
      $role = td({-class => "topcenter"},"Router", br, 
		 img({-src => "/images/h$links.gif"}));
      
    }
    $mtbl .= Tr($role, $node_info{$node},
		td(table(Tr(th(["Local", "Remote", "Status"])),
			 $tunnels{$node})));
  }
  $otbl .= Tr(th("Nodes"), td(table($mtbl)));
  return p("Overlay monitoring request from $requestor, ",
	   "overlay manager replied:"), table($otbl);
}


sub modify_section ($) {
  my $result = shift;
  my @lines = split /\n/, $result;
  while(my $line = shift @lines) {
    if($line =~ /\bXbone\s+(.*)/) { 
      my %h = hash $1;
      if($h{Version} ne $XB_Defs::XBONE_PROTOCOL) {
	# we don't speak this protocol
	fail_page("Unsupported Protocol Version",
		  p("Received message in X-Bone protocol version " .
		    "$h{Version}, only understand protocol version " . 
		    $XB_Defs::XBONE_PROTOCOL . "."));
      }
    } elsif($line =~ /\bXboneReturn\s+(.*)/) {
      # got an error message (there may be another field "Code", suppress it)
      my %h = hash $1;
      if(exists $h{Msg}) { $h{Msg} =~ s/\"(.*)\"/$1/; }
      fail_page("Command Failed",
		dl(dt(b("The overlay manager encountered the " .
			"following error:" . br)),
		   dd($h{Msg})));
    } elsif($line =~ /\bXboneControl\s(.*)/) {
      # see what this is
      my %h = hash $1;
      if($h{Transaction} eq "Reply") {
	# a reply of some sort, handle
	if($h{Command} eq "CreateOverlay") {
	  return create_creation_section @lines;
	} elsif($h{Command} eq "DestroyOverlay") {
	  return create_teardown_section @lines;
	} elsif($h{Command} eq "OverlayStatus") {
	  return create_monitoring_section @lines;
	} else {
	  # some other command we know nothing about
	  fail_page("Command Failed",
		    dl(dt("Don't know what to do with this kind of command " .
			  "reply:"), dd(pre($h{Command})),
		       dt("Here is the whole message received:"), 
		       dd(pre($result))));
	}
      } else {
	# some other type of message we can't handle
	fail_page("Command Failed",
		  dl(dt("Don't know what to do with this kind of message:"), 
		     dd(pre($h{Transaction})),
		     dt("Here is the whole message received:"), 
		     dd(pre($result))));
      }
    } else {
      # no idea what this is, dump it
      fail_page("Command Failed",
		dl(dt("Found this line in response from the overlay manager" .
		      ", no idea what it means:"), dd(pre($line)),
		   dt("The above line was part of this message:"), 
		   dd(pre($result))));
    }
  }
}

1;
