#! /usr/local/bin/perl5 -w
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
#
# ----------------------------------------------------------------------------
#  $RCSfile: index.pl,v $
#
# $Revision: 1.10 $
#   $Author: touch $
#     $Date: 2000/10/24 23:07:47 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Author: Lars Eggert

use strict;
use sigtrap;

my $arch;
BEGIN {
  use Config;
  my $osvers = $Config{osvers};
  $osvers =~ s/(\d+\.\d+).*/$1/;
  $arch = "$Config{archname}-$osvers";
};
use lib ("../lib", "/home/xbone/perl/$arch");

use CGI qw(:standard :html3);
use CGI::Carp qw(fatalsToBrowser);
use XB_Defs;
use XB_Common;

my %om = ( Release=> "$XB_Defs::XBONE_RELEASE, " .
	   $XB_Defs::XBONE_VERSION_DATE,
	   Protocol => $XB_Defs::XBONE_PROTOCOL,
	   "DNS Domain" => $XB_Defs::XBONE_NET);

my $om_rows = "";
while(my ($key, $val) = each %om) { $om_rows .= Tr(th($key), td($val)); }

# notify user if the GUI runs from a different DNS name than the OM
my $warn = "";
if(virtual_host ne $XB_Defs::XBONE_OVERLAY_MANAGER) {
  $warn = span({-class => "redbg"}, 
               p("Warning: The overlay manager this GUI is talking to is ".
                 "configured as $XB_Defs::XBONE_OVERLAY_MANAGER, while ".
		 "the GUI runs from " . virtual_host . "!"));
}
  
my $title = "X-Bone Control Page";
print header, 
  start_html(-title => $title, -style => { -src =>"/xbone.css" }),
  h1($title), user_info, $warn,
  p("This overlay manager ($XB_Defs::XBONE_OVERLAY_MANAGER) is configured",
    "as follows:"),
  table($om_rows),

  p("This is the main control page for",
    a({-href => "http://www.isi.edu/x-bone/"}, "the X-Bone."), br, 
    "Please choose an overlay operation from the following options, or",
    "go to the ",
    a({-href => "http://" . virtual_host . "/"},
      "certification authority (CA) page.")),
  
  ul(li(a({-href => "/cgi-bin/xb-create.pl"}, "Overlay Creation")),
     li(a({-href => "/cgi-bin/xb-status.pl"}, "Overlay Status Monitoring")),
     li(a({-href => "/cgi-bin/xb-destroy.pl"}, "Overlay Teardown")),
     li(a({-href => "/cgi-bin/xb-admin.pl"}, "Overlay Administration"))), p,
  
  i("If you encounter any problems, ",
    a({-href => "mailto:$XB_Defs::CA_EMAIL"}, "please contact us.")),
  end_html;
