#!/usr/local/bin/perl5 -w
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
# 
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# ----------------------------------------------------------------------------
#  $RCSfile: xb-create.pl,v $
#
# $Revision: 1.35 $
#   $Author: touch $
#     $Date: 2000/10/24 23:07:47 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Author: Lars Eggert

use strict;
use sigtrap;

my $arch;
BEGIN {
  use Config;
  my $osvers = $Config{osvers};
  $osvers =~ s/(\d+\.\d+).*/$1/;
  $arch = "$Config{archname}-$osvers";
};
use lib ("../lib", "/home/xbone/perl/$arch");

use CGI qw(:standard :html3);
use CGI::Carp qw(fatalsToBrowser);

use XB_Defs;
use XB_Interface;
use XB_Common;

my @os = qw(freebsd linux solaris);

my %aalgl = ( "undef" => "(None)", "md5" => "MD5", "sha1" => "SHA1" );
my %ealgl = ( "undef" => "(None)", "des" => "DES", "3des" => "3DES" );

my %appll = ( none  => "(None)", 
	      anetd => "Active Nets Daemon",
	      squid => "Squid Web Cache" );

my %nnamesl = ( topology  => "Topology-dependent",
		colors    => "Color Names",
		countries => "Country Names",
		planets   => "Planet Names",
		cities    => "City Names" );

my %osl = map { 
    $_ => " " . $XB_Defs::NODEOS_LABEL{$_} 
} @XB_Defs::ALLOWED_NODEOS;

my %topsl = ( star   => "Star", 
	      ring   => "Ring", 
	      linear => "Linear",
#	      stag   => "Staggered",
#	      tree   => "Spanning Tree" 
	    );

my $topt;
my @top = sort { $a cmp $b } keys %topsl;
while($#top > -1) {
  my $row;
  for(my $i = 1; $i <= 3; $i++) {
    my $item = shift @top;
    $row .= td({-class => "bottom"}, 
	       $item ? (img({-src => "/images/$item.gif", -height => 50,
			     -alt => $topsl{$item}, -width => 100}), br,
			$topsl{$item}) : "&nbsp;");
  }
  $topt .= Tr($row);
}

# get xbone dns suffix from defs, make sure it starts with a dot
my $suffix = $XB_Defs::XBONE_NET;
if($suffix !~ /^\./) { $suffix = ".$suffix"; }

my %default = ( name         => "",
		mcast_radius => 5,
		topology     => "star",
		dns          => 1,
		app          => "none",
		hosts        => 1,
		host_os      => "freebsd",
		routers      => 1,
		router_os    => "freebsd",
#		dns_dom      => $suffix,
		ah           => "undef",
#		naming       => "topology",
		esp          => "undef" );

my @missing_params;
my %used_param = map { $_ => 1 } param;
foreach my $v (keys %default) {
  if($v ne "dns" and 
     not exists $used_param{$v}) { push @missing_params, $v; }
}

if($#missing_params == -1) {
  my ($h, $r, $t) = (param("hosts"), param("routers"), param("topology"));
  my $links;

  # check if user selected correct host/router numbers for topology type
  if($t eq "star") {
    # star must have one router
    if($r != 1) {
      fail_page "Invalid Star Topology", 
        p("Must have exactly one router in a star, you specified $r.");
    }
    $links =  $r + $h - 1;
  } elsif($t eq "ring") {
    # ring must have >= 3 routers
    if($r < 3) {
      fail_page "Invalid Ring Topology", 
        p("Must have at least three routers in a ring, you specified " .
	  $r . ".");
    }
    $links = $h + $r;
  } elsif($t eq "linear") {
    # line must have two end hosts
    if($h != 2) {
      fail_page "Invalid Line Topology", 
        p("Must have exactly two end hosts in a line, you specified " .
	  $h . ".");
    }
    $links = $r + $h -1;
  } else {
    # unsupported topology
    fail_page "Unsupported Topology", 
      p("The only topologies currently supported are: ") .
	ul(li([map { $topsl{$_} } qw(star ring linear)])) .
	  p("You picked: ". $topsl{$t});
  }

  # Temporary check for invalid overlay names
  if (param("name") !~ /^[a-z]([a-z0-9\-]*[a-z0-9])?$/i) {
      # This is an invalid DNS name.
      fail_page "Invalid Overlay Name",
      p("The name specified is an illegal DNS name");
  }


  # XXX temporary application-deployment check
  if(param("app") ne "none") {
    fail_page "Application Deployment Error", 
      p("At this time, automatic deployment of \"" . $appll{param("app")} .
	"\" is not supported.");
  }
  
  if(param("ah") ne "undef" or param("esp") ne "undef") {
    # if IPsec is enabled, diddle the OS field to use IPsec-capable OS flavors
    foreach my $p (qw(host_os router_os)) {
      my @newos;
      foreach my $os (param($p)) {
	if($os eq "linux") { push @newos, "nist"; }
	elsif($os eq "freebsd") { push @newos, qw(kame cairn); }
	else {
	  fail_page "Invalid $1 Operating Systems For IPsec", 
	  p("You have chosen IPsec protection of the overlay links, but the ".
	    $XB_Defs::NODEOS_LABEL{$os} . " operating system picked does ".
	    "not support this."); 
	}
      }
      # overwrite os flags
      param(-name => $p, -value => \@newos); 
    }
  } else {
    # if IPsec is not enabled, send a pattern that matches both IPsec
    # and non-IPsec flavors of an OS
    foreach my $p (qw(host_os router_os)) {
      my @newos;
      foreach my $os (param($p)) {
	if($os eq "linux") { push @newos, qw(linux nist); }
	elsif($os eq "freebsd") { push @newos, qw(kame cairn freebsd); }
	elsif($os eq "solaris") { push @newos, qw(solaris); }
	else {
	  fail_page "Invalid $1 Operating System", 
	  p("You have chosen the $XB_Defs::NODEOS_LABEL{$os} operating ",
	    "system, which I know nothing about."); 
	}
      }
      # overwrite os flags
      param(-name => $p, -value => \@newos); 
    }
  }
  
  # Generate the string to pass to the OverlayManager
  # simply param("dns") doesn't work. It puts a blank space and not 0
  # if the checkbox is not selected.  
  my $cmd = "Xbone Version=$XB_Defs::XBONE_PROTOCOL\n" .
    "XboneControl Command=CreateOverlay Transaction=Request\n".
      "XboneOverlay Name=" . param("name") . $suffix . 
	" SearchRadius=" . param("mcast_radius") . " DNS=" . 
	    (param("dns") == 1 ? "1":"0") .
	"\n" . "XboneNetwork Topology=$t IPsecEncryption=" . param("esp") . 
	    " IPsecAuthentication=" . param("ah") .
	      " Application=" . param("app") . "\n" . "XboneCreator Name=\"" . 
		param("c_name") . "\" Email=" . param("c_mail") . "\n";
  if(CGI::https()) { 
    $cmd .= "XboneSecurity UserId=".$ENV{SSL_CLIENT_EMAIL}." AuthType=x509\n";
  } else {
    $cmd .= "XboneSecurity UserId=" . $ENV{REMOTE_ADDR} . " AuthType=ipaddr\n";
  }
  $cmd .= "XboneNode Class=host NodeCount=$h OS=" . 
    (join "|", param("host_os")) . " \nXboneNode Class=router " .
      "NodeCount=$r OS=" . (join "|", param("router_os")) . " \nXboneEOC\n";
  
  ## Call my interface to the overlay manager          
  my $result = XB_Interface::OverlayManagerInterface $cmd;
  my $section = XB_Interface::modify_section($result);

  # make a cookie
  my @cookies;
  foreach my $v (keys %default) {
    push @cookies, cookie(-name => $v, -expires => "+1y", 
			  -value => [param($v)], -path => "/cgi-bin/");
  }
  
  ## Parse the return string and generate HTML
  print header("-cookie" => \@cookies),
  start_html(-title => "X-Bone Overlay Creation: Return Status", 
	     -style => { -src =>"/xbone.css" }
	    ),
  h1("Return Status"),
  p("The following overlay network has been created successfully:"), 
  $section, 
  p("Back to the ", a({-href => "/"}, "main X-Bone page") . "."),
  end_html;
  
} else {
  # init state from 1. CGI param, 2. cookie, 3. defaults
  my @cookies;
  foreach my $v (keys %default) {
    if($v ne "name" and not exists $used_param{$v}) { 
      # we have no CGI param of that name, try to init from cookie
      if(cookie($v)) { param(-name => $v, -value => [cookie($v)]); }
      else { param(-name => $v, -value => [$default{$v}]); }
    }
    # while we're at it, also create a cookie to pass back
    push @cookies, cookie(-name => $v, -expires => "+1y", 
			  -value => [param($v)], -path => "/cgi-bin/");
  }
  # these override any cookie state
  param(-name => "c_name", -value => $ENV{SSL_CLIENT_CN});
  param(-name => "c_mail", -value => $ENV{SSL_CLIENT_EMAIL});

  # the os vars can have IPec flavors of OS in them, reset those to
  # plain names
  foreach my $v (qw(host_os router_os)) {
    my %os;
    foreach my $s (param($v)) {
      if($s =~ /^(kame|cairn|freebsd)$/) { $os{freebsd} = 1; }
      elsif($s =~ /^(linux|nist)$/) { $os{linux} = 1; }
      elsif($s =~ /^(solaris)$/) { $os{solaris} = 1; }
    }
    param(-name => $v, -value => [keys %os]);
  }
  
  print header("-cookie" => \@cookies),
  start_html(-title => "X-Bone Overlay Creation", 
	     -style => { -src =>"/xbone.css" } ),
  h1("X-Bone Overlay Creation"), user_info, #CGI::dump(), p("@missing_params"),
  p("This page allows you to create a new overlay. Please fill out " .
    span({-class => "redbg"}, "all remaining red fields") . "."),
    start_form(-method => "POST", -action => "/cgi-bin/xb-create.pl"),
    table(Tr(th({-colspan => 3}, "Overlay Creator")),
	  Tr(my_th("c_name", "Creator Name"), 
	     td(textfield(-name => "c_name")),
	     td(br."Your name, taken from your X.509 user certificate. (For " .
		"information only, changes made have no effect.)".
		br."&nbsp;")),
	  
	  Tr(my_th("c_mail", "Creator E-Mail"), 
	     td(textfield(-name => "c_mail")),
	     td(br."Your e-mail address, taken from your X.509 user " .
		"certificate. (For information only, changes made have " .
		"no effect.)".br."&nbsp;")),
	  Tr(td(br, br)),
	  
	  Tr(th({-colspan => 3}, "Overlay-Wide Properties")),
	  Tr(my_th("name", "Name"), 
	     td(textfield(-name => "name")), 
	     td(br."Name of the new overlay. ".
		"Suffix $suffix will be added automatically.".
		"If \"use DNS\" is checked below, ".
		"the overlay name will also become part of the DNS names ".
		"of your overlay nodes."
	       .br."&nbsp;")),
	  
	  Tr(th("DNS"), 
	     td(checkbox(-name => "dns", -label => " use DNS", -value => 1))),
#		popup_menu(-name => "dns_dom", -value => [$suffix])),
#	     td("If you check \"use ".
#		"DNS\", the overlay manager will assign DNS names in the ".
#		"selected domain to the nodes of the new overlay. ".
#		"If unchecked, no DNS entries are ".
#		"created, and you will need to use IP ".
#		"addresses directly to reach overlay nodes.",
#		p(#{-class => "redbg"}, 
#		  i("The \"use DNS\" switch is currently unsupported.")))),
  
#	  Tr(th("Node Naming"),
#	     td(popup_menu(-name => "naming", -value => [keys %nnamesl],
#			   -labels => \%nnamesl)),
#	     td("If \"use DNS\" is checked above, nodes in the new " .
#		"overlay will be assigned names from the chosen set of ".
#		"name candidates. If \"use DNS\" is unchecked, this ".
#		"option has no effect.", 
#		p(#{-class => "redbg"},   
#		  i("Currently unsupported, selection has no effect.")))),  
	  
	  Tr(my_th("mcast_radius", "Search Radius"), 
	     td(textfield(-name => "mcast_radius")),
	     td(br."Multicast search radius limiting the region in which " .
		"the overlay manager " .
		"will look for X-Bone hosts willing to participate in " .
		"setting up the new overlay.".br."&nbsp;")),
	  
	  Tr(th("Topology"),
	     td(popup_menu(-name => "topology", -value => [keys %topsl], 
			   -labels => \%topsl)),
	     td(p(br."These topologies are available for new overlays:",
		table($topt) #,
#		p(#{-class => "redbg"},    
#		  i("\"Staggered\" and \"Spanning Tree\" topologies ".
#		  "are currently unsupported."))
	       .br."&nbsp;"))),
	  
	  Tr(th("Application"),
	     td(popup_menu(-name => "app", -value => [keys %appll],
			   -labels => \%appll)),
	     td(br."Automatically deploy and start an application after the " .
		"overlay has been set up.".br."&nbsp;")),
	  Tr(td(br, br)),
	  
	  Tr(th({-colspan => 3}, "Host Properties")),
	  Tr(my_th("hosts", "Number of Hosts"), 
	     td(textfield(-name => "hosts")),
	     td(br."Number of hosts in the overlay. (Hosts are overlay " .
		"nodes that do not route packets.)".br."&nbsp;")),
	  
	  Tr(my_th("host_os", "Host Operating System"), 
	     td(checkbox_group(-name => "host_os", -linebreak => 1,
			       -labels => \%osl, -value => \@os)),
	     td(br."Operating system requirements for the hosts. Only ". 
		"hosts of the checked operating systems will be picked ". 
		"for the new overlay.".br."&nbsp;")), 
	  Tr(td(br, br)),
	  
	  Tr(th({-colspan => 3}, "Router Properties")),
	  Tr(my_th("routers", "Number of Routers"), 
	     td(textfield(-name => "routers")),
	     td(br."Number of routers in the overlay. (Routers are overlay " .
		"nodes that route packets.)".br."&nbsp;")),
	  
	  Tr(my_th("router_os", "Router Operating System"), 
	     td(checkbox_group(-name => "router_os", -linebreak => 1,
			       -labels => \%osl, -value => \@os)),
	     td(br."Operating system requirements for the routers. Only ".
		"routers of the checked operating systems will be picked ".
		"for the new overlay.".br."&nbsp;")),  
	  Tr(td(br, br)),
	  
	  Tr(th({-colspan => 3}, "Link Properties")),
	  Tr(th("Authentication"), 
	     td(popup_menu(-name => "ah", -value => [keys %aalgl], 
			   -labels => \%aalgl)),
	     td(br."IPsec authentication algorithm used to authenticate " .
		"all overlay traffic.".br."&nbsp;")),
	  
	  Tr(th("Encryption"), 
	     td(popup_menu(-name => "esp", -value => [keys %ealgl], 
			   -labels => \%ealgl)),
	     td(br."IPsec encryption algorithm used to encrypt " .
		"all overlay traffic.".br."&nbsp;"))), p,
	       
	       submit({-class => "black", -value => "Create This Overlay"}), 
	       " ", CGI::reset({-class => "black"}, "Undo Changes"),
	       p({-class => "small"}, 
		 "A cookie with the above information will be created.", br, 
		 "It will be used to initialize the form when you revisit ".
		 "this page."), end_form,
		   p("Back to the ", 
		     a({-href => "/"}, "main X-Bone page") . "."),
		   end_html;
}









