#!/usr/local/bin/perl5 -w
#
# ----------------------------------------------------------------------------
#				    X-BONE
#
#		 USC Information Sciences Institute (USC/ISI)
#		    Marina del Rey, California 90292, USA
#			   Copyright (c) 1998-2000
# ----------------------------------------------------------------------------
#
# Copyright (c) 1998-2000 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
#
# -------------------------------------------------------------------------
#
# Effort sponsored by the Defense Advanced Research Projects Agency (DARPA)
# and Air Force Research Laboratory, Air Force Materiel Command, USAF, under
# agreement number F30602-98-1-0200. The U.S. Government is authorized to 
# reproduce and distribute reprints for Governmental purposes not withstanding
# any copyright annotation therein.
# 
# The views and conclusions contained herein are those of the authors and
# should not be interpreted as necessarily representing the official policies
# or endorsements, either expressed or implied, of the Defense Advanced
# Research Projects Agency (DARPA), the Air Force Research Laboratory, or the
# U.S. Government.
# 
# ----------------------------------------------------------------------------
#  $RCSfile: xb-destroy.pl,v $
#
# $Revision: 1.17 $
#   $Author: touch $
#     $Date: 2000/10/24 17:28:42 $
#    $State: Exp $
# ----------------------------------------------------------------------------
#
# Primary Author: Lars Eggert

use strict;
use sigtrap;

my $arch;
BEGIN {
  use Config;
  my $osvers = $Config{osvers};
  $osvers =~ s/(\d+\.\d+).*/$1/;
  $arch = "$Config{archname}-$osvers";
};
use lib ("../lib", "/home/xbone/perl/$arch");

use CGI qw(:standard :html3);
use CGI::Carp qw(fatalsToBrowser);

use XB_Defs;
use XB_Common;
use XB_Interface;


my $cmd = "Xbone Version=$XB_Defs::XBONE_PROTOCOL\n";
if(defined param("XboneOverlayName")) {
  $cmd .= "XboneControl Command=DestroyOverlay Transaction=Request\n" .
    "XboneOverlay Name=" . param("XboneOverlayName") . "\n";
} else {
  $cmd .= "XboneControl Command=ListOverlays Transaction=Request\n";
}
if(CGI::https()) {
  $cmd .= "XboneSecurity UserId=" . $ENV{SSL_CLIENT_EMAIL} . 
    " AuthType=x509\n";
} else {
  $cmd .= "XboneSecurity UserId=" . $ENV{REMOTE_ADDR} .
    " AuthType=ipaddr\n";
}
$cmd .= "XboneEOC\n";

## Call my interface to the overlay manager          
my $result = XB_Interface::OverlayManagerInterface $cmd;


my $title = "X-Bone Overlay Teardown";
print header(),
  start_html(-title => $title, -style => { -src =>"/xbone.css" }),
  h1($title);

if(defined param("XboneOverlayName")) {
  my $section = XB_Interface::modify_section($result);
  print $section, p("Back to the ", a({-href => "/"}, "main X-Bone page") .
		    "."), end_html;
} else {

  my @overlays;
  foreach my $l (split /\n/, $result) {
    if($l =~ /^XboneOverlay Name=(\w\S+)/) { push @overlays, $1; }
  }
  my $menu;
  if($#overlays == -1) { $menu = td("No active overlays found."); }
  else {
    $menu = td(popup_menu(-name => "XboneOverlayName", -value => \@overlays)) .
      td(submit("Delete This Overlay"));
  }
  
  print user_info, p("Delete which overlay?"),
  start_form(-method => "POST", -action => "/cgi-bin/xb-destroy.pl"),
  table(Tr(th("Pick Overlay"), $menu)), end_form,
  p("Back to the ", a({-href => "/"}, "main X-Bone page") . "."), end_html;
}
